@@ -1,76 +1,79 @@
+/*! \file aic.c */
 /*------------------------------------------------------
 Maximum likelihood estimation
 of migration rate  and effectice population size
 using a Metropolis-Hastings Monte Carlo algorithm
 -------------------------------------------------------
- AIC model test   R O U T I N E S
- 
- Peter Beerli 2001, Seattle
- beerli@csit.fsu.edu
- 
- Copyright 2001-2002 Peter Beerli and Joseph Felsenstein
- 
- This software is distributed free of charge for non-commercial use
- and is copyrighted. Of course, we do not guarantee that the software
- works and are not responsible for any damage you may cause or have.
- 
-$Id: aic.c,v 1.20 2003/11/04 16:12:07 beerli Exp $ 
+	AIC model test   R O U T I N E S
+
+	Peter Beerli 2001, Seattle
+	beerli@csit.fsu.edu
+
+	Copyright 2001-2002 Peter Beerli and Joseph Felsenstein
+
+	This software is distributed free of charge for non-commercial use
+	and is copyrighted. Of course, we do not guarantee that the software
+	works and are not responsible for any damage you may cause or have.
+
+$Id: aic.c,v 1.25 2004/08/28 19:59:29 beerli Exp $ 
 -------------------------------------------------------*/
+
 #include "migration.h"
 #include "tools.h"
 #include "broyden.h"
 #include "combroyden.h"
 #include "options.h"
-#include "sort.h"
+#include "migrate_mpi.h"
 #include "aic.h"
+#include "sighandler.h"
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
 
 
-void print_progressheader_aic(boolean progress, FILE *file, double mleaic, double numparam);
+void print_progressheader_aic(boolean progress, FILE *file, MYREAL mleaic, MYREAL numparam);
 
 void aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
-                long zero, long which, char *temppattern, double *param0,
+                long zero, long which, char *temppattern, MYREAL *param0,
                 char migtype);
 
 boolean legal_pattern (char *matrix, long numpop);
 
 void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                      long zero, long which, char *temppattern,
-                     double *param0, char migtype);
+                     MYREAL *param0, char migtype);
 
 void add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long zero, long m,
-                  double *param0, char *temppattern, char *custm2,
+                  MYREAL *param0, char *temppattern, char *custm2,
                   long *aicnum, aic_fmt ** aicvec, nr_fmt * nr);
 
-void add_aicvec(double aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum);
+void add_aicvec(MYREAL aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum);
 
-void aic_print_driver(aic_fmt *aicvec, double mleaic, long aicnum, nr_fmt *nr, world_fmt *world);
+void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, world_fmt *world);
 
-void aic_score_driver(aic_struct *aic, double mle, double mleaic, nr_fmt *nr, world_fmt *world, char *temppattern);
+void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, world_fmt *world, char *temppattern);
 
 void akaike_information (world_fmt * world, long *Gmax);
 
-void print_header_aic (nr_fmt * nr, double mleaic);
+void print_header_aic (nr_fmt * nr, MYREAL mleaic);
 
 void print_aicfile(aic_fmt **aicvec, long *aicnum, nr_fmt *nr);
 
 void print_progress_aic(boolean add
-                            , aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long freeparam);
+						, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long freeparam);
 
 long find_paramnum(world_fmt *world, char *connect);
 
-// changes a linear matrix of ddd mm mm mm
-// to a diagonal matrix dmm mdm mmd
-// it destroys pattern
-char *
-reshuffle (char *pattern, char *origpattern, long numpop)
+///
+/// changes a linear matrix of ddd mm mm mm
+/// to a diagonal matrix dmm mdm mmd
+/// it destroys pattern
+char * reshuffle (char *pattern, char *origpattern, long numpop)
 {
     long space = 0;
     long i, j, z = numpop;
-
+	
     for (i = 0; i < numpop; i++)
     {
         for (j = 0; j < numpop; j++)
@@ -86,9 +89,8 @@ reshuffle (char *pattern, char *origpattern, long numpop)
     return pattern;
 }
 
-/* run time reporter */
-void
-print_progressheader_aic(boolean progress, FILE *file, double mleaic, double numparam)
+/// print the progress header for AIC based model selection
+void print_progressheader_aic(boolean progress, FILE *file, MYREAL mleaic, MYREAL numparam)
 {
     if (progress)
     {
@@ -101,43 +103,45 @@ print_progressheader_aic(boolean progress, FILE *file, double mleaic, double num
     }
 }
 
-/* prints the header of the AIC table
-     and runtime information
-*/
+///
+/// print the header for the AIC based model selection
+/// the routine reports also progress to screen and logfile
 void
-print_header_aic (nr_fmt * nr, double mleaic)
+print_header_aic (nr_fmt * nr, MYREAL mleaic)
 {
     world_fmt *world = nr->world;
-
+	
     print_progressheader_aic(world->options->progress, stdout,mleaic, world->options->aicmod*world->numpop2);
     print_progressheader_aic(world->options->writelog, stdout,mleaic, world->options->aicmod*world->numpop2);
-
+	
     FPRINTF (world->outfile, "\n\n\n Akaike's Information Criterion  (AIC)\n");
     FPRINTF (world->outfile, "=========================================\n\n");
     FPRINTF (world->outfile, "[Linearized migration matrix, x=diagonal]\n");
 }
 
-// calculates akaike information score
-void aic_score_driver(aic_struct *aic, double mle, double mleaic, nr_fmt *nr, world_fmt *world, char *temppattern)
+///
+/// Calculates AIC score
+/// \callgraph
+void aic_score_driver(aic_struct *aic, MYREAL mle, MYREAL mleaic, nr_fmt *nr, world_fmt *world, char *temppattern)
 {
     char * savecustm;
     char *savecustm2;
-
-    savecustm = (char *) calloc (world->numpop2, sizeof (char));
-    savecustm2 = (char *) calloc (world->numpop2, sizeof (char));
-
+	
+    savecustm = (char *) mycalloc (2 * world->numpop2, sizeof (char));
+    savecustm2 =  savecustm + world->numpop2;
+	
     memcpy (savecustm, world->options->custm2, sizeof (char) * nr->numpop2);
     memcpy (savecustm2, world->options->custm, sizeof (char) * nr->numpop2);
-
+	
     aic->aicnum = 1;
-    aic->aicvec = (aic_fmt *) calloc (aic->aicnum, sizeof (aic_fmt));
+    aic->aicvec = (aic_fmt *) mycalloc (aic->aicnum, sizeof (aic_fmt));
     aic->aicvec[0].mle = mle;
     aic->aicvec[0].aic = mleaic;
     aic->aicvec[0].lrt = 0.0;
     aic->aicvec[0].prob = 1.0;
     aic->aicvec[0].probcorr = 1.0;
     aic->aicvec[0].numparam = nr->partsize;
-    aic->aicvec[0].pattern = (char *) calloc (nr->partsize + 1, sizeof (char));
+    aic->aicvec[0].pattern = (char *) mycalloc (nr->partsize + 1, sizeof (char));
     memcpy (aic->aicvec[0].pattern, world->options->custm2, sizeof (char) * nr->partsize);
     if (world->options->fast_aic)
     {
@@ -166,22 +170,21 @@ void aic_score_driver(aic_struct *aic, double mle, double mleaic, nr_fmt *nr, wo
     }
     memcpy (world->options->custm2, savecustm, sizeof (char) * nr->numpop2);
     memcpy (world->options->custm, savecustm2, sizeof (char) * nr->numpop2);
-    free(savecustm);
-    free(savecustm2);
+    free(savecustm); // savecustom2 is also freed, because it's part of savecustm
 }
 
-
-// print aic results, assume they are already ordered by qsort()
-// printing is:
-// pattern {migration matrix}
-// aic value, number of parameters, MLE of pattern, Prob of standard LRT, Prob of weighted chisquare
-void aic_print_driver(aic_fmt *aicvec, double mleaic, long aicnum, nr_fmt *nr, world_fmt *world)
+///
+/// print aic results, assume they are already ordered by qsort()
+/// printing is:
+/// pattern {migration matrix}
+/// aic value, number of parameters, MLE of pattern, Prob of standard LRT, Prob of weighted chisquare
+void aic_print_driver(aic_fmt *aicvec, MYREAL mleaic, long aicnum, nr_fmt *nr, world_fmt *world)
 {
     long i;
     char *temppattern;
     boolean mldone=FALSE;
     char dashes[101] = "----------------------------------------------------------------------------------------------------";
-    temppattern = (char *) calloc (world->numpop2 + 1 + world->numpop, sizeof (char));
+    temppattern =  (char *) mycalloc (world->numpop2 + 1 + world->numpop, sizeof (char));
     FPRINTF (world->outfile,
              "%-*.*s  AIC     #param   Log(L)   LRT        Prob    Probc\n",
              (int) MAX (18, nr->partsize + nr->numpop),
@@ -205,15 +208,17 @@ void aic_print_driver(aic_fmt *aicvec, double mleaic, long aicnum, nr_fmt *nr, w
             FPRINTF (world->outfile, "%-*.*s %53.53s\n",
                      (int) MAX (18, nr->partsize + nr->numpop),
                      (int) (nr->partsize + nr->numpop),dashes,
-                  dashes);
+					 dashes);
         }
         free (aicvec[i].pattern);
     }
+    FPRINTF (world->outfile, "\n\n\n");
     free(temppattern);
 }
 
-// this drives the aic calculation and is called from main.c
-//
+///
+/// this drives the aic calculation and is called from main.c
+/// \callgraph
 void
 akaike_information (world_fmt * world, long *Gmax)
 {
@@ -222,21 +227,21 @@ akaike_information (world_fmt * world, long *Gmax)
     long kind = world->loci > 1 ? MULTILOCUS : SINGLELOCUS;
     long repstop;
     long repstart;
-
-    double mleaic;
-    double mle;
+	
+    MYREAL mleaic;
+    MYREAL mle;
     boolean multilocus;
-
+	
     char *temppattern;
-
+	
     prepare_broyden (kind, world, &multilocus);
     world->options->migration_model = MATRIX_ARBITRARY;
-
-    temppattern = (char *) calloc (world->numpop2 + 1 + world->numpop, sizeof (char));
-    aic.param0 = (double *) calloc (world->numpop2 + 1, sizeof (double));
-
+	
+    temppattern =  (char *) mycalloc (world->numpop2 + 1 + world->numpop, sizeof (char));
+    aic.param0 = (MYREAL *) mycalloc (world->numpop2 + 1, sizeof (MYREAL));
+	
     set_replicates (world, world->repkind, world->rep, &repstart, &repstop);
-
+	
     if (kind == MULTILOCUS)
     {
         mle = world->atl[0][world->loci].param_like;
@@ -247,26 +252,26 @@ akaike_information (world_fmt * world, long *Gmax)
         mle = world->atl[repstop == 1 ? 0 : repstop][0].param_like;
         mleaic = -2. * mle + 2. * find_paramnum(world,NULL);
     }
-    nr = (nr_fmt *) calloc (1, sizeof (nr_fmt));
-
+    nr = (nr_fmt *) mycalloc (1, sizeof (nr_fmt));
+	
     create_nr (nr, world, *Gmax, 0, world->loci, world->repkind, world->rep);
-
+	
     SETUPPARAM0 (world, nr, world->repkind, repstart, repstop,
                  world->loci, kind, multilocus);
-
+	
     print_header_aic (nr, mleaic);
-
+	
     if (kind == MULTILOCUS)
-        memcpy (aic.param0, nr->world->atl[0][nr->world->loci].param, sizeof (double) * nr->numpop2);
+        memcpy (aic.param0, nr->world->atl[0][nr->world->loci].param, sizeof (MYREAL) * nr->numpop2);
     else
-        memcpy (aic.param0, nr->world->atl[repstop ==1 ? 0 : repstop][nr->world->locus].param, sizeof (double) * nr->numpop2);
-
+        memcpy (aic.param0, nr->world->atl[repstop ==1 ? 0 : repstop][nr->world->locus].param, sizeof (MYREAL) * nr->numpop2);
+	
     aic_score_driver(&aic, mle, mleaic, nr, world, temppattern);
-
+	
     qsort ((void *) aic.aicvec, aic.aicnum, sizeof (aic_fmt), aiccmp);
-
+	
     aic_print_driver(aic.aicvec, mleaic, aic.aicnum, nr, world);
-
+	
     free (aic.aicvec);
     fflush (world->outfile);
     free (aic.param0);
@@ -274,19 +279,19 @@ akaike_information (world_fmt * world, long *Gmax)
     destroy_nr (nr, world);
 }
 
-// does check all enumeration on one level and then picks best
-// this is different to the "branch-bound" algorithm
-//
-void
-fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
+///
+/// Does check all enumeration on one level and then picks only the best model to continue to
+/// the next level
+/// [this is different to the "branch-bound" algorithm]
+void fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
                 long zero, long which, char *temppattern,
-                double *param0, char migtype)
+                MYREAL *param0, char migtype)
 {
     long m, ii;
-    double likes = 0;
-    double normd = 0;
-    double aic;
-    double borderaic = (*aicvec)[0].aic + nr->world->options->aicmod * nr->numpop2;
+    MYREAL likes = 0;
+    MYREAL normd = 0;
+    MYREAL aic;
+    MYREAL borderaic = (*aicvec)[0].aic + nr->world->options->aicmod * nr->numpop2;
     char savecustm2;
     long remainnum = 0;
     boolean legal;
@@ -295,14 +300,14 @@ fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     long numparam;
     long freeparam;
     aic_fmt *best;
-
-    scustm2 = (char *) calloc (nr->partsize, sizeof (char));
+	
+    scustm2 = (char *) mycalloc (nr->partsize, sizeof (char));
     memcpy (scustm2, custm2, sizeof (char) * nr->partsize);
     if (migtype == 'm')
         remainnum = 1;
     numparam = zero;
     freeparam = (nr->numpop2 - numparam - 1 + remainnum);
-    best = (aic_fmt *) calloc (nr->partsize, sizeof (aic_fmt));
+    best = (aic_fmt *) mycalloc (nr->partsize, sizeof (aic_fmt));
     for (m = nr->numpop; m < nr->numpop2; m++)
     {
         best[m].aic = DBL_MAX;
@@ -311,7 +316,7 @@ fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
             continue;
         savecustm2 = custm2[m];
         custm2[m] = migtype;
-        memcpy (nr->world->param0, param0, sizeof (double) * nr->numpop2);
+        memcpy (nr->world->param0, param0, sizeof (MYREAL) * nr->numpop2);
         resynchronize_param (nr->world);
         if ((legal = legal_pattern (nr->world->options->custm2, nr->numpop)))
         {
@@ -347,24 +352,26 @@ fast_aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     free (best);
 }
 
-void add_aicvec(double aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum)
+/// add model and AIC score to the list of models for the final printing
+void add_aicvec(MYREAL aic, aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long remainnum)
 {
-    double lrt = -2. * (nr->llike - (*aicvec)[0].mle);
-
+    MYREAL lrt = -2. * (nr->llike - (*aicvec)[0].mle);
+	
     *aicvec = (aic_fmt *)
-              realloc (*aicvec, sizeof (aic_fmt) * (*aicnum + 1));
+		 myrealloc (*aicvec, sizeof (aic_fmt) * (*aicnum + 1));
     (*aicvec)[*aicnum].pattern = (char *)
-                                 calloc (nr->partsize + 1, sizeof (char));
+		mycalloc (nr->partsize + 1, sizeof (char));
     (*aicvec)[*aicnum].aic = aic;
     (*aicvec)[*aicnum].mle = nr->llike;
     (*aicvec)[*aicnum].numparam = nr->numpop2 - numparam - 1 + remainnum;
     memcpy ((*aicvec)[*aicnum].pattern, nr->world->options->custm2, sizeof (char) * nr->partsize);
-
+	
     (*aicvec)[*aicnum].lrt = lrt;
     (*aicvec)[*aicnum].prob = probchi (numparam, (*aicvec)[*aicnum].lrt);
     (*aicvec)[*aicnum].probcorr = probchiboundary ((*aicvec)[*aicnum].lrt, numparam, numparam);
 }
 
+/// print AIC model selection results
 void
 print_aicfile(aic_fmt **aicvec, long *aicnum, nr_fmt *nr)
 {
@@ -376,23 +383,24 @@ print_aicfile(aic_fmt **aicvec, long *aicnum, nr_fmt *nr)
                  (*aicvec)[*aicnum].numparam,
                  (*aicvec)[*aicnum].prob,
                  (*aicvec)[*aicnum].probcorr);
-
+		
         for (i = 0; i < nr->partsize; i++)
             FPRINTF (nr->world->options->aicfile, "%f ", nr->world->param0[i]);
         FPRINTF (nr->world->options->aicfile, "\n");
     }
 }
 
+/// print progress report for AIC model selection
 void
 print_progress_aic(boolean add
-                       , aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long freeparam)
+				   , aic_fmt **aicvec, long *aicnum, nr_fmt *nr, long numparam, long freeparam)
 {
-    double mle, aic, lrt, prob, probcorr;
+    MYREAL mle, aic, lrt, prob, probcorr;
     char *temppattern;
     char *custm2 = nr->world->options->custm2;
-    temppattern = (char *) calloc (nr->world->numpop2 + 1 + nr->world->numpop, sizeof (char));
+    temppattern =  (char *) mycalloc (nr->world->numpop2 + 1 + nr->world->numpop, sizeof (char));
     if(add
-      )
+	   )
     {
         if (nr->world->options->progress)
             FPRINTF (stdout,
@@ -435,13 +443,14 @@ print_progress_aic(boolean add
     free(temppattern);
 }
 
+/// add the AIC models and scores to the result list
 void
 add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long zero, long m,
-             double *param0, char *temppattern, char *custm2,
+             MYREAL *param0, char *temppattern, char *custm2,
              long *aicnum, aic_fmt ** aicvec, nr_fmt * nr)
 {
-    double aic = -2. * nr->llike + 2. * freeparam;
-
+    MYREAL aic = -2. * nr->llike + 2. * freeparam;
+	
     if (aic < (*aicvec)[0].aic + nr->world->options->aicmod * freeparam)
     {
         if (migtype != 'm' || (migtype == 'm' && nr->world->options->mmn > 1))
@@ -460,48 +469,48 @@ add_aiclist (char migtype, long numparam, long freeparam, long remainnum, long z
     }
 }
 
-
+/// calculates AIC scores
 void
 aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
-           long zero, long which, char *temppattern, double *param0,
+           long zero, long which, char *temppattern, MYREAL *param0,
            char migtype)
 {
     long m;
-    double likes = 0;
-    double normd = 0;
+    MYREAL likes = 0;
+    MYREAL normd = 0;
     char savecustm2;
     long remainnum = 0;
     boolean legal;
     char *custm2 = nr->world->options->custm2;
     long numparam = 0;
     long freeparam;
-
+	
     switch (migtype)
     {
-    case '0':
-        numparam = nr->world->options->zeron;
-        remainnum = 0;
-        break;
-    case 'm':
-        numparam = nr->world->options->mmn;
-        remainnum = 1;
-        if (nr->world->options->custm2[which] == 'm')
-            return;
-        break;
+		case '0':
+			numparam = nr->world->options->zeron;
+			remainnum = 0;
+			break;
+		case 'm':
+			numparam = nr->world->options->mmn;
+			remainnum = 1;
+			if (nr->world->options->custm2[which] == 'm')
+				return;
+				break;
     }
     freeparam = (nr->numpop2 - numparam - 1 + remainnum);
     for (m = which; m < nr->numpop2; m++)
     {
         savecustm2 = custm2[m];
         custm2[m] = migtype;
-        memcpy (nr->world->param0, param0, sizeof (double) * nr->numpop2);
+        memcpy (nr->world->param0, param0, sizeof (MYREAL) * nr->numpop2);
         resynchronize_param (nr->world);
-
+		
         if ((legal = legal_pattern (nr->world->options->custm2, nr->numpop)))
         {
             do_profiles (nr->world, nr, &likes, &normd, PROFILE,
                          nr->world->rep, nr->world->repkind);
-
+			
             add_aiclist(migtype, numparam, freeparam, remainnum, zero, m, param0,
                         temppattern, nr->world->options->custm2, aicnum, aicvec, nr);
         }
@@ -519,6 +528,7 @@ aic_score (aic_fmt ** aicvec, long *aicnum, nr_fmt * nr,
     }
 }
 
+/// check how many parameter are set to zero or are set to m (average)
 boolean
 check_numparam (long which, long migtype, worldoption_fmt * options,
                 long *numparam, long *remainnum)
@@ -526,25 +536,30 @@ check_numparam (long which, long migtype, worldoption_fmt * options,
     boolean rc = FALSE;
     switch (migtype)
     {
-    case '0':
-        *numparam = options->zeron;
-        *remainnum = 0;
-        break;
-    case 'm':
-        *numparam = options->mmn;
-        *remainnum = 1;
-        if (options->custm2[which] == 'm')
-            rc = TRUE;
-        break;
+		case '0':
+			*numparam = options->zeron;
+			*remainnum = 0;
+			break;
+		case 'm':
+			*numparam = options->mmn;
+			*remainnum = 1;
+			if (options->custm2[which] == 'm')
+				rc = TRUE;
+				break;
     }
     return rc;
 }
 
+///
+/// check whether the migration pattern in matrix is legal or not
+/// a legal migration pattern, needs to connect all populations
+/// eahc population needs to be connected to at least one other population
+/// in one or both direction of the migration route  
 boolean
 legal_pattern (char *matrix, long numpop)
 {
     long from, to, i;
-    double summ = -1;
+    MYREAL summ = -1;
     long oldto;
     for (i = 0; i < numpop; i++)
     {
@@ -584,7 +599,7 @@ long find_paramnum(world_fmt *world, char *connect)
     total = world->numpop * world->numpop + (world->options->gamma ? 1 : 0) ;
     
     if(connect==NULL)
-      {
+	{
         // checkout zeroes and constants in custom migration matrix
         // removes 1 for each zero and (constant>0)
         total -= world->options->zeron + world->options->constn;
@@ -595,19 +610,19 @@ long find_paramnum(world_fmt *world, char *connect)
         // checkout symmetric values
         // removes 1 parameter for each migration pair
         total -= world->options->symn + world->options->sym2n;
-      }
+	}
     else
-      {
-        temp1 = (char *) calloc(strlen(connect)+1, sizeof(char));
-        temp2 = (char *) calloc(strlen(connect)+1, sizeof(char));
-        temp = (char *) calloc(strlen(connect)+1, sizeof(char));
+	{
+        temp1 = (char *) mycalloc(strlen(connect)+1, sizeof(char));
+        temp2 = (char *) mycalloc(strlen(connect)+1, sizeof(char));
+        temp = (char *) mycalloc(strlen(connect)+1, sizeof(char));
         t = temp;
         strcpy(temp1,connect);
         temp2 = strtok(temp1," ;,:\n\t\r");
         while(temp2!= NULL)
-          {
+		{
             switch(temp2[0])
-              {
+			{
                 case '*': *t++ = '*'; break;
                 case 'c': *t++ = 'c'; break;
                 case 'm': *t++ = 'm'; break;
@@ -617,9 +632,9 @@ long find_paramnum(world_fmt *world, char *connect)
                 case ' ' : break;
                 default:
                     *t++ = 'c';
-              }
+			}
             temp2 = strtok(NULL," ;,:\n\t\r");
-          }
+		}
         cc = scan_connect (temp, 0, world->numpop2, 'c');
         tm = scan_connect (temp, 0, world->numpop, 'm');
         mm = scan_connect (temp, world->numpop, world->numpop2, 'm');
@@ -633,6 +648,6 @@ long find_paramnum(world_fmt *world, char *connect)
         free(temp);
         free(temp1);
         free(temp2);
-      }
+	}
     return total;
 }
