@@ -4,19 +4,20 @@
 //
 // started November 2000
 // Peter Beerli Seattle 2000
-// $Id: bayes.h,v 1.7 2003/10/21 15:57:52 beerli Exp $
+// $Id: bayes.h,v 1.10 2004/04/19 15:27:11 beerli Exp $
 //
 #include "migration.h"
 #ifdef BAYESUPDATE
-extern boolean acceptBayes (double newval, double oldval);
-extern double propose_uni_newparam (double param, double delta, double minparam, double maxparam);
-extern double propose_exp_newparam (double param, double delta, double minparam, double maxparam);
+extern boolean acceptBayes (MYREAL newval, MYREAL oldval);
 extern long bayes_update (world_fmt *world);
 extern void bayes_free(world_fmt *world);
 extern void bayes_fill(world_fmt *world, option_fmt *options);
 extern void bayes_init(bayes_fmt *bayes, long size);
 extern void bayes_save(world_fmt *world);
 extern void bayes_stat(world_fmt *world);
+extern void bayes_init_histogram(world_fmt * world, option_fmt * options);
+extern void calculate_credibility_interval(world_fmt * world, long locus);
+extern void bayes_reset(world_fmt * world);
 
 #endif /*baysupdate*/
 
