@@ -14,20 +14,30 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c,v 1.47 2003/11/04 16:12:08 beerli Exp $
+$Id: data.c,v 1.54 2005/01/21 16:48:33 beerli Exp $
  
 -------------------------------------------------------*/
+/*! \file data.c
+
+Data manipulation routines
+
+*/
+
 #include <string.h>
 
 #include "migration.h"
-
+#include "sighandler.h"
+#include "tools.h"
+#include "migrate_mpi.h"
+#include "data.h"
 extern long number_genomes (char type);
 
 #ifdef DMALLOC_FUNC_CHECK
@@ -57,7 +67,7 @@ void read_alleles (FILE * infile, data_fmt * data, long pop, long ind);
 long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
                    long locus, long pop, long ind, long baseread);
 void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
-                             double **m);
+                             MYREAL **m);
 void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
                       long pop, long baseread);
 void print_alleledata (world_fmt * world, data_fmt * data,
@@ -83,10 +93,11 @@ void read_uepfile (data_fmt * data, option_fmt * options, long numpop);
 
 
 /*=====================================================*/
+/// creates the data structure
 void
 create_data (data_fmt ** data)
 {
-    (*data) = (data_fmt *) calloc (1, sizeof (data_fmt));
+    (*data) = (data_fmt *) mycalloc (1, sizeof (data_fmt));
 }
 
 /*
@@ -154,10 +165,6 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options)
     if (options->progress)
         fprintf (stdout, "\n\n");
     init_data_structure3 (data);
-#ifndef NEWSTYLE
-    /* sets all data->numind to the numind[0] */
-    set_numind (data);
-#endif
 
     switch (options->datatype)
     {
@@ -186,25 +193,25 @@ init_data_structure1 (data_fmt ** data, option_fmt * options)
     if ((*data)->yy == NULL)
     {
         (*data)->yy =
-            (char *****) malloc (sizeof (char ****) * (*data)->numpop);
-        (*data)->seq = (seqmodel_fmt *) calloc (1, sizeof (seqmodel_fmt));
+            (char *****) mymalloc (sizeof (char ****) * (*data)->numpop);
+        (*data)->seq = (seqmodel_fmt *) mycalloc (1, sizeof (seqmodel_fmt));
         (*data)->popnames =
-            (char **) malloc (sizeof (char *) * (*data)->numpop);
+            (char **) mymalloc (sizeof (char *) * (*data)->numpop);
         (*data)->indnames =
-            (char ***) malloc (sizeof (char **) * (*data)->numpop);
-        (*data)->numind = (long **) malloc (sizeof (long *) * (*data)->numpop);
-        (*data)->numalleles = (long **) malloc (sizeof (long *) * (*data)->numpop);
+            (char ***) mymalloc (sizeof (char **) * (*data)->numpop);
+        (*data)->numind = (long **) mymalloc (sizeof (long *) * (*data)->numpop);
+        (*data)->numalleles = (long **) mymalloc (sizeof (long *) * (*data)->numpop);
         for (pop = 0; pop < (*data)->numpop; pop++)
         {
             (*data)->popnames[pop] =
-                (char *) calloc (1, sizeof (char) * LINESIZE);
+                (char *) mycalloc (1, sizeof (char) * LINESIZE);
             (*data)->numind[pop] =
-                (long *) malloc (sizeof (long) * (*data)->loci);
+                (long *) mymalloc (sizeof (long) * (*data)->loci);
             (*data)->numalleles[pop] =
-                (long *) malloc (sizeof (long) * (*data)->loci);
+                (long *) mymalloc (sizeof (long) * (*data)->loci);
         }
         (*data)->seq->sites =
-            (long *) calloc (1, sizeof (long) * (*data)->loci);
+            (long *) mycalloc (1, sizeof (long) * (*data)->loci);
     }
     else
     {
@@ -225,31 +232,31 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
     }
     if (indalloc == 0)
         indalloc = 2;
-    (*data)->yy[pop] = (char ****) malloc (sizeof (char ***) * indalloc);
-    (*data)->indnames[pop] = (char **) calloc (1, sizeof (char *) * indalloc);
+    (*data)->yy[pop] = (char ****) mymalloc (sizeof (char ***) * indalloc);
+    (*data)->indnames[pop] = (char **) mycalloc (1, sizeof (char *) * indalloc);
     for (ind = 0; ind < indalloc; ind++)
     {
         (*data)->indnames[pop][ind] =
-            (char *) calloc (1, sizeof (char) * (1 + options->nmlength));
+            (char *) mycalloc (1, sizeof (char) * (1 + options->nmlength));
         (*data)->yy[pop][ind] =
-            (char ***) malloc (sizeof (char **) * (*data)->loci);
+            (char ***) mymalloc (sizeof (char **) * (*data)->loci);
         for (locus = 0; locus < (*data)->loci; locus++)
         {
             if (!strchr (SEQUENCETYPES, options->datatype))
             {
                 (*data)->yy[pop][ind][locus] =
-                    (char **) calloc (1, sizeof (char *) * 2);
+                    (char **) mycalloc (1, sizeof (char *) * 2);
                 (*data)->yy[pop][ind][locus][0] =
-                    (char *) calloc (1, sizeof (char) * (options->allelenmlength+1));
+                    (char *) mycalloc (1, sizeof (char) * (options->allelenmlength+1));
                 (*data)->yy[pop][ind][locus][1] =
-                    (char *) calloc (1, sizeof (char) * (options->allelenmlength+1));
+                    (char *) mycalloc (1, sizeof (char) * (options->allelenmlength+1));
             }
             else
             {
                 (*data)->yy[pop][ind][locus] =
-                    (char **) calloc (1, sizeof (char *));
+                    (char **) mycalloc (1, sizeof (char *));
                 (*data)->yy[pop][ind][locus][0] =
-                    (char *) calloc (1,
+                    (char *) mycalloc (1,
                                      sizeof (char) * ((*data)->seq->sites[locus]+1));
             }
         }
@@ -287,18 +294,18 @@ init_data_structure3 (data_fmt * data)
 {
     long locus, pop, maxi;
     data->allele =
-        (allele_fmt **) calloc (1, sizeof (allele_fmt *) * data->loci);
+        (allele_fmt **) mycalloc (1, sizeof (allele_fmt *) * data->loci);
     for (locus = 0; locus < data->loci; locus++)
     {
         maxi = 0;
         for (pop = 0; pop < data->numpop; pop++)
             maxi += data->numalleles[pop][locus];
         data->allele[locus] =
-            (allele_fmt *) calloc (1, sizeof (allele_fmt) * maxi);
+            (allele_fmt *) mycalloc (1, sizeof (allele_fmt) * maxi);
     }
-    data->maxalleles = (long *) calloc (1, sizeof (long) * data->loci);
+    data->maxalleles = (long *) mycalloc (1, sizeof (long) * data->loci);
     data->skiploci =
-        (boolean *) calloc (1, sizeof (boolean) * (data->loci + 1));
+        (boolean *) mycalloc (1, sizeof (boolean) * (data->loci + 1));
 }
 
 
@@ -393,7 +400,7 @@ read_sites (data_fmt * data)
 {
     long locus;
     char *input, *p, *a;
-    input = (char *) calloc (LINESIZE, sizeof (char));
+    input = (char *) mycalloc (LINESIZE, sizeof (char));
     FGETS (input, LINESIZE, data->infile);
     if ((p = (char *) strpbrk (input, CRLF)) != NULL)
         *p = '\0';
@@ -430,7 +437,7 @@ read_sites (data_fmt * data)
 {
     long locus;
     char *input;
-    input = (char *) calloc(LINESIZE,sizeof(char));
+    input = (char *) mycalloc(LINESIZE,sizeof(char));
     
     for (locus = 0; locus < data->loci-1; locus++)
     {
@@ -461,7 +468,7 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
     FGETS (input, sizeof (input), infile);
     if ((p = (char *) strpbrk (input, CRLF)) != NULL)
         *p = '\0';
-#ifdef NEWSTYLE
+
     // allows that sequence data can have different numbers of individuals for different loci
     // data syntax changes: #ind1 #ind2 #IND3 .... pop_name
     havepopname=FALSE;
@@ -509,25 +516,15 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
                 data->popnames[pop]);
         data->numalleles[pop][0] = data->numind[pop][0]*genomes;
     }
-#else
-    // assume all loci have the same number of individuals
-    sscanf (input, "%ld%[^\n]", &(data->numind[pop][0]),
-            data->popnames[pop]);
-    data->numalleles[pop][0] = data->numind[pop][0]*genomes;
-#endif
 
     translate (data->popnames[pop], ' ', '_');
     translate (data->popnames[pop], '\t', '_');
-    //trial #ifdef NEWSTYLE
+    //trial 
     //  for(locus=0;locus<data->loci; locus++)
     //    {
     //      if (data->numind[pop][locus] == 0)
     //        data->hasghost = FALSE ;//TRUE;
     //    }
-    //#else
-    //  if (data->numind[pop][0] == 0)
-    //    data->hasghost = TRUE;
-    //#endif
 }
 */
 
@@ -538,8 +535,8 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
     boolean havepopname=FALSE;
     long locus;
     char *input;
-    input = (char *) calloc(LINESIZE,sizeof(char));
-#ifdef NEWSTYLE
+    input = (char *) mycalloc(LINESIZE,sizeof(char));
+
     // allows that sequence data can have different numbers of individuals for different loci
     // data syntax changes: #ind1 #ind2 #IND3 .... pop_name
     havepopname=FALSE;
@@ -586,16 +583,10 @@ read_popheader (FILE * infile, data_fmt * data, long pop, long genomes)
                 data->popnames[pop]);
         data->numalleles[pop][0] = data->numind[pop][0]*genomes;
     }
-#else
-    // assume all loci have the same number of individuals
-    FGETS(input,LINESIZE,infile);
-    sscanf (input, "%ld%[^\n]", &(data->numind[pop][0]),
-            data->popnames[pop]);
-    data->numalleles[pop][0] = data->numind[pop][0]*genomes;
-#endif
 
     translate (data->popnames[pop], ' ', '_');
     translate (data->popnames[pop], '\t', '_');
+    free(input);
 }
 
 
@@ -658,11 +649,11 @@ read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
 {
     char *input, *isave, dlm[2], ddlm[2], *p, *a, *a1, *a2;
     long locus, i;
-    input = (char *) calloc (1, sizeof (char) * (SUPERLINESIZE + 1));
+    input = (char *) mycalloc (1, sizeof (char) * (SUPERLINESIZE + 1));
     isave = input;
-    a = (char *) calloc (1, sizeof (char) * LINESIZE);
-    a1 = (char *) calloc (1, sizeof (char) * LINESIZE);
-    a2 = (char *) calloc (1, sizeof (char) * LINESIZE);
+    a = (char *) mycalloc (1, sizeof (char) * LINESIZE);
+    a1 = (char *) mycalloc (1, sizeof (char) * LINESIZE);
+    a2 = (char *) mycalloc (1, sizeof (char) * LINESIZE);
     dlm[0] = data->dlm, dlm[1] = '\0';
     ddlm[0] = ' ', ddlm[1] = '\0';
     FGETS (input, SUPERLINESIZE, infile);
@@ -716,9 +707,9 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
 {
     char *input, *isave, *p, *a;
     long locus;
-    a = (char *) calloc (1, sizeof (char) * LINESIZE);
+    a = (char *) mycalloc (1, sizeof (char) * LINESIZE);
 
-    input = (char *) calloc (1, sizeof (char) * SUPERLINESIZE);
+    input = (char *) mycalloc (1, sizeof (char) * SUPERLINESIZE);
     isave = input;
     FGETS (input, SUPERLINESIZE, infile);
     if ((p = (char *) strpbrk (input, CRLF)) != NULL)
@@ -731,7 +722,7 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
         }
         if (sscanf (input, "%s", a) == 1)
         {
-            input += strlen (a);
+            input += (long) strlen (a);
         }
 
         data->yy[pop][ind][locus][0][0] = a[0];
@@ -809,7 +800,7 @@ read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
 }
 
 void
-read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m)
+read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m)
 {
     char input[SUPERLINESIZE];
     long i, j;
@@ -825,7 +816,11 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, double **m)
             FGETS (input, nmlength + 1, dfile);
             for (j = 0; j < tips; j++)
             {
+#ifdef USE_MYREAL_FLOAT
+                fscanf (dfile, "%f", &m[i][j]);
+#else
                 fscanf (dfile, "%lf", &m[i][j]);
+#endif
             }
             // reads the last \n from the
             // data matrix
@@ -857,7 +852,7 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, int **uep,
         {
             for (i = 0; i < tips; i++)
             {
-                uep[i] = (int *) calloc (*uepsites, sizeof (int));
+                uep[i] = (int *) mycalloc (*uepsites, sizeof (int));
                 FGETS (input, nmlength + 1, uepfile); //reads each line
                 for (j = 0; j < *uepsites; ++j)
                     fscanf (uepfile, "%i", &uep[i][j]);
@@ -869,8 +864,8 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, int **uep,
         {
             for (i = 0; i < tips; i++)
             {
-                uep[i] = (int *) calloc (*uepsites, sizeof (int));
-                uep[i + tips] = (int *) calloc (*uepsites, sizeof (int));
+                uep[i] = (int *) mycalloc (*uepsites, sizeof (int));
+                uep[i + tips] = (int *) mycalloc (*uepsites, sizeof (int));
                 FGETS (input, nmlength + 1, uepfile); //reads each line
                 for (j = 0; j < *uepsites; ++j)
                     fscanf (uepfile, "%i", &uep[i][j]);
@@ -953,8 +948,8 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     char dstring[LINESIZE];
     long *total;
     long *totalmiss;
-    total = (long *) calloc(data->loci,sizeof(long));
-    totalmiss = (long *) calloc(data->loci,sizeof(long));
+    total = (long *) mycalloc(data->loci,sizeof(long));
+    totalmiss = (long *) mycalloc(data->loci,sizeof(long));
     fprintf (file, "Summary of data:\n");
     fprintf (file, "---------------\n");
     switch (options->datatype)
@@ -1088,14 +1083,14 @@ print_alleledata (world_fmt * world, data_fmt * data, option_fmt * options)
             for (locus = 0; locus < data->loci; locus++)
             {
                 mult80 +=
-                    1 + strlen (data->yy[pop][ind][locus][0]) +
-                    strlen (data->yy[pop][ind][locus][1]);
+                    1 + (long) (strlen (data->yy[pop][ind][locus][0]) +
+                    strlen (data->yy[pop][ind][locus][1]));
                 if (mult80 >= 80)
                 {
                     mult80 = 0;
                     fprintf (world->outfile, "\n");
                     for (i = 0; i < options->nmlength; i++)
-                        fputc (' ', world->outfile);
+                        FPRINTF(world->outfile, " ");
                 }
                 fprintf (world->outfile, " %s.%-s",
                          data->yy[pop][ind][locus][0],
@@ -1133,14 +1128,14 @@ print_header (FILE * outfile, long pop, world_fmt * world,
     long locus, mult80 = 80;
     char input[LINESIZE];
     fprintf (outfile, "\n%-s", data->popnames[pop]);
-    for (i = 0; i < (long) (80 - strlen (data->popnames[pop])); i++)
-        fputc ('-', outfile);
+    for (i = 0; i < (long) (80 - (long) strlen (data->popnames[pop])); i++)
+         fprintf(world->outfile, "-");
     fprintf (outfile, "\n\n");
     if (!strchr (SEQUENCETYPES, options->datatype))
     {
         fprintf (outfile, "%-s  ", (data->loci == 1 ? "locus" : "loci "));
         for (i = 0; i < (options->nmlength - 6); i++)
-            fputc (' ', outfile);
+            fprintf(world->outfile, " ");
         for (locus = 0; locus < data->loci; locus++)
         {
             if (locus * 4 + options->nmlength > mult80)
@@ -1148,7 +1143,7 @@ print_header (FILE * outfile, long pop, world_fmt * world,
                 mult80 += 80;
                 fprintf (outfile, "\n");
                 for (i = 0; i < options->nmlength; i++)
-                    fputc (' ', outfile);
+                    fprintf (outfile, " ");
             }
             fprintf (outfile, "  %2li", locus + 1);
         }
@@ -1266,7 +1261,7 @@ print_locus_head (long locus, world_fmt * world, option_fmt * options,
                   data_fmt * data)
 {
     char *head;
-    head = (char *) calloc (1, sizeof (char) * MAX (10, options->nmlength));
+    head = (char *) mycalloc (1, sizeof (char) * MAX (10, options->nmlength));
     sprintf (head, "Locus %li", locus);
     fprintf (world->outfile, "%-*.*s --------10 --------20 --------30",
              (int) options->nmlength, (int) options->nmlength, head);
@@ -1280,8 +1275,8 @@ read_geofile (data_fmt * data, option_fmt * options, long numpop)
 {
     long i, j, pop;
     long numpop2 = numpop * numpop;
-    data->geo = (double *) calloc (1, sizeof (double) * numpop2);
-    data->lgeo = (double *) calloc (1, sizeof (double) * numpop2);
+    data->geo = (MYREAL *) mycalloc (1, sizeof (MYREAL) * numpop2);
+    data->lgeo = (MYREAL *) mycalloc (1, sizeof (MYREAL) * numpop2);
     if (!options->geo)
     {
         for (i = 0; i < numpop2; i++)
@@ -1289,8 +1284,8 @@ read_geofile (data_fmt * data, option_fmt * options, long numpop)
     }
     else
     {
-        data->ogeo = (double **) calloc (1, sizeof (double *) * numpop);
-        data->ogeo[0] = (double *) calloc (1, sizeof (double) * numpop2);
+        data->ogeo = (MYREAL **) mycalloc (1, sizeof (MYREAL *) * numpop);
+        data->ogeo[0] = (MYREAL *) mycalloc (1, sizeof (MYREAL) * numpop2);
         for (pop = 1; pop < numpop; pop++)
             data->ogeo[pop] = data->ogeo[0] + numpop * pop;
         read_distance_fromfile (data->geofile, numpop, options->nmlength,
@@ -1323,7 +1318,7 @@ read_uepfile (data_fmt * data, option_fmt * options, long numpop)
     for (i = 0; i < numpop; ++i)
         sumtips += data->numind[i][0];   //Assumes that UEP has the same number of individuals as
     // locus 1 (Is this OK? most dataset with UEP will have 1 locus?????)
-    data->uep = (int **) calloc (number_genomes (options->datatype) * sumtips,
+    data->uep = (int **) mycalloc (number_genomes (options->datatype) * sumtips,
                                  sizeof (int *));
     read_uep_fromfile (data->uepfile, sumtips, options->nmlength, data->uep,
                        &data->uepsites, options->datatype);
