@@ -20,9 +20,10 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: data.h,v 1.14 2003/11/04 16:12:08 beerli Exp $
+$Id: data.h,v 1.15 2004/02/03 16:33:14 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
@@ -35,7 +36,7 @@ extern void print_data_summary (FILE * file, world_fmt * world,
                                     option_fmt * options, data_fmt * data);
 extern void free_datapart (data_fmt * data, option_fmt * options, long locus);
 extern void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
-                                        double **m);
+                                        MYREAL **m);
 extern void read_geofile (data_fmt * data, option_fmt * options, long numpop);
 // used in MPI context
 extern void init_data_structure1 (data_fmt ** data, option_fmt * options);
