@@ -6,9 +6,9 @@
   using coalescent trees
  
   Peter Beerli
-  Genetics 357360
-  University of Washington
-  Seattle, WA 98195-7360, USA
+  School of Computational Science and Biological Sciences
+  Florida State University
+  Tallahassee, FL 32306-4120
   beerli@csit.fsu.edu
  
   With help of Joe Felsenstein (joe@genetics.washington.edu),
@@ -17,16 +17,16 @@
   
 Copyright 2002 Peter Beerli and Joseph Felsenstein
  
-$Id: definitions.h,v 1.85 2003/11/19 03:52:37 beerli Exp $
+$Id: definitions.h,v 1.96 2005/01/21 16:48:34 beerli Exp $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "1.7.6.1"
+#define MIGRATEVERSION "2.0.5"
 #else
 #ifdef SGI
 #undef MIGRATEVERSION
-#define MIGRATEVERSION "1.7.6.1"
+#define MIGRATEVERSION "2.0.5"
 #endif
 #endif
 
@@ -34,6 +34,12 @@ $Id: definitions.h,v 1.85 2003/11/19 03:52:37 beerli Exp $
 /*------------------------------------------------
   Adjusting the source to the system configuration
 */
+#define USE_MYREAL_DOUBLE 1
+#define MYREAL double
+// use this when using floats 
+// #define USE_MYREAL_FLOAT 0
+//#define MYREAL float
+
 #ifdef HAVE_CONFIG_H
 #include "conf.h"
 #endif
@@ -43,32 +49,42 @@ $Id: definitions.h,v 1.85 2003/11/19 03:52:37 beerli Exp $
 #endif
 
 #define FGETS  myfgets
+
 #ifdef MPI
 #define FPRINTF mpi_fprintf
 #else
 #define FPRINTF fprintf
 #endif
+
 #ifdef __MWERKS__
-#include "conf.h"
-#undef HAVE_LGAMMA
-#define HAVE_STRFTIME 1
-#define HIGHBITS
-#define LAMARC_MALLOC
-#define NEWSTYLE
-#ifdef macintosh
-#define MAC
-#include <ansi_prefix.mac.h> /*fixes time problems */
+    #include "conf.h"
+    #undef HAVE_LGAMMA
+    #define HAVE_STRFTIME 1
+    #define HIGHBITS
+    #define LAMARC_MALLOC
+    #ifdef macintosh
+    #define MAC
+    #include <ansi_prefix.mac.h> /*fixes time problems */
 #else /*macintosh */
-#define DOS
-#undef HAVE_STRINGS_H
-#undef HAVE_LGAMMA
-#endif /*windows */
+    #define DOS
+    #undef HAVE_STRINGS_H
+    #undef HAVE_LGAMMA
+    #endif /*windows */
 #endif /*MWERKS*/
+
+#ifdef WIN32
+#define BAYESUPDATE
+    #define DOS
+    #undef HAVE_STRINGS_H
+    #undef HAVE_LGAMMA
+#endif
+
 /* we have found no strftime() and replace the time
    output with blanks*/
 #ifndef HAVE_STRFTIME
-#define NOTIME_FUNC
+    #define NOTIME_FUNC
 #endif
+
 #define STDOUTNUM 0
 #define INFILE "infile"
 #define INFILENUM 1
@@ -110,6 +126,7 @@ $Id: definitions.h,v 1.85 2003/11/19 03:52:37 beerli Exp $
 #define AICFILENUM 19
 #define BAYESFILE "bayesfile"
 #define BAYESFILENUM 20
+#define BAYESNUMBIN 200
 #ifdef __STDIO_LOADED
 #define VMS
 #define MACHINE_TYPE "VAX/VMS C"
@@ -132,22 +149,40 @@ void vax_tweak_fmt (char *);
 #include <stdlib.h>
 #include <math.h>
 #include <ctype.h>
+
 #ifdef MAC
-#include "mac_interface.h"
+    #include "mac_interface.h"
 #endif /*MAC*/
+
 #include <time.h>
 #include <math.h>
 #include <limits.h>
-#include <signal.h>
-#ifdef __MWERKS__
-#undef DBL_EPSILON
-#undef DBL_MAX
-#include <float.h>
-#endif /*__MWERKS__*/
-#ifdef HAVE_MALLOCWRAP
+
+
+#ifdef WIN32
+    #define MYINLINE __inline
+    #undef DBL_EPSILON
+    #undef DBL_MAX
+    #include <float.h>
+    #define MYISNAN _isnan
+    #define MYSNPRINTF _snprintf
+#else  
+	#define MYISNAN  isnan
+    #define MYSNPRINTF snprintf
+    #ifdef __MWERKS__
+		#define MYINLINE /*inline*/
+		#undef DBL_EPSILON
+		#undef DBL_MAX
+		#include <float.h>
+	#else
+		#define MYINLINE inline
+	#endif
+#endif
+
 /*if we use this then we guard against allocation problems */
 /* redefinitions of calloc, malloc, realloc are in sighandler.h*/
-#define LAMARC_MALLOC
+#ifdef HAVE_MALLOCWRAP
+//#define LAMARC_MALLOC
 #endif
 
 #ifdef HAVE_DEBUGMPI
@@ -161,7 +196,7 @@ void vax_tweak_fmt (char *);
 #define LGAMMA mylgamma
 #endif
 
-#include "sighandler.h"
+//#include "sighandler.h"
 #define FClose(file) if (file) fclose(file) ; file=NULL
 typedef unsigned char boolean;
 
@@ -192,7 +227,7 @@ typedef unsigned char boolean;
 
 // I was experimenting with a fast log implementation
 // but it is
-//not fast enough #define LOG(a) ((double) fast_log(((float) a)))
+//not fast enough #define LOG(a) ((MYREAL) fast_log(((float) a)))
 // so back to standard log
 #define LOG(a) log((a))
 
@@ -212,40 +247,44 @@ typedef unsigned char boolean;
 #endif
 /*=================================================================
   SPECIFIC FOR MIGRATE
- 
-  *----------------------------------------------------------------
-  + = works
-  - = not yet implemented (needed?)
- 
+  
   */
+#define MASTER 0
+#define FIRSTWORKER 1
+#define STOP_REPLICATORS -1234
 /* number of k int k-allele model, the number
    MUST really be bigger than the number of 
    observed alleles, much bigger does not harm */
-#define MAXALLELES            93
-#define SMALLBUFSIZE    255
-#define MAXBUFSIZE        100000
-#define LINESIZE            10000 /* setting this smaller can break */
-#define LONGLINESIZE       10000 /* setting this smaller can break */
-#define SUPERLINESIZE    100000/*used to read many many loci-sites and indiv numbers*/ 
-#define STRSIZE              255 /* setting this smaller can break */
-#define DEFAULT_NMLENGTH      10 /* length of individual names */
-#define DEFAULT_ALLELENMLENGTH 6 /* length of allele names */
-#define DEFAULT_POPNMLENGTH  100 /* length of world names */
-#define NUMPOP 2
-#define BURNINPERIOD 10000
-#define SCALEINTERVAL 5
-#define DEFAULTALLOCSIZE 2
+#define MAXALLELES            93L
+#define SMALLBUFSIZE    255L
+#define MAXBUFSIZE        100000L
+#define LINESIZE            10000L /* setting this smaller can break */
+#define LONGLINESIZE       10000L /* setting this smaller can break */
+#define SUPERLINESIZE    100000L /*used to read many many loci-sites and indiv numbers*/ 
+#define STRSIZE              255L /* setting this smaller can break */
+#define DEFAULT_NMLENGTH      10L /* length of individual names */
+#define DEFAULT_ALLELENMLENGTH 6L /* length of allele names */
+#define DEFAULT_POPNMLENGTH  100L /* length of world names */
+#define NUMPOP 2L
+#define BURNINPERIOD 10000L
+#define SCALEINTERVAL 5L
+#define DEFAULTALLOCSIZE 2L
 /* don't change below here ------------------------------------
    some other constants */
 /* numerical borders and epsilons*/
 #ifndef HUGE
 #define HUGE 1e200
 #endif 
+#define ONE                     1L
+#define TWO                     2L
+#define THREE                   3L
+#define FOUR                    4L
 #define SMALL_VALUE       10e-21
 #define EPSILON         0.000001 /* a small number */
 #define EPSILON4        0.000099 /* another small number */
 #define SMALLEPSILON       1e-15 /* a smaller number */
 #define BIGEPSILON         0.001 /* a not so small number */
+#define PERCENTILETOLERANCE 0.01 /* difference larger than this are flagged FAILED*/
 #define LONGCHAINEPSILON  10e100 /* an unreasonable big number,
 so that the given number of
 long chains is used */
@@ -257,7 +296,7 @@ long chains is used */
 #ifndef DBL_EPSILON
 #include <float.h>
 #ifndef DBL_EPSILON
-#define DBL_MAX ((double)1.7976931348623157e308)
+#define DBL_MAX ((MYREAL)1.7976931348623157e308)
 #define DBL_EPSILON 2.2204460492503131e-16
 #endif
 #endif
@@ -286,15 +325,22 @@ long chains is used */
 #define ATOL(a) strtol((a),(char **) NULL, 10)
 #define ATOI(a) strtol((a),(char **) NULL, 10)
 #define ATOF(a) strtod((a),(char **) NULL, 10)
+
+/* prior setting */
+#define ADAPTIVEPRIOR 3
+#define EXPPRIORB 2
+#define EXPPRIOR 1
+#define UNIFORMPRIOR 0
+
 /* theta (4 Ne mu) related material */
 #define SMALLEST_THETA    1e-10
 #define BIGGEST_THETA      1e4
-#define SMALLEST_MIGRATION 1e-10
+#define SMALLEST_MIGRATION 1      /*m/mu equally likely*/
 #define BIGGEST_MIGRATION   10e9
-// defines the value that is used as a minimum summary statistic for migraiton events on all trees
+// defines the value that is used as a minimum summary statistic for migration events on all trees
 // version 1.2.4 had a value of 0.00000001, it seems that values of 0.1 are too large.
 // in version -1.6.9 this is set to zero, in 1.7 trial with 0.01 PB Dec 11 2002 
-#define MINMIGSUMSTAT 0.01
+#define MINMIGSUMSTAT 1.0
 /* the migration limit is per population, should be plenty
    for moderate sample sizes */
 #define MIGRATION_LIMIT     5000
@@ -362,6 +408,8 @@ long chains is used */
 #define MULTILOCUS             1 /*+ */
 #define PROFILE                2 /*+ */
 /* print and plot material*/
+#define MULTILOCUSPLOT         (boolean) 1
+#define SINGLELOCUSPLOT		   (boolean) 0
 #define MAXPRINTVALLENGTH 16 /*+ maximal number of printpos */
 #define PLOTALL                0 /*+ plot to outfile and mathfile */
 #define PLOTOUTFILE            1 /*+ plot only to outifle */
@@ -385,7 +433,7 @@ long chains is used */
 #define LASTCHAIN              3 /*+ */
 /* likelihood ration stuff*/
 #define LRATIO_STRINGS      1000 /*+ */
-#define HUNDRED              100
+#define HUNDRED             100
 /* profile likelihood stuff*/
 /* ALL and NONE are alread defined*/
 #define TABLES                 2 /*+ */
@@ -418,6 +466,7 @@ long chains is used */
 
 
 #ifdef MEMDEBUG
-#include <memdebug.h>
+//#include <memdebug.h>
+#include "CMemLeak.h"
 #endif
 #endif /*definitions */
