@@ -10,20 +10,24 @@ using a Metropolis-Hastings Monte Carlo algorithm
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
- Copyright 2001 Peter Beerli and Joseph Felsenstein
+ Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: fst.c,v 1.21 2003/11/04 16:12:08 beerli Exp $
+$Id: fst.c,v 1.24 2004/08/28 19:59:31 beerli Exp $
  
 -------------------------------------------------------*/
+/*! \file fst.c
+
+*/
 
 #include "migration.h"
 #include "tools.h"
-
+#include "sighandler.h"
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
@@ -33,18 +37,18 @@ void fst_type (char type);
 
 void calc_fst (world_fmt * world, data_fmt * data);
 /* private functions */
-void frequencies (double ***f, char *****data, long numpop,
+void frequencies (MYREAL ***f, char *****data, long numpop,
                   long **numind, long loci);
-void calc_fw (double ***f, long numpop, long locus, double *fw);
-void calc_fb (double ***f, long numpop, long locus, double *fb);
-void calc_seq_fw (data_fmt * data, long numpop, long locus, double *fw);
-void calc_seq_fb (data_fmt * data, long numpop, long locus, double *fb);
-void solveNm_vartheta (double *fw, double *fb, long numpop, double *params);
-void solveNm_varm (double *fw, double *fb, long numpop, double *params);
-void solveNm_sym (double *fw, double *fb, long numpop, double *params);
+void calc_fw (MYREAL ***f, long numpop, long locus, MYREAL *fw);
+void calc_fb (MYREAL ***f, long numpop, long locus, MYREAL *fb);
+void calc_seq_fw (data_fmt * data, long numpop, long locus, MYREAL *fw);
+void calc_seq_fb (data_fmt * data, long numpop, long locus, MYREAL *fb);
+void solveNm_vartheta (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params);
+void solveNm_varm (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params);
+void solveNm_sym (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params);
 
 /* global variable SOLVENM points to function solveNm_varxx() */
-static void (*solveNm) (double *, double *, long, double *);
+static void (*solveNm) (MYREAL *, MYREAL *, long, MYREAL *);
 
 /*=======================================================*/
 void
@@ -54,14 +58,14 @@ fst_type (char type)
     {
     case 'T':
         solveNm =
-            (void (*)(double *, double *, long, double *)) solveNm_vartheta;
+            (void (*)(MYREAL *, MYREAL *, long, MYREAL *)) solveNm_vartheta;
         break;
     case 'M':
-        solveNm = (void (*)(double *, double *, long, double *)) solveNm_varm;
+        solveNm = (void (*)(MYREAL *, MYREAL *, long, MYREAL *)) solveNm_varm;
         break;
     case 'S':
     default:
-        solveNm = (void (*)(double *, double *, long, double *)) solveNm_sym;
+        solveNm = (void (*)(MYREAL *, MYREAL *, long, MYREAL *)) solveNm_sym;
         break;
     }
 }
@@ -71,27 +75,27 @@ calc_fst (world_fmt * world, data_fmt * data)
 {
     long pop, locus;
     long connections = world->numpop * (world->numpop - 1) / 2;
-    double ***fstfreq = NULL,  *fw, *fb, *sumfb, *sumfw;
+    MYREAL ***fstfreq = NULL,  *fw, *fb, *sumfb, *sumfw;
     if (connections == 0)
         return;
-    fw = (double *) calloc (1, sizeof (double) * world->numpop);
-    fb = (double *) calloc (1, sizeof (double) * connections);
-    sumfw = (double *) calloc (1, sizeof (double) * world->numpop);
-    sumfb = (double *) calloc (1, sizeof (double) * connections);
+    fw = (MYREAL *) mycalloc (1, sizeof (MYREAL) * world->numpop);
+    fb = (MYREAL *) mycalloc (1, sizeof (MYREAL) * connections);
+    sumfw = (MYREAL *) mycalloc (1, sizeof (MYREAL) * world->numpop);
+    sumfb = (MYREAL *) mycalloc (1, sizeof (MYREAL) * connections);
     if (world->numpop > 2)
     {
         fst_type ('S');
     }
     if (!strchr (SEQUENCETYPES, world->options->datatype))
     {
-        fstfreq = (double ***) calloc (1, sizeof (double **) * world->numpop);
+        fstfreq = (MYREAL ***) mycalloc (1, sizeof (MYREAL **) * world->numpop);
         for (pop = 0; pop < world->numpop; pop++)
         {
             fstfreq[pop] =
-                (double **) calloc (1, sizeof (double *) * world->loci);
+                (MYREAL **) mycalloc (1, sizeof (MYREAL *) * world->loci);
             for (locus = 0; locus < world->loci; locus++)
                 fstfreq[pop][locus] =
-                    (double *) calloc (1, sizeof (double) * 255);
+                    (MYREAL *) mycalloc (1, sizeof (MYREAL) * 255);
         }
         frequencies (fstfreq, data->yy, world->numpop, data->numind,
                      world->loci);
@@ -150,16 +154,16 @@ calc_fst (world_fmt * world, data_fmt * data)
 /*=======================================================*/
 
 void
-frequencies (double ***f,  char *****data, long numpop,
+frequencies (MYREAL ***f,  char *****data, long numpop,
              long **numind, long loci)
 {
     long **buckets;
     long pop, locus, ind, a;
     long total=0;
-    buckets = (long **) malloc (sizeof (long *) * numpop);
+    buckets = (long **) mymalloc (sizeof (long *) * numpop);
     for (pop = 0; pop < numpop; pop++)
     {
-        buckets[pop] = (long *) calloc (255, sizeof (long));
+        buckets[pop] = (long *) mycalloc (255, sizeof (long));
         for (locus = 0; locus < loci; locus++)
         {
             memset (buckets[pop], 0, sizeof (long) * 255);
@@ -180,7 +184,7 @@ frequencies (double ***f,  char *****data, long numpop,
             for (a = 0; a < 255; a++)
             {
                 if (total > 0)
-                    f[pop][locus][a] = (double) buckets[pop][a] / (double) total;
+                    f[pop][locus][a] = (MYREAL) buckets[pop][a] / (MYREAL) total;
             }
         }
         free (buckets[pop]);
@@ -191,7 +195,7 @@ frequencies (double ***f,  char *****data, long numpop,
 
 
 void
-calc_fw (double ***f, long numpop, long locus, double *fw)
+calc_fw (MYREAL ***f, long numpop, long locus, MYREAL *fw)
 {
     long pop, i;
     for (pop = 0; pop < numpop; pop++)
@@ -204,7 +208,7 @@ calc_fw (double ***f, long numpop, long locus, double *fw)
     }
 }
 void
-calc_fb (double ***f, long numpop, long locus, double *fb)
+calc_fb (MYREAL ***f, long numpop, long locus, MYREAL *fb)
 {
     long i, p1, p2, zz = 0;
     for (p1 = 0; p1 < numpop; p1++)
@@ -222,11 +226,11 @@ calc_fb (double ***f, long numpop, long locus, double *fb)
 }
 
 void
-calc_seq_fw (data_fmt * data, long numpop, long locus, double *fw)
+calc_seq_fw (data_fmt * data, long numpop, long locus, MYREAL *fw)
 {
     long pop, i, k, j;
-    double nn;
-    double diff;
+    MYREAL nn;
+    MYREAL diff;
     for (pop = 0; pop < numpop; pop++)
     {
         fw[pop] = 0;
@@ -254,11 +258,11 @@ calc_seq_fw (data_fmt * data, long numpop, long locus, double *fw)
 }
 
 void
-calc_seq_fb (data_fmt * data, long numpop, long locus, double *fb)
+calc_seq_fb (data_fmt * data, long numpop, long locus, MYREAL *fb)
 {
     long i, k, j;
-    double nn, temp;
-    double diff;
+    MYREAL nn, temp;
+    MYREAL diff;
     long p1, p2, zz = 0;
     for (p1 = 0; p1 < numpop; p1++)
     {
@@ -266,7 +270,7 @@ calc_seq_fb (data_fmt * data, long numpop, long locus, double *fb)
         {
             temp = 0.;
             nn =
-                (double) data->seq->sites[locus] * data->numind[p1][locus] *
+                (MYREAL) data->seq->sites[locus] * data->numind[p1][locus] *
                 data->numind[p2][locus];
             for (i = 0; i < data->numind[p1][locus]; i++)
             {
@@ -291,12 +295,12 @@ calc_seq_fb (data_fmt * data, long numpop, long locus, double *fb)
 
 
 void
-solveNm_varm (double *fw, double *fb, long numpop, double *params)
+solveNm_varm (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params)
 {
     long i, p1 /*,p2 */ ;
-    /*Version 2.0  double sumfw = sum(fw,numpop);
-    double sumfb = sum(fb,numpop*(numpop-1)/2); */
-    double first, denom;
+    /*Version 2.0  MYREAL sumfw = sum(fw,numpop);
+    MYREAL sumfb = sum(fb,numpop*(numpop-1)/2); */
+    MYREAL first, denom;
     long offset2 = numpop + numpop * (numpop - 1);
     long offset = numpop;
     long numfb = 0;
@@ -336,11 +340,11 @@ solveNm_varm (double *fw, double *fb, long numpop, double *params)
 }
 
 void
-solveNm_vartheta (double *fw, double *fb, long numpop, double *params)
+solveNm_vartheta (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params)
 {
     long i;
-    double nom;
-    double denom, fulldenom;
+    MYREAL nom;
+    MYREAL denom, fulldenom;
     long offset2 = numpop + numpop * (numpop - 1);
     long numfb = 0;
     nom = (-2. * fb[0] + fw[0] + fw[1]);
@@ -380,17 +384,17 @@ solveNm_vartheta (double *fw, double *fb, long numpop, double *params)
 }
 
 void
-solveNm_sym (double *fw, double *fb, long numpop, double *params)
+solveNm_sym (MYREAL *fw, MYREAL *fb, long numpop, MYREAL *params)
 {
     long p1, p2;
-    double nom;
-    double denom, fulldenom;
-    double fbji;
-    double **xxx;
+    MYREAL nom;
+    MYREAL denom, fulldenom;
+    MYREAL fbji;
+    MYREAL **xxx;
     long offset2 = numpop + numpop * (numpop - 1);
     long z = 0;
-    xxx = (double **) calloc (1, sizeof (double *) * numpop);
-    xxx[0] = (double *) calloc (1, sizeof (double) * numpop * numpop);
+    xxx = (MYREAL **) mycalloc (1, sizeof (MYREAL *) * numpop);
+    xxx[0] = (MYREAL *) mycalloc (1, sizeof (MYREAL) * numpop * numpop);
     for (p1 = 1; p1 < numpop; p1++)
     {
         xxx[p1] = xxx[0] + numpop * p1;
