@@ -14,36 +14,45 @@ using a Metropolis-Hastings Monte Carlo algorithm
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: gammalike.c,v 1.15 2003/11/04 16:12:08 beerli Exp $
+$Id: gammalike.c,v 1.19 2004/08/28 19:59:31 beerli Exp $
 */
+/*! \file gammalike.c 
+
+Calculates the gamma distributed rate variation among loci
+and derivatives
+*/
+
+
 #include "migration.h"
+#include "tools.h"
 #include "broyden.h"
 #include "combroyden.h"
 #include "laguerre.h"
+#include "sighandler.h"
 
-double gamma_loci_like (helper_fmt *helper, double *oparam,
-                        double *olparam, double denom);
-double gamma_locus_like (nr_fmt * nr, double *oparam,
-                         double *olparam, double denom, long locus);
-double loggf (double r, double alpha, double theta1);
+MYREAL gamma_loci_like (helper_fmt *helper, MYREAL *oparam,
+                        MYREAL *olparam, MYREAL denom);
+MYREAL gamma_locus_like (nr_fmt * nr, MYREAL *oparam,
+                         MYREAL *olparam, MYREAL denom, long locus);
+MYREAL loggf (MYREAL r, MYREAL alpha, MYREAL theta1);
 void gamma_loci_derivative (helper_fmt * helper);
 void gamma_locus_derivative (helper_fmt * helper, long locus);
-//double gamma_theta1_derivative (double theta, double *param, nr_fmt * nr,
+//MYREAL gamma_theta1_derivative (MYREAL theta, MYREAL *param, nr_fmt * nr,
 //                              long locus, int *sign);
-//double gamma_other_derivative (long which, double theta, double *param,
+//MYREAL gamma_other_derivative (long which, MYREAL theta, MYREAL *param,
 //                             nr_fmt * nr, long locus, int *sign);
-//double gamma_alpha_derivative (double theta, double *param, nr_fmt * nr,
+//MYREAL gamma_alpha_derivative (MYREAL theta, MYREAL *param, nr_fmt * nr,
 //                             long locus, int *sign);
-void gamma_parts_gradient (double x, double *param, nr_fmt * nr, long locus);
-double dt_locus_gsum (long which, double x, double *param, nr_fmt * nr,
+void gamma_parts_gradient (MYREAL x, MYREAL *param, nr_fmt * nr, long locus);
+MYREAL dt_locus_gsum (long which, MYREAL x, MYREAL *param, nr_fmt * nr,
                       timearchive_fmt ** atl, long locus);
 
 
-double
-gamma_loci_like(helper_fmt *helper, double *oparam, double *olparam, double denom)
+MYREAL
+gamma_loci_like(helper_fmt *helper, MYREAL *oparam, MYREAL *olparam, MYREAL denom)
 {
     long locus;
-    double summ = 0.0;
+    MYREAL summ = 0.0;
     nr_fmt *nr = helper->nr;
     for (locus = 0; locus < nr->world->loci; locus++)
     {
@@ -57,23 +66,23 @@ gamma_loci_like(helper_fmt *helper, double *oparam, double *olparam, double deno
 }
 
 
-double find_gamma_summ(double la, double theta1, nr_fmt *nr, double *oparam, double *olparam,
+MYREAL find_gamma_summ(MYREAL la, MYREAL theta1, nr_fmt *nr, MYREAL *oparam, MYREAL *olparam,
                        long locus, helper_fmt *helper);
 
 
-double
-gamma_loci_like_iterative(helper_fmt * helper, double *oparam, double *olparam, double denom)
+MYREAL
+gamma_loci_like_iterative(helper_fmt * helper, MYREAL *oparam, MYREAL *olparam, MYREAL denom)
 {
     // hack to get around the problem of not being able to get a clean derivative of alpha
     // uses a simple maxima finder for alpha
     long locus=helper->nr->world->locus;
-    double fa = 0.0;
-    double fla, fha;
-    double la  = -10;
-    double ha = +4;
-    double a;
-    double fm, m;
-    double theta1 = helper->nr->param[0];
+    MYREAL fa = 0.0;
+    MYREAL fla, fha;
+    MYREAL la  = -10;
+    MYREAL ha = +4;
+    MYREAL a;
+    MYREAL fm, m;
+    MYREAL theta1 = helper->nr->param[0];
     nr_fmt *nr = helper->nr;
     char save_a = nr->world->options->custm[nr->world->numpop2];
     //  if(save_a == 'c')
@@ -106,7 +115,7 @@ gamma_loci_like_iterative(helper_fmt * helper, double *oparam, double *olparam,
         fa = find_gamma_summ( m,  theta1, nr,  oparam, olparam,  locus, helper);
 
     }
-    nr->param[nr->numpop2] = exp(m);
+    nr->param[nr->numpop2] = EXP(m);
     nr->lparam[nr->numpop2] = m;
     nr->world->options->custm[nr->world->numpop2]=save_a;
     nr->world->options->custm2[nr->world->numpop2]=save_a;
@@ -114,20 +123,20 @@ gamma_loci_like_iterative(helper_fmt * helper, double *oparam, double *olparam,
 }
 
 
-double
-gamma_loci_like_new_notworking(helper_fmt * helper, double *oparam, double *olparam, double denom)
+MYREAL
+gamma_loci_like_new_notworking(helper_fmt * helper, MYREAL *oparam, MYREAL *olparam, MYREAL denom)
 {
     long locus=0;
     long trials=0;
-    double logalpha= -DBL_MAX;
-    double psiabc;
-    double theta1 = helper->nr->param[0];
+    MYREAL logalpha= -DBL_MAX;
+    MYREAL psiabc;
+    MYREAL theta1 = helper->nr->param[0];
     nr_fmt *nr = helper->nr;
     char save_a = nr->world->options->custm[nr->world->numpop2];
 
-    double a, b, c,  m=0.5;
-    double la, lb, lc, ll=0, ql;
-    //double lm;
+    MYREAL a, b, c,  m=0.5;
+    MYREAL la, lb, lc, ll=0, ql;
+    //MYREAL lm;
     a= 0.00000000001;
     b= 1.0;
     c=2.0;
@@ -165,7 +174,7 @@ gamma_loci_like_new_notworking(helper_fmt * helper, double *oparam, double *olpa
                 break;
             }
             ll = find_gamma_summ( m,  theta1, nr,  oparam, olparam,  locus, helper);
-            replace_with (m, &a, &b, &c, m, &la, &lb, &lc, ll);
+            replace_with ((int) m, &a, &b, &c, m, &la, &lb, &lc, ll);
         }
         ll = find_gamma_summ( logalpha,  theta1, nr,  oparam, olparam,  locus, helper);
         ql = quadratic_lamda (logalpha, a, b, c, la, lb, lc);
@@ -176,26 +185,26 @@ gamma_loci_like_new_notworking(helper_fmt * helper, double *oparam, double *olpa
         }
         else
         {
-            replace_with (1., &a, &b, &c, logalpha, &la, &lb, &lc,
+            replace_with (1, &a, &b, &c, logalpha, &la, &lb, &lc,
                           ll);
             m = MYMAX3 (a, b, c);
         }
     }
-    nr->param[nr->numpop2] = exp(logalpha);
+    nr->param[nr->numpop2] = EXP(logalpha);
     nr->lparam[nr->numpop2] = logalpha;
     nr->world->options->custm[nr->world->numpop2]=save_a;
     nr->world->options->custm2[nr->world->numpop2]=save_a;
     return ll;
 }
 
-double find_gamma_summ(double lla, double theta1, nr_fmt *nr, double *oparam, double *olparam,
+MYREAL find_gamma_summ(MYREAL lla, MYREAL theta1, nr_fmt *nr, MYREAL *oparam, MYREAL *olparam,
                        long locus, helper_fmt *helper)
 {
-    double la = exp(lla);
-    double denom =  -LGAMMA(la) - la * log(theta1/la);
-    double summ=0.0;
-    double so = oparam[nr->numpop2];
-    double slo = olparam[nr->numpop2];
+    MYREAL la = EXP(lla);
+    MYREAL denom =  -LGAMMA(la) - la * log(theta1/la);
+    MYREAL summ=0.0;
+    MYREAL so = oparam[nr->numpop2];
+    MYREAL slo = olparam[nr->numpop2];
     oparam[nr->numpop2] = la;
     olparam[nr->numpop2] = lla;
     if (olparam[nr->numpop2] > 9.903487553)
@@ -217,18 +226,18 @@ double find_gamma_summ(double lla, double theta1, nr_fmt *nr, double *oparam, do
     return summ;
 }
 
-double
-gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam, double denom,
+MYREAL
+gamma_locus_like (nr_fmt * nr, MYREAL *oparam, MYREAL *olparam, MYREAL denom,
                   long locus)
 {
     //  nr_fmt *nr = helper->nr;
     long r;
-    double tempmax = -DBL_MAX;
-    double *temp;
-    double summ = 0.;
-    double alpha = oparam[nr->numpop2];
-    double theta1 = oparam[0];
-    temp = (double *) calloc (GAMMA_INTERVALS, sizeof (double));
+    MYREAL tempmax = -DBL_MAX;
+    MYREAL *temp;
+    MYREAL summ = 0.;
+    MYREAL alpha = oparam[nr->numpop2];
+    MYREAL theta1 = oparam[0];
+    temp = (MYREAL *) mycalloc (GAMMA_INTERVALS, sizeof (MYREAL));
     for (r = 0; r < GAMMA_INTERVALS; r++)
     {
         set_gamma_param (nr->param, oparam,
@@ -246,8 +255,8 @@ gamma_locus_like (nr_fmt * nr, double *oparam, double *olparam, double denom,
     return log (summ) + tempmax;
 }
 
-double
-loggf (double r, double alpha, double theta1)
+MYREAL
+loggf (MYREAL r, MYREAL alpha, MYREAL theta1)
 {
     return -r * alpha / theta1 + log (r) * (alpha - 1.);
 }
@@ -258,21 +267,21 @@ gamma_loci_difference (helper_fmt * helper)
     long i;
     nr_fmt *nr = helper->nr;
     long size = nr->partsize;
-    double mle, high, low, param1;
-    double *lparam;
-    double *param;
-    double *xv = helper->xv;
-    double *expxv = helper->expxv;
-    double epsilon = EPSILON;
-    //  double logepsilon = log(epsilon);
-    memset (nr->d, 0, size * sizeof (double));
+    MYREAL mle, high, low, param1;
+    MYREAL *lparam;
+    MYREAL *param;
+    MYREAL *xv = helper->xv;
+    MYREAL *expxv = helper->expxv;
+    MYREAL epsilon = EPSILON;
+    //  MYREAL logepsilon = log(epsilon);
+    memset (nr->d, 0, size * sizeof (MYREAL));
     setdoublevec1d (&lparam, helper->xv, size);
     setdoublevec1d (&param, helper->expxv, size);
     mle = gamma_loci_like (helper, param, lparam, helper->weight);
     for (i = 0; i < size; i++)
     {
-        memcpy (param, expxv, sizeof (double) * size);
-        memcpy (lparam, xv, sizeof (double) * size);
+        memcpy (param, expxv, sizeof (MYREAL) * size);
+        memcpy (lparam, xv, sizeof (MYREAL) * size);
         param1 = (param[i] += epsilon);
         lparam[i] = log (param[i]);
         high = gamma_loci_like (helper, param, lparam, helper->weight);
@@ -297,7 +306,7 @@ void
 gamma_loci_derivative (helper_fmt * helper)
 {
     long locus;
-    memset (helper->nr->d, 0, helper->nr->partsize * sizeof (double));
+    memset (helper->nr->d, 0, helper->nr->partsize * sizeof (MYREAL));
     for (locus = 0; locus < helper->nr->world->loci; locus++)
     {
         if (helper->nr->skiploci[locus])
@@ -311,27 +320,27 @@ void
 gamma_locus_derivative (helper_fmt * helper, long locus)
 {
     long r, i, ii, z;
-    double **temp, *tempmax;
-    double ll;
-    double sumi;
+    MYREAL **temp, *tempmax;
+    MYREAL ll;
+    MYREAL sumi;
     int **stemp;
-    //double *summll;
-    //double summllsum;
-    //double summllmax;
-    double rate;
-    double lograte;
-    double tmp;
-    double denom = helper->weight;
+    //MYREAL *summll;
+    //MYREAL summllsum;
+    //MYREAL summllmax;
+    MYREAL rate;
+    MYREAL lograte;
+    MYREAL tmp;
+    MYREAL denom = helper->weight;
     nr_fmt *nr = helper->nr;
-    double alpha = helper->expxv[nr->numpop2];
-    double theta1 = helper->expxv[0];
+    MYREAL alpha = helper->expxv[nr->numpop2];
+    MYREAL theta1 = helper->expxv[0];
     long partsize = nr->partsize;
-    //summll = (double *) calloc(GAMMA_INTERVALS, sizeof(double));
-    tempmax = (double *) calloc (partsize, sizeof (double));
-    temp = (double **) calloc (partsize, sizeof (double *));
-    temp[0] = (double *) calloc (partsize * GAMMA_INTERVALS, sizeof (double));
-    stemp = (int **) calloc (partsize, sizeof (int *));
-    stemp[0] = (int *) calloc (partsize * GAMMA_INTERVALS, sizeof (int));
+    //summll = (MYREAL *) mycalloc(GAMMA_INTERVALS, sizeof(MYREAL));
+    tempmax = (MYREAL *) mycalloc (partsize, sizeof (MYREAL));
+    temp = (MYREAL **) mycalloc (partsize, sizeof (MYREAL *));
+    temp[0] = (MYREAL *) mycalloc (partsize * GAMMA_INTERVALS, sizeof (MYREAL));
+    stemp = (int **) mycalloc (partsize, sizeof (int *));
+    stemp[0] = (int *) mycalloc (partsize * GAMMA_INTERVALS, sizeof (int));
     tempmax[0] = -DBL_MAX;
     for (r = 1; r < partsize; r++)
     {
@@ -384,7 +393,7 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
         denom = -LGAMMA(alpha) - alpha * log(theta1/alpha);
         for (r = 0; r < GAMMA_INTERVALS; r++)
             sumi +=
-                ((double) stemp[i][r]) * EXP (temp[i][r] -
+                ((MYREAL) stemp[i][r]) * EXP (temp[i][r] -
                                               tempmax[i]  +  denom);
         if (i == 0)
         {
@@ -412,26 +421,26 @@ gamma_locus_derivative (helper_fmt * helper, long locus)
 }
 
 void
-gamma_parts_gradient (double x, double *param, nr_fmt * nr, long locus)
+gamma_parts_gradient (MYREAL x, MYREAL *param, nr_fmt * nr, long locus)
 {
     long g;
 
-    //  double copysum=0;
+    //  MYREAL copysum=0;
     long pop, i;
-    double expapg;  //, summ;
-    //double summm = 0.0;
+    MYREAL expapg;  //, summ;
+    //MYREAL summm = 0.0;
     long msta, msto;
-    double waitmig;
-    double pointmig;
+    MYREAL waitmig;
+    MYREAL pointmig;
     tarchive_fmt *tl;
-    double *part;   //, df = 0.0;
-    //  double value ;
-    double theta1 = param[0];
-    double *geo = nr->world->data->geo;
+    MYREAL *part;   //, df = 0.0;
+    //  MYREAL value ;
+    MYREAL theta1 = param[0];
+    MYREAL *geo = nr->world->data->geo;
     long rep;
-    double theta_ratio = theta1 / x;
-    part = (double *) malloc (sizeof (double) * nr->partsize);
-    memset (nr->parts, 0, sizeof (double) * nr->partsize);
+    MYREAL theta_ratio = theta1 / x;
+    part = (MYREAL *) mymalloc (sizeof (MYREAL) * nr->partsize);
+    memset (nr->parts, 0, sizeof (MYREAL) * nr->partsize);
     for (rep = nr->repstart; rep < nr->repstop; rep++)
     {
         for (g = 0; g < nr->atl[rep][locus].T; g++)
