@@ -4,17 +4,25 @@
 part of migrate 
 Peter Beerli 2000
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
 This software is distributed free of charge for non-commercial use
 and is copyrighted. Of course, we do not guarantee that the software
 works and are not responsible for any damage you may cause or have.
  
-$Id: heating.c,v 1.17 2003/11/20 19:49:04 beerli Exp $
+$Id: heating.c,v 1.23 2004/08/28 19:59:31 beerli Exp $
  
 */
+/*! \file heating.c 
+
+Heating handler for threaded heating
+
+*/
+
 #include "migration.h"
 #include "mcmc.h"
+#include "sighandler.h"
 
 #if PTHREADS   /* ==================threaded version======== */
 
@@ -25,11 +33,12 @@ $Id: heating.c,v 1.17 2003/11/20 19:49:04 beerli Exp $
 void fill_tpool (tpool_t tpool, world_fmt ** universe, int universe_size);
 void tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size,
                  int do_not_block_when_full);
-void thread_tree_update (void *world);
+void thread_tree_update (void *thisworld);
 int tpool_destroy (tpool_t tpool, int finish);
 void tpool_thread (tpool_t tpool);
 int tpool_synchronize (tpool_t tpool, int finish);
 
+extern void run_one_update(world_fmt *world); //defined in main.c
 
 void
 tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size,
@@ -38,13 +47,13 @@ tpool_init (tpool_t * tpoolp, int num_worker_threads, int max_queue_size,
     int i, rtn;
     tpool_t tpool;
 
-    tpool = (tpool_t) malloc (sizeof (struct _tpool_t));
+    tpool = (tpool_t) mymalloc (sizeof (struct _tpool_t));
 
     tpool->num_threads = num_worker_threads;
     tpool->max_queue_size = max_queue_size;
     tpool->do_not_block_when_full = do_not_block_when_full;
     tpool->threads =
-        (pthread_t *) malloc (sizeof (pthread_t) * num_worker_threads);
+        (pthread_t *) mymalloc (sizeof (pthread_t) * num_worker_threads);
 
     tpool->cur_queue_size = 0;
     tpool->queue_head = NULL;
@@ -99,7 +108,7 @@ tpool_add_work (tpool_t tpool, void *routine, void *arg)
         return -1;
     }
 
-    workp = (tpool_work_t *) malloc (sizeof (tpool_work_t));
+    workp = (tpool_work_t *) mymalloc (sizeof (tpool_work_t));
     workp->routine = routine;
     workp->arg = arg;
     workp->next = NULL;
@@ -118,13 +127,16 @@ tpool_add_work (tpool_t tpool, void *routine, void *arg)
     return 1;
 }
 
-void
-fill_tpool (tpool_t tpool, world_fmt ** universe, int universe_size)
+/// \brief put work for heated chains on to the heated-chain-stack
+///
+/// queue the work needed to be done into the stack of heated chains
+/// if there is an error the routien will abort with an 
+/// error "filling heating queue failed"
+void fill_tpool (tpool_t tpool, world_fmt ** universe, int universe_size)
 {
     int i;
     for (i = 0; i < universe_size; i++)
     {
-        //     printf("\nqueued: %s ",universe[i]->name);
         if (!tpool_add_work (tpool, thread_tree_update, (void *) universe[i]))
             error ("Filling heating queue failed");
     }
@@ -172,28 +184,21 @@ tpool_thread (tpool_t tpool)
     }
 }
 
-void
-thread_tree_update (void *world)
+/// \brief updates the heated tree and records acceptance
+///
+/// updates the tree during threaded heating scheme 
+void thread_tree_update (void *thisworld)
 {
     long i;
-    world_fmt *thisworld = (world_fmt *) world;
-    for (i = 0; i < thisworld->options->heating_interval; i++)
+    world_fmt * world = (world_fmt *) thisworld;
+    for (i = 0; i < world->options->heating_interval; i++)
     {
-#ifdef BAYESUPDATE
-        if(thisworld->options->bayes_infer)
-        {
-            thisworld->bayesaccept = bayes_update (thisworld);
-            thisworld->bayes->accept[world->numpop2] += thisworld->bayesaccept;   
-        }
-        else
-        {
-            thisworld->accept += tree_update (thisworld, thisworld->G);
-        }
-#endif /*not compiled for Bayesian choice*/
-        thisworld->accept += tree_update (thisworld, thisworld->G);
+		run_one_update(thisworld);
     }
 }
 
+///
+/// synchronizes the heated chains and controls the locking of chains
 int
 tpool_synchronize (tpool_t tpool, int fullnumber)
 {
@@ -284,10 +289,10 @@ tpool_destroy (tpool_t tpool, int finish)
 void adjust_temperatures(world_fmt ** universe, long hchains, long step, long steps)
 {
     long i;
-    const double bigger = 1.1;
-    const double smaller = 0.9;
-    const double corrsum = steps / HEATCHECKINTERVAL;
-    double *delta;
+    const MYREAL bigger = 1.1;
+    const MYREAL smaller = 0.9;
+    const MYREAL corrsum = steps / HEATCHECKINTERVAL;
+    MYREAL *delta;
     if(step == 0)
     {
         for(i=0; i< hchains; i++)
@@ -304,7 +309,7 @@ void adjust_temperatures(world_fmt ** universe, long hchains, long step, long st
         if ( (step % HEATCHECKINTERVAL ) == 0)
         {
             universe[0]->averageheat= 1.0;
-            delta = (double *) calloc(hchains,sizeof(double));
+            delta = (MYREAL *) mycalloc(hchains,sizeof(MYREAL));
 	    // FPRINTF(stdout,"\n%f %li\n",1./universe[0]->heat,universe[0]->treeswapcount);
             for(i=1; i< hchains; i++)
             {
