@@ -9,26 +9,28 @@
  Peter Beerli 2000, Seattle
  beerli@csit.fsu.edu
  
- Copyright 2001 Peter Beerli and Joseph Felsenstein
+ Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: lrt.c,v 1.26 2003/11/04 16:12:08 beerli Exp $
+$Id: lrt.c,v 1.36 2005/01/21 16:48:34 beerli Exp $
  
 -------------------------------------------------------*/
+/* \file lrt.c 
+Calculates likelihood ratio tests
+*/
 #define SICK_VALUE    -1
 #include "migration.h"
-//#include "mcmc.h"
-
-//#include "fst.h"
-//#include "random.h"
+#include "sighandler.h"
 #include "tools.h"
 #include "broyden.h"
 #include "combroyden.h"
 #include "options.h"
 #include "aic.h"
+#include "migrate_mpi.h"
 #ifndef LAGUERRE
 #include "derivatives2.h"
 #endif
@@ -42,25 +44,25 @@ $Id: lrt.c,v 1.26 2003/11/04 16:12:08 beerli Exp $
 #endif
 
 void print_lratio_test (world_fmt * world, long *Gmax);
-void test_loci_like (nr_fmt * nr, double *param0,
-                     double *param1, long df, long zeros,
+void test_loci_like (nr_fmt * nr, MYREAL *param0,
+                     MYREAL *param1, long df, long zeros,
                      long loci, world_fmt * world,
                      long *maxwhich, long maxnum,
                      boolean withhead, char *this_string1);
-long set_test_param (double *param, lr_data_fmt  *lrtdata, world_fmt * world,
+long set_test_param (MYREAL *param, lr_data_fmt  *lrtdata, world_fmt * world,
                      long lrline, long locus, long *maxwhich,
                      long *maxnum, long *zeros);
-void print_lrt_box(world_fmt *world, double *param0, double *param1,  long zeros, long elem,
-                   char * this_string1, double like0, double like1, double df);
+void print_lrt_box(world_fmt *world, MYREAL *param0, MYREAL *param1,  long zeros, long elem,
+                   char * this_string1, MYREAL like0, MYREAL like1, long df);
 
-long parse_h0(char ***box, long *size, char *thisString, double *param0, double *param1 , long elem, world_fmt *world);
+long parse_h0(char ***box, long *size, char *thisString, MYREAL *param0, MYREAL *param1 , long elem, world_fmt *world);
 void parse_h0part(char ***box, long *size, long *tempsize,
-                double *param, long elem, char head[], world_fmt *world);
+                MYREAL *param, long elem, char head[], world_fmt *world);
 void parse_h0string(char ***box, long *size, long *tempsize,
                     char *thisString);
 
 void print_box(world_fmt* world,char **box,long size,
-               double like0, double like1,double lrt, double chiprob, double chiprob2, double aic, long df, long aicparamnum);
+               MYREAL like0, MYREAL like1,MYREAL lrt, MYREAL chiprob, MYREAL chiprob2, MYREAL aic, long df, long aicparamnum);
 
 long simplify_lrtvalues(char * in, char*** out);
 
@@ -84,8 +86,8 @@ print_lratio_test (world_fmt * world, long *Gmax)
     long zeros;
     int header;
     nr_fmt *nr;
-    double *param0;
-    double *param1;
+    MYREAL *param0;
+    MYREAL *param1;
     long *maxwhich;
     long nparam;
     long maxnum = 0;
@@ -93,15 +95,15 @@ print_lratio_test (world_fmt * world, long *Gmax)
                (world->options->replicatenum == 0 ?
                 world->options->lchains : world->options->replicatenum);
     long repstop = world->repstop;
-    param0 = (double *) calloc (1, sizeof (double) * (world->numpop2 + 1));
-    param1 = (double *) calloc (1, sizeof (double) * (world->numpop2 + 1));
-    maxwhich = (long *) calloc (1, sizeof (long) * (world->numpop2 + 1));
+    param0 = (MYREAL *) mycalloc (1, sizeof (MYREAL) * (world->numpop2 + 1));
+    param1 = (MYREAL *) mycalloc (1, sizeof (MYREAL) * (world->numpop2 + 1));
+    maxwhich = (long *) mycalloc (1, sizeof (long) * (world->numpop2 + 1));
 
 
     if (world->options->progress)
         FPRINTF (stdout, "           Printing likelihood ratio tests\n");
 
-    nr = (nr_fmt *) calloc (1, sizeof (nr_fmt));
+    nr = (nr_fmt *) mycalloc (1, sizeof (nr_fmt));
     create_nr (nr, world, *Gmax, 0, world->loci, world->repkind, world->rep);
     for (locus = 0; locus < world->loci; locus++)
     {
@@ -129,7 +131,7 @@ print_lratio_test (world_fmt * world, long *Gmax)
         if (world->options->lratio->data[c].type == MLE)
         {
             memcpy (param1, world->atl[rep][locus].param,
-                    sizeof (double) * nparam);
+                    sizeof (MYREAL) * nparam);
             df = set_test_param (param0,
                                  &world->options->lratio->data[c],
                                  world, 0, -1, maxwhich, &maxnum, &zeros);
@@ -146,192 +148,19 @@ print_lratio_test (world_fmt * world, long *Gmax)
     destroy_nr (nr, world);
 }
 
-void
-tell_chiprob2 (char *message, double zeros, double df, double chi)
-{
-    if (chi > chiboundary (zeros, df - zeros, 0.001))
-    {
-        if (chi > chiboundary (zeros, df - zeros, 0.0001))
-            strcpy (message, "<0.0001");
-        else
-            strcpy (message, "<0.001 ");
-    }
-    else
-    {
-        if (chi > chiboundary (zeros, df - zeros, 0.01))
-            strcpy (message, "<0.01  ");
-        else
-        {
-            if (chi > chiboundary (zeros, df - zeros, 0.05))
-                strcpy (message, "<0.05  ");
-            else
-                strcpy (message, ">0.05  ");
-        }
-    }
-}
-
-/* obsolete?
-void
-test_locus_like (nr_fmt * nr, double *param0, double *param1, long df,
-                 long zeros, long locus, world_fmt * world, long *maxwhich,
-                 long maxnum, boolean withhead, char *this_string)
-{
-    char *teststat, temp[LRATIO_STRINGS];
-    double like1, like0, testval, chiprob, chiprob2;
-    int length;
-    char tmp[LRATIO_STRINGS];
-    long zi, i, part1len = 1, part2len = 1;
-    long elem, w = 0, z = 0, pop;
-    double normd = 0;
-    long *which;
-    double  *saveparam0, tparam;
-    double *lparam0;
-    double *lparam1;
-    long spaces = 0;
-    char spacer[100];
-    char *message;
-    helper_fmt helper;
-    message = (char *) calloc (20, sizeof (char));
-    which = (long *) calloc (1, sizeof (long) * (world->numpop2 + 1));
-    lparam0 = (double *) calloc (world->numpop2 + 1, sizeof (double));
-    lparam1 = (double *) calloc (world->numpop2 + 1, sizeof (double));
-    saveparam0 = (double *) malloc (sizeof (double) * (world->numpop2 + 1));
-    memcpy (saveparam0, param0, sizeof (double) * (world->numpop2 + 1));
-    nr->skiploci = world->data->skiploci;
-    if (world->options->gamma)
-        elem = (nr->numpop2 = world->numpop2) + 1;
-    else
-        elem = nr->numpop2 = world->numpop2;
-    nr->numpop = world->numpop;
-    if (maxnum > 0)
-    {
-        for (i = 0; i < elem; i++)
-        {
-            if (i != maxwhich[z])
-            {
-                which[w] = i;
-                nr->values[w++] = param0[i];
-            }
-            else
-            {
-                if (maxnum > z + 1)
-                    z++;
-            }
-        }
-        nr->profilenum = w;
-        maximize (param0, world, nr, PROFILE, world->repkind);
-        like0 = nr->llike;
-        normd = nr->normd;
-        memcpy (param0, world->param0, sizeof (double) * nr->partsize);
-    }
-    else
-    {
-        set_logparam (lparam0, param0, elem);
-        fill_helper (&helper, param0, lparam0, world, nr);
-        like0 = CALCLIKE (&helper, FALSE, FALSE);
-    }
-    set_logparam (lparam1, param1, elem);
-    fill_helper (&helper, param1, lparam1, world, nr);
-    like1 = CALCLIKE (&helper, param1, lparam1);
-    testval = -2. * (like0 - like1);
-    chiprob = probchi (df, testval);
-    //tell_chiprob2(message, zeros, df, testval);
-    chiprob2 = probchiboundary (testval, zeros, df);
-    sprintf (message, " %f ", chiprob2);
-    teststat = (char *) calloc (1, sizeof (char) * LRATIO_STRINGS);
-    length = MAX (0, 5 - sprintf (tmp, "%i", ((int) param0[0])));
-    sprintf (teststat, "Loc%3li: %.*f", locus, length, param0[0]);
-    for (i = 1; i < elem; i++)
-    {
-        zi = mml2m (i, world->numpop);
-        if (zi >= world->numpop)
-        {
-            pop = (zi - world->numpop) / (world->numpop - 1);
-            tparam = param0[zi] * saveparam0[pop];
-        }
-        else
-            tparam = param0[zi];
-        length = MAX (0, 5 - sprintf (tmp, "%i", ((int) tparam)));
-        sprintf (temp, " %.*f", length, tparam);
-        strcat (teststat, temp);
-        if ((i + 1) % world->numpop == 0)
-            strcat (teststat, "\n        ");
-    }
-    sprintf (temp, "=assumed to be equal to=");
-    strcat (teststat, temp);
-    part1len = 43;  //strlen (teststat);
-    length = MAX (0, 5 - sprintf (tmp, "%i", ((int) param1[0])));
-    sprintf (temp, "\n        %.*f", length, param1[0]);
-    strcat (teststat, temp);
-    for (i = 1; i < elem; i++)
-    {
-        zi = mml2m (i, world->numpop);
-        if (zi >= world->numpop)
-        {
-            pop = (zi - world->numpop) / (world->numpop - 1);
-            tparam = param1[zi] * param1[pop];
-        }
-        else
-            tparam = param1[zi];
-        length = MAX (0, 5 - sprintf (tmp, "%i", ((int) tparam)));
-        sprintf (temp, " %.*f", length, tparam);
-        strcat (teststat, temp);
-        if ((i + 1) % world->numpop == 0)
-            strcat (teststat, "\n       ");
-    }
-    //  sprintf (temp, "}");
-    //  strcat (teststat, temp);
-    part2len = 43;  // strlen (teststat);
-    spaces = 43 - 5 * (1 + world->numpop);
-    strcpy (spacer, "                          ");
-
-    if (withhead)
-    {
-        FPRINTF (world->outfile,
-                 "==============================================================================\n");
-        FPRINTF (world->outfile, "Likelihood ratio tests\n");
-        FPRINTF (world->outfile,
-                 "==============================================================================\n");
-        FPRINTF (world->outfile, "Over all loci\n");
-        FPRINTF (world->outfile,
-                 "[Theta values are on the diagonal of the Migration matrix,\nmigration rates are specified as Theta*M]\n");
-        part1len = MAX (part1len, part2len - part1len);
-        FPRINTF (world->outfile,
-                 "%*.*s           %-10.10s   %-3.3s   %-4.4s %-5.5s\n",
-                 (int) part1len, (int) part1len, " ", "Test value", "Df",
-                 "Prob", "Prob*");
-        for (i = 0; i < part1len + 35; i++)
-            fputc ('-', world->outfile);
-        FPRINTF (world->outfile, "\n");
-    }
-    if (testval > 10000 && chiprob < EPSILON)
-        sprintf (temp, ">10000.00");
-    else
-        sprintf (temp, "%10.2f", testval);
-    FPRINTF (world->outfile, "You entered\n[%s]\n", this_string);
-    FPRINTF (world->outfile, "%s %s -2(%6.3f-%6.3f)=%11.11s %3li %8.5f %s\n",
-             teststat, spacer, like0, like1, temp, df, chiprob, message);
-    free (teststat);
-    free (lparam0);
-    free (lparam1);
-    free (saveparam0);
-    free (which);
-    free (message);
-}
-*/
 
 //remember: param0 is the parameterset to test and
 // NOT the parameterset from migrate.
 #define BOXSIZE 50 /*defines the print width of the left box containing the H0*/
 #define BOXSIZE2 40 /*defines the print width of the left box with the legend*/
 void
-test_loci_like (nr_fmt * nr, double *param0, double *param1, long df,
+test_loci_like (nr_fmt * nr, MYREAL *param0, MYREAL *param1, long df,
                 long zeros, long loci, world_fmt * world, long *maxwhich,
                 long maxnum, boolean withhead, char *this_string1)
 {
     
  //   char *teststat, temp[LRATIO_STRINGS];
-    double like1, like0;//, testval, chiprob, chiprob2;
+    MYREAL like1, like0;//, testval, chiprob, chiprob2;
  //   int length;
  //  
     long i, j, g = 0;
@@ -339,25 +168,25 @@ test_loci_like (nr_fmt * nr, double *param0, double *param1, long df,
     //long zi;
     long z = 0, w = 0;
     // long pop;
-    double normd = 0;
+    MYREAL normd = 0;
     long *which;
-    double /**values,*/ *saveparam0;
-    //double tparam;
+    MYREAL /**values,*/ *saveparam0;
+    //MYREAL tparam;
 //    long spaces = 0;
     char temp[100];
     helper_fmt helper;
-    double *lparam0;
-    double *lparam1;
-    double aicfull;
+    MYREAL *lparam0;
+    MYREAL *lparam1;
+    MYREAL aicfull;
     long aicfullparamnum;
 //    char *message;
-    lparam0 = (double *) calloc (world->numpop2 + 1, sizeof (double));
-    lparam1 = (double *) calloc (world->numpop2 + 1, sizeof (double));
-    which = (long *) calloc (1, sizeof (long) * (world->numpop2 + 1));
-//    message = (char *) calloc (20, sizeof (char));
-    //  values = (double *) calloc (1, sizeof (double) * (world->numpop2 + 1));
-    saveparam0 = (double *) malloc (sizeof (double) * (world->numpop2 + 1));
-    memcpy (saveparam0, param0, sizeof (double) * (world->numpop2 + 1));
+    lparam0 = (MYREAL *) mycalloc (world->numpop2 + 1, sizeof (MYREAL));
+    lparam1 = (MYREAL *) mycalloc (world->numpop2 + 1, sizeof (MYREAL));
+    which = (long *) mycalloc (1, sizeof (long) * (world->numpop2 + 1));
+//    message = (char *) mycalloc (20, sizeof (char));
+    //  values = (MYREAL *) mycalloc (1, sizeof (MYREAL) * (world->numpop2 + 1));
+    saveparam0 = (MYREAL *) mymalloc (sizeof (MYREAL) * (world->numpop2 + 1));
+    memcpy (saveparam0, param0, sizeof (MYREAL) * (world->numpop2 + 1));
     for (i = 0; i < loci; i++)
     {
         for (j = 0; j < world->repstop; j++)
@@ -385,10 +214,10 @@ test_loci_like (nr_fmt * nr, double *param0, double *param1, long df,
             }
         }
         nr->profilenum = w;
-        maximize (param0, world, nr, PROFILE, world->repkind);
+        maximize (&param0, world, nr, PROFILE, world->repkind);
         like0 = nr->llike;
         normd = nr->normd;
-        memcpy (param0, world->param0, sizeof (double) * nr->partsize);
+        memcpy (param0, world->param0, sizeof (MYREAL) * nr->partsize);
     }
     else
     {
@@ -443,8 +272,8 @@ test_loci_like (nr_fmt * nr, double *param0, double *param1, long df,
     free (which);    
 }
 
-void print_lrt_box(world_fmt *world, double *param0, double *param1, long zeros, long elem,
-                   char * this_string1, double like0, double like1, double df)
+void print_lrt_box(world_fmt *world, MYREAL *param0, MYREAL *param1, long zeros, long elem,
+                   char * this_string1, MYREAL like0, MYREAL like1, long df)
 {
     // Alternative plot
     //
@@ -457,29 +286,27 @@ void print_lrt_box(world_fmt *world, double *param0, double *param1, long zeros,
     //---------------------------------------------------------------------
     //
     // LRT
-    double testval = -2. * (like0 - like1);
+    MYREAL testval = -2. * (like0 - like1);
     // standard probability assuming indpendence and range of -inf .. +inf 
-    double chiprob = probchi (df, testval);
+    MYREAL chiprob = probchi (df, testval);
     // probability assuming independence and range of 0 .. inf
-    double chiprob2 = probchiboundary (testval, zeros, df);
+    MYREAL chiprob2 = probchiboundary (testval, zeros, df);
     // AIC value penalizing for number of paramters
     long aicparamnum = find_paramnum(world,this_string1);
-    double aic = -2. * like0 + 2. * aicparamnum;
+    MYREAL aic = -2. * like0 + 2. * aicparamnum;
 
     char **box;
-    long size = HUNDRED; // we need at least 7 lines to print 
-    //the right side of the table, but for some large dataset this breaks
-    //sloppy fix, in version 1.8 this needs to be fixed correctly
-    //the reallocation algorithm seems to miss lines
+    long size = HUNDRED; // we need at least 7 lines to print the right side of the table
+    /// \todo  reallocation of likelihood ratio boxes needs reevaluation
     long newsize;
     long i;
     
     if(world->options->progress)
         print_line(stdout,'-',79,CONT);
     print_line(world->outfile,'-',79,CONT);
-    box = (char **) calloc (size, sizeof (char *));
+    box = (char **) mycalloc (size, sizeof (char *));
     for(i=0; i<size; i++)
-        box[i] = (char *) calloc(LRATIO_STRINGS,sizeof(char));
+        box[i] = (char *) mycalloc(LRATIO_STRINGS,sizeof(char));
     newsize = parse_h0(&box,&size, this_string1,param0,param1,elem,world);
     print_box(world,box,newsize,like0,like1,testval, chiprob, chiprob2, aic, df, aicparamnum);
     if(world->options->progress)
@@ -490,7 +317,7 @@ void print_lrt_box(world_fmt *world, double *param0, double *param1, long zeros,
     free(box);
 }
 
-long parse_h0(char ***box, long *size, char *thisString, double *param0, double *param1 , long elem, world_fmt *world)
+long parse_h0(char ***box, long *size, char *thisString, MYREAL *param0, MYREAL *param1 , long elem, world_fmt *world)
 {
     long tempsize=0;
     parse_h0part(box, size, &tempsize, param0, elem, "H0:", world);
@@ -502,13 +329,13 @@ long parse_h0(char ***box, long *size, char *thisString, double *param0, double
 }
 
 void parse_h0part(char ***box, long *size, long *tempsize,
-                double *param, long elem, char head[], world_fmt *world)
+                MYREAL *param, long elem, char head[], world_fmt *world)
 {
     int length;
     long i;
     long zi;
     long pop;
-    double tparam;
+    MYREAL tparam;
     char tmp[LRATIO_STRINGS];
     long counter = 0;
     length = (int) MAX (0, 5 - sprintf (tmp, "%i", ((int) param[0])));
@@ -533,8 +360,8 @@ void parse_h0part(char ***box, long *size, long *tempsize,
             (*tempsize)++;
             if(*tempsize>= *size-1)
               {
-                *box = (char **) realloc(*box,sizeof(char*) * (*tempsize+1));
-                (*box)[*tempsize] = (char *) calloc(LRATIO_STRINGS,sizeof(char));
+                *box = (char **) myrealloc(*box,sizeof(char*) * (*tempsize+1));
+                (*box)[*tempsize] = (char *) mycalloc(LRATIO_STRINGS,sizeof(char));
                 *size = *tempsize+1; 
               }
             counter = sprintf((*box)[*tempsize],"%3.3s","   ");
@@ -547,7 +374,7 @@ void parse_h0string(char ***box, long *size, long *tempsize,
 {
     long i;
     long counter = 0;
-    long strsize = strlen(thisString);
+    long strsize = (long) strlen(thisString);
     counter = sprintf ((*box)[*tempsize],"[");
     for (i = 0; i < strsize; i++)
       {
@@ -557,8 +384,8 @@ void parse_h0string(char ***box, long *size, long *tempsize,
             (*tempsize)++;
             if(*tempsize>= *size-1)
               {
-                *box = (char **) realloc(*box,sizeof(char*) * (*tempsize+1));
-                (*box)[*tempsize] = (char *) calloc(LRATIO_STRINGS,sizeof(char));
+                *box = (char **) myrealloc(*box,sizeof(char*) * (*tempsize+1));
+                (*box)[*tempsize] = (char *) mycalloc(LRATIO_STRINGS,sizeof(char));
                 *size = *tempsize+1;
               }
             counter = sprintf((*box)[*tempsize]," ");
@@ -568,7 +395,7 @@ void parse_h0string(char ***box, long *size, long *tempsize,
 }
 
 void print_box(world_fmt* world,char **box,long size,
-               double like0, double like1,double lrt, double chiprob, double chiprob2, double aic, long df, long aicparamnum)
+               MYREAL like0, MYREAL like1,MYREAL lrt, MYREAL chiprob, MYREAL chiprob2, MYREAL aic, long df, long aicparamnum)
 {
     long i;
     FPRINTF(world->outfile,"%-*.*s | LnL(test) = %f\n",BOXSIZE,BOXSIZE, box[0],like0);
@@ -596,100 +423,9 @@ void print_box(world_fmt* world,char **box,long size,
     }
 }
 
-/*
-    teststat = (char *) calloc (1, sizeof (char) * LRATIO_STRINGS);
-    length = MAX (0, 5 - sprintf (tmp, "%i", ((int) param0[0])));
-    sprintf (teststat, "H0:%.*f", length, param0[0]);
-    for (i = 1; i < elem; i++)
-    {
-        zi = mml2m (i, world->numpop);
-        if (zi >= world->numpop)
-        {
-            pop = (zi - world->numpop) / (world->numpop - 1);
-            tparam = param0[zi] * param0[pop];
-        }
-        else
-            tparam = param0[zi];
-        length = MAX (0, 5 - sprintf (tmp, "%i", ((int) tparam)));
-        sprintf (temp, " %.*f", length, tparam);
-        strcat (teststat, temp);
-        if ((i + 1) % world->numpop == 0)
-            strcat (teststat, "\n  ");
-    }
-    sprintf (temp, "=assumed to be equal to=    -2(%6.3f-%6.3f)=", like0,
-             like1);
-    strcat (teststat, temp);
-    if (testval > 10000 && chiprob < EPSILON)
-        sprintf (temp, ">10000.00  %3li %8.5f %s\n", df, chiprob, message);
-    else
-        sprintf (temp, "%-9.2f  %3li %8.5f %s\n", testval, df, chiprob, message);
-    strcat (teststat, temp);
-    part1len = 43;  //strlen (teststat);
-    length = MAX (0, 5 - sprintf (tmp, "%i", ((int) param1[0])));
-    sprintf (temp, "   %.*f", length, param1[0]);
-    strcat (teststat, temp);
-    for (i = 1; i < elem; i++)
-    {
-        zi = mml2m (i, world->numpop);
-        if (zi >= world->numpop)
-        {
-            pop = (zi - world->numpop) / (world->numpop - 1);
-            tparam = param1[zi] * param1[pop];
-        }
-        else
-            tparam = param1[zi];
-        length = MAX (0, 5 - sprintf (tmp, "%i", ((int) tparam)));
-        sprintf (temp, " %.*f", length, tparam);
-        strcat (teststat, temp);
-        if ((i + 1) % world->numpop == 0)
-            strcat (teststat, "\n  ");
-    }
-    //  sprintf (temp, "}");
-    part2len = 43;  //strlen (teststat);
-    spaces = 43 - 5 * (1 + world->numpop);
-
-    strcpy (spacer, "                          ");
-
-    if (withhead)
-    {
-        FPRINTF (world->outfile,
-                 "==============================================================================\n");
-        FPRINTF (world->outfile, "Likelihood ratio tests\n");
-        FPRINTF (world->outfile,
-                 "==============================================================================\n");
-        FPRINTF (world->outfile, "Over all loci\n");
-        FPRINTF (world->outfile,
-                 "[Theta values are on the diagonal of the Migration matrix,\nmigration rates are specified as Theta*M]\n");
-        part1len = MAX (part1len, part2len - part1len);
-        FPRINTF (world->outfile,
-                 "%*.*s      %-10.10s   %-3.3s   %-4.4s %-5.5s\n",
-                 (int) part1len, (int) part1len, " ", "Test value", "Df",
-                 "Prob", "Prob*");
-        for (i = 0; i < part1len + 35; i++)
-            fputc ('-', world->outfile);
-        FPRINTF (world->outfile, "\n");
-    }
-
-    if (testval > 10000 && chiprob < EPSILON)
-        sprintf (temp, ">10000.00");
-    else
-        sprintf (temp, "%10.2f", testval);
-    FPRINTF (world->outfile, "You entered\n[%s]\n", this_string1);
-    //FPRINTF (world->outfile, "%s %s -2(%6.3f-%6.3f)=%11.11s %3li %8.5f %s\n", teststat, spacer, like0, like1, temp, df, chiprob, message);
-    FPRINTF (world->outfile, "%s\n", teststat);
-    free (teststat);
-    free (lparam0);
-    free (lparam1);
-    free (saveparam0);
-    free (which);
-    free (message);
-}
-*/
-
-
 
 long
-set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrline,
+set_test_param (MYREAL *param, lr_data_fmt *lrtdata, world_fmt * world, long lrline,
                 long locus, long *maxwhich, long *maxnum, long *zeros)
 {
     long elem = world->options->gamma ? world->numpop2 + 1 : world->numpop2;
@@ -700,22 +436,22 @@ set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
     char *paramtype;
     char *ss;
     char **custm;
-    double *meanparam;
+    MYREAL *meanparam;
     long zzz;
     long zi;
     long el;
     long df = 0;
     long count=0;
-    double mean=0.;
+    MYREAL mean=0.;
     long offset;
     long limit;
     long zz;
     long numpop = world->numpop;
     long pop1, pop2;
     
-    paramtype = (char *) calloc (1, sizeof (char) * elem);
-    ss = (char *) calloc (LONGLINESIZE, sizeof (char));
-    custm = (char **) calloc (LONGLINESIZE, sizeof (char *));
+    paramtype = (char *) mycalloc (1, sizeof (char) * elem);
+    ss = (char *) mycalloc (LONGLINESIZE, sizeof (char));
+    custm = (char **) mycalloc (LONGLINESIZE, sizeof (char *));
 
     strcpy (ss, lrtdata->value1);
 
@@ -724,7 +460,7 @@ set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
     *zeros = 0;
     
     if(elem != elements)
-        warning("Not enough elements in the l-ratio specification");
+        warning("Not enough elements in the l-ratio specification\n");
 
     zzz=0;
     if (world->loci - world->skipped > 1)
@@ -853,7 +589,7 @@ set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
                 paramtype[zi] = '+';
                 if(custm[el][0] != world->options->custm2[zi])
                     df++;
-                if (custm[el][0] == '0' && strlen(custm[el])==1)
+                if (custm[el][0] == '0' && (long) strlen(custm[el])==1)
                 {
                     if(custm[el][0] != world->options->custm2[zi])
                         (*zeros)++;
@@ -890,7 +626,7 @@ long simplify_lrtvalues(char * in, char*** out)
 {
     long elements=0;
     long count = 0;
-    (*out)[0] = (char *) calloc(LONGLINESIZE,sizeof(char));
+    (*out)[0] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
     while(*in!='\0')
       {
         switch(*in)
@@ -904,7 +640,7 @@ long simplify_lrtvalues(char * in, char*** out)
             case ';':
                 count=0;
                 elements++;
-                (*out)[elements] = (char *) calloc(LONGLINESIZE,sizeof(char));
+                (*out)[elements] = (char *) mycalloc(LONGLINESIZE,sizeof(char));
                 break;
             default:
                 (*out)[elements][count++] = *in;
@@ -912,13 +648,13 @@ long simplify_lrtvalues(char * in, char*** out)
           }
         in++;
     }
-    *out = (char**) realloc(*out, elements * sizeof(char*));
+    *out = (char**) myrealloc(*out, elements * sizeof(char*));
     return elements;
 }
 
 #if 0
 long
-set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrline,
+set_test_param (MYREAL *param, lr_data_fmt *lrtdata, world_fmt * world, long lrline,
                 long locus, long *maxwhich, long *maxnum, long *zeros)
 {
     long i = 0, z = 0, zi = 0, zz = 0, zzz = 0, df = 0;
@@ -926,16 +662,16 @@ set_test_param (double *param, lr_data_fmt *lrtdata, world_fmt * world, long lrl
     long offset = 0, limit = 0, pop, pop1, pop2;
     long numpop = world->numpop;
     char *tmp, *tmp2, *ss, *custm, *paramtype;
-    double *meanparam, mean;
+    MYREAL *meanparam, mean;
     long elem = world->options->gamma ? world->numpop2 + 1 : world->numpop2;
     long repstop = !world->options->replicate ? 0 :
                    (world->options->replicatenum == 0 ?
                     world->options->lchains : world->options->replicatenum);
     *zeros = 0;
-    ss = (char *) calloc (1, sizeof (char) * LOGLINESIZE);
-    custm = (char *) calloc (1, sizeof (char) * LONGLINESIZE);
-    tmp = (char *) calloc (1, sizeof (char) * LONGLINESIZE);
-    paramtype = (char *) calloc (1, sizeof (char) * elem);
+    ss = (char *) mycalloc (1, sizeof (char) * LOGLINESIZE);
+    custm = (char *) mycalloc (1, sizeof (char) * LONGLINESIZE);
+    tmp = (char *) mycalloc (1, sizeof (char) * LONGLINESIZE);
+    paramtype = (char *) mycalloc (1, sizeof (char) * elem);
 
     *maxnum = 0;
     strcpy (ss, lrtdata->value1);
