@@ -11,15 +11,23 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc1.c,v 1.67 2003/11/04 16:12:09 beerli Exp $
+$Id: mcmc1.c,v 1.73 2004/11/16 02:54:30 beerli Exp $
 -------------------------------------------------------*/
+/* \file mcmc1.c
+
+Tree changer and acceptance rejection scheme
+
+*/
 #include "migration.h"
+#include "sighandler.h"
+#include "tools.h"
 #include "random.h"
 #include "tree.h"
 #include "mcmc2.h"
@@ -73,13 +81,13 @@ int pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
 
 boolean acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
                     timelist_fmt * tyme);
-double eventtime (proposal_fmt * proposal, long pop, vtlist * tentry,
+MYREAL eventtime (proposal_fmt * proposal, long pop, vtlist * tentry,
                   char *event);
 node *showsister (node * theNode);
 void count_migrations (node * p, long *count);
 long migration_from (long to, proposal_fmt * proposal);
 
-double prob_tree (world_fmt * world, timelist_fmt * tyme);
+MYREAL prob_tree (world_fmt * world, timelist_fmt * tyme);
 
 /* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
 void
@@ -116,7 +124,7 @@ tree_update (world_fmt * world, long g)
     long slider;
     long bordernum;
     long actualpop = -99, zz;
-    double endtime, nexttime, age;
+    MYREAL endtime, nexttime, age;
 #ifdef UEP
 
     boolean uepsuccess = FALSE;
@@ -179,7 +187,7 @@ tree_update (world_fmt * world, long g)
             {
                 if (!migrate (proposal, proposal->origin))
                 {
-		  //                    free_proposal (proposal);
+                    free_proposal (proposal);
                     free_timevector (timevector);
                     return 0;
                 }
@@ -209,12 +217,8 @@ tree_update (world_fmt * world, long g)
         if (!pre_population
                 (proposal, (*timevector).tl, (*timevector).T - 1, &slider))
         {
-	  //            free_proposal (proposal);
+            free_proposal (proposal);
             free_timevector (timevector);
-#ifdef BAYESUPDATE
-	    if(world->options->verbose)
-	      FPRINTF(stdout,"   \n");
-#endif
             return 0;
         }
         pretendcoalesce1p (proposal);
@@ -253,25 +257,17 @@ tree_update (world_fmt * world, long g)
 #endif
         /* create a new timelist */
         construct_tymelist (world, &world->treetimes[0]);
-        if (world->options->treeprint != NONE)
+        if (world->options->treeprint != NONE && (world->heat == 1.0))
             print_tree (world, g, &treefilepos);
         world->migration_counts = 0;
         /* report the number of migration on the tree */
         count_migrations (world->root->next->back, &world->migration_counts);
         free_proposal (proposal);
         free_timevector (timevector);
-#ifdef BAYESUPDATE
-	    if(world->options->verbose)
-	      FPRINTF(stdout,"***\n");
-#endif
         return 1;   /* new tree accepted */
     }
     free_proposal (proposal);
     free_timevector (timevector);
-#ifdef BAYESUPDATE
-	    if(world->options->verbose)
-	      FPRINTF(stdout,"   \n");
-#endif
     return 0;   /* not accepted */
 
 }
@@ -283,14 +279,14 @@ void
 new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
 {
     long i;
-    (*ntl) = (timelist_fmt *) calloc (1, sizeof (timelist_fmt));
-    (*ntl)->tl = (vtlist *) malloc ((*otl).allocT * sizeof (vtlist));
+    (*ntl) = (timelist_fmt *) mycalloc (1, sizeof (timelist_fmt));
+    (*ntl)->tl = (vtlist *) mymalloc ((*otl).allocT * sizeof (vtlist));
     (*ntl)->allocT = otl->allocT;
     (*ntl)->T = otl->T;
     memcpy ((*ntl)->tl, otl->tl, otl->allocT * sizeof (vtlist));
     for (i = 0; i < (*ntl)->allocT; i++)
     {
-        (*ntl)->tl[i].lineages = (long *) calloc (numpop, sizeof (long));
+        (*ntl)->tl[i].lineages = (long *) mycalloc (numpop, sizeof (long));
         memcpy ((*ntl)->tl[i].lineages, otl->tl[i].lineages,
                 numpop * sizeof (long));
     }
@@ -301,13 +297,15 @@ new_localtimelist (timelist_fmt ** ntl, timelist_fmt * otl, long numpop)
 void
 new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
 {
+#ifdef UEP    
     long j;
+#endif
     long mal = world->data->maxalleles[world->locus];
     long listsize = ((*tl).allocT + (*tl).T + 5);
     long sumtips = world->sumtips;
 
     // allocate the scratchpad (contains a shadow of the tree)
-    (*proposal) = (proposal_fmt *) calloc (1, sizeof (proposal_fmt));
+    (*proposal) = (proposal_fmt *) mycalloc (1, sizeof (proposal_fmt));
 
     // pointers and values to outside structures
     (*proposal)->world = world;
@@ -327,7 +325,7 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
 
     // node data holding vector for aboveorigin, bordernodes, line_f, line_t
     (*proposal)->nodedata =
-        (node **) calloc (2 * listsize + 2 * sumtips, sizeof (node *));
+        (node **) mycalloc (2 * listsize + 2 * sumtips, sizeof (node *));
     // nodes above the picked node
     (*proposal)->aboveorigin = (*proposal)->nodedata;
     // adjacent nodes of the pcked nodes
@@ -337,55 +335,45 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
     (*proposal)->line_t =  (*proposal)->nodedata + listsize + 2 * sumtips;
 
     // mf holds also mt array  
-    (*proposal)->mf = (double *) calloc (2* (*proposal)->endsite, sizeof (double));
+    (*proposal)->mf = (MYREAL *) mycalloc (2* (*proposal)->endsite, sizeof (MYREAL));
     (*proposal)->mt = (*proposal)->mf + (*proposal)->endsite;
 
     if (strchr (SEQUENCETYPES, (*proposal)->datatype))
     {
-        (*proposal)->xf.s =
-            (phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
-        (*proposal)->xt.s =
-            (phenotype) malloc (world->data->seq->endsite * sizeof (ratelike *));
-        for (j = 0; j < (*proposal)->endsite; j++)
-        {
-            (*proposal)->xf.s[j] =
-                (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
-            (*proposal)->xt.s[j] =
-                (ratelike) malloc (world->options->rcategs * sizeof (sitelike));
-        }
+        allocate_xseq(&(*proposal)->xf, world->data->seq->endsite, world->options->rcategs);
+        allocate_xseq(&(*proposal)->xt, world->data->seq->endsite, world->options->rcategs);
     }
     else
     {
-        (*proposal)->xf.a = (double *) calloc (1, sizeof (double) * mal);
-        (*proposal)->xt.a = (double *) calloc (1, sizeof (double) * mal);
+        (*proposal)->xf.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
+        (*proposal)->xt.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
     }
     (*proposal)->old_migr_table_counter = 4 * sumtips /* 100 */ ;
     (*proposal)->old_migr_table_counter2 = 4 * sumtips /* 100 */ ;
     (*proposal)->migr_table =
-        (migr_table_fmt *) calloc (1,
+        (migr_table_fmt *) mycalloc (1,
                                    sizeof (migr_table_fmt) *
                                    (*proposal)->old_migr_table_counter);
     (*proposal)->migr_table2 =
-        (migr_table_fmt *) calloc ((*proposal)->old_migr_table_counter2,
+        (migr_table_fmt *) mycalloc ((*proposal)->old_migr_table_counter2,
                                    sizeof (migr_table_fmt));
     (*proposal)->migr_table_counter = 0;
     (*proposal)->migr_table_counter2 = 0;
 #ifdef UEP
-
     if (world->options->uep)
     {
         (*proposal)->ueplike =
-            (double **) calloc (world->data->uepsites, sizeof (double *));
+            (MYREAL **) mycalloc (world->data->uepsites, sizeof (MYREAL *));
         (*proposal)->ueplike[0] =
-            (double *) calloc (world->numpop * world->data->uepsites,
-                               sizeof (double));
+            (MYREAL *) mycalloc (world->numpop * world->data->uepsites,
+                               sizeof (MYREAL));
         for (j = 1; j < world->data->uepsites; ++j)
             (*proposal)->ueplike[j] = (*proposal)->ueplike[0] + j * world->numpop;
 
-        (*proposal)->ut.s = (pair *) calloc (world->data->uepsites, sizeof (pair));
-        (*proposal)->uf.s = (pair *) calloc (world->data->uepsites, sizeof (pair));
-        (*proposal)->umt = (double *) calloc (world->data->uepsites, sizeof (double));
-        (*proposal)->umf = (double *) calloc (world->data->uepsites, sizeof (double));
+        (*proposal)->ut.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
+        (*proposal)->uf.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
+        (*proposal)->umt = (MYREAL *) mycalloc (world->data->uepsites, sizeof (MYREAL));
+        (*proposal)->umf = (MYREAL *) mycalloc (world->data->uepsites, sizeof (MYREAL));
     }
 #endif
 }
@@ -398,7 +386,7 @@ jumblenodes (node ** s, long n)
 
     long i, rr, tn = n;
 
-    temp = (node **) calloc (1, sizeof (node *) * n);
+    temp = (node **) mycalloc (1, sizeof (node *) * n);
     memcpy (temp, s, sizeof (node *) * n);
     for (i = 0; i < n && tn > 0; i++)
     {
@@ -415,7 +403,7 @@ chooseOrigin (proposal_fmt * proposal)
 {
     long elem = 0, oldelem = (proposal->sumtips * 2);
     node *tmp, **goal;
-    goal = (node **) calloc (1, sizeof (node *) * oldelem);
+    goal = (node **) mycalloc (1, sizeof (node *) * oldelem);
     traverseAllNodes (crawlback (proposal->root->next), &goal, &elem, &oldelem,
                       NO_MIGR_NODES);
     tmp = goal[RANDINT (0, elem - 2)];
@@ -485,7 +473,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
             {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) realloc ((*nodelist), sizeof (node *) * elem);
+                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
@@ -502,7 +490,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
             {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) realloc ((*nodelist), sizeof (node *) * elem);
+                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
@@ -525,7 +513,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
             {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) realloc ((*nodelist), sizeof (node *) * elem);
+                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
@@ -538,7 +526,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
             {
                 elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
                 (*nodelist) =
-                    (node **) realloc ((*nodelist), sizeof (node *) * elem);
+                    (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
                         sizeof (node *) * (elem - (*oldnode_elem)));
                 *oldnode_elem = elem;
@@ -639,7 +627,7 @@ migrate (proposal_fmt * proposal, node * up)
     {
         proposal->old_migr_table_counter += 10;
         proposal->migr_table =
-            (migr_table_fmt *) realloc (proposal->migr_table,
+            (migr_table_fmt *) myrealloc (proposal->migr_table,
                                         sizeof (migr_table_fmt) *
                                         (proposal->old_migr_table_counter));
     }
@@ -668,7 +656,7 @@ migrateb (proposal_fmt * proposal, node * up)
     {
         proposal->old_migr_table_counter2 += 10;
         proposal->migr_table2 =
-            (migr_table_fmt *) realloc (proposal->migr_table2,
+            (migr_table_fmt *) myrealloc (proposal->migr_table2,
                                         sizeof (migr_table_fmt) *
                                         (proposal->old_migr_table_counter2));
     }
@@ -726,7 +714,7 @@ migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter,
         if (air)
         {
             proposal->migr_table =
-                (migr_table_fmt *) realloc (proposal->migr_table,
+                (migr_table_fmt *) myrealloc (proposal->migr_table,
                                             sizeof (migr_table_fmt) *
                                             (*old_migr_table_counter));
             array = proposal->migr_table;
@@ -734,7 +722,7 @@ migrate_old (proposal_fmt * proposal, node * up, long *old_migr_table_counter,
         else
         {
             proposal->migr_table2 =
-                (migr_table_fmt *) realloc (proposal->migr_table2,
+                (migr_table_fmt *) myrealloc (proposal->migr_table2,
                                             sizeof (migr_table_fmt) *
                                             (*old_migr_table_counter));
             array = proposal->migr_table;
@@ -759,9 +747,9 @@ long
 migration_from_old (long to, proposal_fmt * proposal)
 {
     long j, ii, msta, msto;
-    double *geo = proposal->world->data->geo;
-    double *r, rr = RANDUM ();
-    r = (double *) calloc (1, sizeof (double) * proposal->numpop);
+    MYREAL *geo = proposal->world->data->geo;
+    MYREAL *r, rr = RANDUM ();
+    r = (MYREAL *) mycalloc (1, sizeof (MYREAL) * proposal->numpop);
     msta = mstart (to, proposal->numpop);
     msto = mend (to, proposal->numpop);
     r[0] = proposal->param0[msta] * geo[msta];
@@ -785,8 +773,8 @@ long
 migration_from (long to, proposal_fmt * proposal)
 {
     long ii = 0;
-    double *r = proposal->world->migproblist[to];
-    double rr = RANDUM ();
+    MYREAL *r = proposal->world->migproblist[to];
+    MYREAL rr = RANDUM ();
     while (rr > r[ii])
     {
         ii++;
@@ -879,7 +867,7 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
 
 /*
    boolean
-   same_pop(node * up, double tyme, long pop)
+   same_pop(node * up, MYREAL tyme, long pop)
    {
    node *oldnn = showtop(up->back);
    node *nn = up;
@@ -906,7 +894,7 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
     long pop1 = -99, pop2 = -98;
     //  long msta1=0, msto1=0;
     //  long msta2=0, msto2=0;
-    double age1, denom, rr, r0, r1, horizon, mm, mm2;
+    MYREAL age1, denom, rr, r0, r1, horizon, mm, mm2;
     if (gte > 0)
         proposal->realtarget = ltime[gte - 1].eventnode;
     else
@@ -1072,21 +1060,12 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
 void
 free_proposal (proposal_fmt * proposal)
 {
-    long j;
-    free(proposal->nodedata);
-   // free (proposal->aboveorigin);
-    // free (proposal->bordernodes);
-    // free (proposal->line_f);
-    // free (proposal->line_t);
-    free (proposal->mf);
-   // free (proposal->mt);
+    free(proposal->nodedata); // ..line_t are also freed with this
+    free (proposal->mf); // proposal->mt is also freed with this
     if (strchr (SEQUENCETYPES, proposal->datatype))
     {
-        for (j = 0; j < proposal->endsite; j++)
-        {
-            free (proposal->xt.s[j]);
-            free (proposal->xf.s[j]);
-        }
+        free (proposal->xf.s[0]);
+        free (proposal->xt.s[0]);
         free (proposal->xf.s);
         free (proposal->xt.s);
     }
@@ -1140,8 +1119,8 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
 
     boolean report = TRUE;
     long oldg = -1;
-    //double zz;
-    double rr, expo;
+    //MYREAL zz;
+    MYREAL rr, expo;
     long rm;
     long rmc = rmigrcount (proposal);
     //zz = world->heat;
@@ -1172,8 +1151,6 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
                        proposal->world->data->uepsites);
         proposal->ueplikelihood = pseudo_ueplikelihood (world, proposal);
         proposal->likelihood = pseudotreelikelihood (world, proposal);
-        //     printf("%s> pl=%f pul=%f\n",proposal->world->name,proposal->likelihood,proposal->ueplikelihood);
-        // proposal->likelihood += proposal->ueplikelihood;
     }
     else
     {
@@ -1219,13 +1196,13 @@ rmigrcount (proposal_fmt * proposal)
     return count;
 }
 
-double
+MYREAL
 eventtime (proposal_fmt * proposal, long pop, vtlist * tentry, char *event)
 {
     static boolean mig_force = TRUE;
-    double interval, lines, denom;
-    //double rate = proposal->world->options->mu_rates[proposal->world->locus];
-    double mm = proposal->mig0list[pop];
+    MYREAL interval, lines, denom;
+    //MYREAL rate = proposal->world->options->mu_rates[proposal->world->locus];
+    MYREAL mm = proposal->mig0list[pop];
     long design0 = proposal->design0list[pop];
     lines = 2.0 * tentry->lineages[pop];
     denom = mm + (lines / proposal->param0[pop]);
@@ -1319,11 +1296,11 @@ count_migrations (node * p, long *count)
     }
 }
 
-double
+MYREAL
 prob_tree (world_fmt * world, timelist_fmt * tyme)
 {
     long j, pop;
-    double mm, cc, ss = 0;
+    MYREAL mm, cc, ss = 0;
 
     tyme->tl[0].interval = tyme->tl[0].age;
     for (j = 1; j < tyme->T; j++)
