@@ -11,18 +11,25 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc2.c,v 1.23 2003/11/04 16:12:09 beerli Exp $
+$Id: mcmc2.c,v 1.27 2004/08/28 19:59:32 beerli Exp $
 -------------------------------------------------------*/
+/* \file mcmc2.c
+Contains low-level routines to manipulate the treechanger
+*/
 
 #include "migration.h"
+#include "sighandler.h"
 #include "tree.h"
 #include "uep.h"
+#include "tools.h"
+#include "migrate_mpi.h"
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -62,18 +69,18 @@ void target_stancoa (proposal_fmt * proposal);
 void t_s_upper (proposal_fmt * proposal, node * connect, node * obb);
 node *t_s_obbncon (proposal_fmt * proposal, node * obb);
 void t_s_tncon (proposal_fmt * proposal, node * obb);
-void t_s_tcon (proposal_fmt * proposal, node * upper);
+void t_s_tcon (proposal_fmt * proposal, node * uppern);
 long erase_migr_nodes (node * up);
 node *findcrossing (node ** ptrl1, node ** ptrl2);
 void connectnodes (node * mother, node * brother, node * sister);
 void gotoroot (node * origin, node ** ptrlist);
-void adjust (node * theNode, double tyme, long level);
+void adjust (node * theNode, MYREAL tyme, long level);
 void localevaluate (node * mother);
 void copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2);
 void fix_root_pop (node * p);
-void pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, double *lx,
-                     node * mother, node * newdaughter, double v);
-node *crawl_down (node * theNode, double tyme);
+void pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
+                     node * mother, node * newdaughter, MYREAL v);
+node *crawl_down (node * theNode, MYREAL tyme);
 
 
 /*=========================================================*/
@@ -149,7 +156,7 @@ coalesce1p (proposal_fmt * proposal)
     }
     set_pop (proposal->oback, proposal->origin->actualpop,
              proposal->origin->actualpop);
-    /*   free_migr_node(proposal->origin, crawlback(proposal->origin)); */
+  //  free_migr_node(proposal->origin, crawlback(proposal->origin));
     if (proposal->migr_table_counter > 0)
     {
         set_pop (proposal->oback,
@@ -374,7 +381,7 @@ target_oback (proposal_fmt * proposal)
 #endif
 
     memcpy (proposal->mf, proposal->origin->scale,
-            sizeof (double) * proposal->endsite);
+            sizeof (MYREAL) * proposal->endsite);
     proposal->v = (proposal->time - proposal->origin->tyme);
     proposal->vs = (proposal->time - proposal->osister->tyme);
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
@@ -396,7 +403,7 @@ target_ocousin (proposal_fmt * proposal)
 #endif
 
     memcpy (proposal->mf, proposal->origin->scale,
-            sizeof (double) * proposal->endsite);
+            sizeof (MYREAL) * proposal->endsite);
     copy_x (proposal, proposal->xt, proposal->ocousin->x);
 #ifdef UEP
 
@@ -405,7 +412,7 @@ target_ocousin (proposal_fmt * proposal)
 #endif
 
     memcpy (proposal->mt, proposal->ocousin->scale,
-            sizeof (double) * proposal->endsite);
+            sizeof (MYREAL) * proposal->endsite);
     proposal->v = (proposal->time - proposal->origin->tyme);
     proposal->vs = fabs (proposal->ocousin->tyme - proposal->time);
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
@@ -436,7 +443,7 @@ target_rbcoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mt, proposal->osister->scale,
-                sizeof (double) * proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->v);
@@ -453,7 +460,7 @@ target_rbcoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mf, proposal->origin->scale,
-                sizeof (double) * proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
     }
@@ -480,7 +487,7 @@ target_obbcoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mt, proposal->osister->scale,
-                sizeof (double) * proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->v);
@@ -494,7 +501,7 @@ target_obbcoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mf, proposal->origin->scale,
-                sizeof (double) * proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
         proposal->v = fabs (showtop (crawlback (obb))->tyme - proposal->time);
@@ -511,9 +518,9 @@ target_obbcoa (proposal_fmt * proposal)
 void
 target_stancoa (proposal_fmt * proposal)
 {
-    node *obb, *nn, *oldnn, *d1, *d2, *proposalack, *upper = NULL; /* o R */
+    node *obb, *nn, *oldnn, *d1, *d2, *proposalack, *uppern = NULL; /* o R */
     node **double_line;
-        double_line = (node **) calloc (2, sizeof (node *));
+        double_line = (node **) mycalloc (2, sizeof (node *));
     if (proposal->ocousin != NULL)
     {
         obb = showtop (crawlback (proposal->oback));
@@ -547,9 +554,9 @@ target_stancoa (proposal_fmt * proposal)
 #endif
 
                 memcpy (proposal->mt, proposal->osister->scale,
-                        sizeof (double) * proposal->endsite);
+                        sizeof (MYREAL) * proposal->endsite);
                 /*       if (obb!=proposal->connect) { */
-                upper = t_s_obbncon (proposal, obb);
+                uppern = t_s_obbncon (proposal, obb);
                 /*} */
                 proposal->v = proposal->time - proposal->origin->tyme;
                 copy_x (proposal, proposal->xf, proposal->origin->x);
@@ -560,14 +567,14 @@ target_stancoa (proposal_fmt * proposal)
 #endif
 
                 memcpy (proposal->mf, proposal->origin->scale,
-                        sizeof (double) * proposal->endsite);
+                        sizeof (MYREAL) * proposal->endsite);
                 if (proposal->target != proposal->connect)
                 {
                     t_s_tncon (proposal, obb);
                 }
                 else
                 {
-                    t_s_tcon (proposal, upper);
+                    t_s_tcon (proposal, uppern);
                 }
             }
         }
@@ -584,7 +591,7 @@ target_stancoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mf, proposal->origin->scale,
-                sizeof (double) * proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
         copy_x (proposal, proposal->xt, proposal->target->x);
 #ifdef UEP
 
@@ -593,8 +600,8 @@ target_stancoa (proposal_fmt * proposal)
 #endif
 
         memcpy (proposal->mt, proposal->target->scale,
-                sizeof (double) * proposal->endsite);
-        //CHECK HEREH      memcpy(proposal->mt,proposal->osister->scale, sizeof(double)*proposal->endsite);
+                sizeof (MYREAL) * proposal->endsite);
+        //CHECK HEREH      memcpy(proposal->mt,proposal->osister->scale, sizeof(MYREAL)*proposal->endsite);
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
         proposal->v =
@@ -639,7 +646,7 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
 #endif
 
     memcpy (proposal->mf, proposal->origin->scale,
-            sizeof (double) * proposal->endsite);
+            sizeof (MYREAL) * proposal->endsite);
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->target->x, proposal->target->scale, proposal->vs);
     nn = showtop (crawlback (proposal->target));
@@ -731,13 +738,13 @@ t_s_tncon (proposal_fmt * proposal, node * obb)
 }
 
 void
-t_s_tcon (proposal_fmt * proposal, node * upper)
+t_s_tcon (proposal_fmt * proposal, node * uppern)
 {
     node *obb, *d1, *d2;
     children (proposal->target, &d1, &d2);
-    if (d1 == upper)
+    if (d1 == uppern)
         d1 = d2;
-    proposal->vs = fabs (proposal->target->tyme - showtop (upper)->tyme);
+    proposal->vs = fabs (proposal->target->tyme - showtop (uppern)->tyme);
     pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs, d1->x, d1->scale, d1->v); /* eval target */
     proposal->vs = proposal->time - proposal->target->tyme;
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, proposal->xt, proposal->mt, proposal->vs); /*eval oback */
@@ -863,7 +870,7 @@ gotoroot (node * origin, node ** ptrlist)
 }
 
 void
-adjust (node * theNode, double tyme, long level)
+adjust (node * theNode, MYREAL tyme, long level)
 {
     if (level < 0 || theNode == NULL)
         return;
@@ -941,10 +948,13 @@ localevaluate (node * mother)
 void
 copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2)
 {
-
-    const size_t sitelikesize = sizeof (sitelike);
-    
-    long i, j;
+#ifdef ALTIVEC            
+    const size_t sitelikesize = sizeof (FloatVec);
+#else
+    const size_t sitelikesize = sizeof (sitelike);//*proposal->world->options->rcategs;
+#endif    
+        long i;
+        long j;
 
     switch (proposal->datatype)
     {
@@ -952,7 +962,7 @@ copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2)
     case 'b':
     case 'm':
         memcpy (xx1.a, xx2.a,
-                sizeof (double) *
+                sizeof (MYREAL) *
                 proposal->world->data->maxalleles[proposal->world->locus]);
         break;
     case 's':
@@ -963,7 +973,7 @@ copy_x (proposal_fmt * proposal, xarray_fmt xx1, xarray_fmt xx2)
         {
             for (j = 0; j < proposal->world->options->rcategs; j++)
             {
-                memcpy (xx1.s[i][j], xx2.s[i][j], sitelikesize);
+                memcpy (xx1.s[i], xx2.s[i], sitelikesize*proposal->world->options->rcategs);
             }
         }
         break;
@@ -992,8 +1002,8 @@ fix_root_pop (node * p)
    DOES NOT CHANGE THE TREE
  */
 void
-pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, double *lx,
-                node * mother, node * newdaughter, double v)
+pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
+                node * mother, node * newdaughter, MYREAL v)
 {
     node *nn = NULL, *d1 = NULL, *d2 = NULL, *oldnn = NULL;
 
@@ -1036,7 +1046,7 @@ findcrossing (node ** ptrl1, node ** ptrl2)
 }
 
 node *
-crawl_down (node * theNode, double tyme)
+crawl_down (node * theNode, MYREAL tyme)
 {
     node *otmp, *tmp = theNode->back;
 
