@@ -10,11 +10,17 @@
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-  $Id: mig-histogram.c,v 1.6 2003/11/04 16:12:09 beerli Exp $
+  $Id: mig-histogram.c,v 1.10 2005/01/21 16:48:35 beerli Exp $
  
  */
+/*! \file mig-histogram.c
+Calculation and printing of migration event histogram
+*/
 
 #include "mig-histogram.h"
+#include "sighandler.h"
+#include "migrate_mpi.h"
+
 void setup_mighist (world_fmt * world, option_fmt * options);
 void print_mighist (world_fmt * world);
 void
@@ -22,21 +28,21 @@ setup_plotfield (plotfield_fmt * plotfield, char thisplotype,
                  long xsize, long ysize, char xlabel[], char ylabel[],
                  char yflabel[], char title[], boolean print);
 
-long calc_migtable (double **migtable, histogram_fmt * histogram,
+long calc_migtable (MYREAL **migtable, histogram_fmt * histogram,
                     mighistloci_fmt * aa, world_fmt * world, long loci);
-double average (double *vec, long size, long *weight, double *se, long *n);
-double quantile (double *vec, long size, long *weight, double quantile);
+MYREAL average (MYREAL *vec, long size, long *weight, MYREAL *se, long *n);
+MYREAL calc_quantile (MYREAL *vec, long size, long *weight, MYREAL quantile);
 
 #define NBINS 30
 void
 print_histogram_ascii (FILE * out, histogram_fmt ** histogram,
                        plotfield_fmt ** plotfield, long loci, long nmigs,
-                       long bins, long *sum, double ***migtable);
+                       long bins, long *sum, MYREAL ***migtable);
 
 void
-prepare_hist (histogram_fmt * hist, double *time, long count, long *weight);
+prepare_hist (histogram_fmt * hist, MYREAL *mtime, long count, long *weight);
 
-void minmax (histogram_fmt * hist, double *tempmin, double *tempmax);
+void minmax (histogram_fmt * hist, MYREAL *tempmin, MYREAL *tempmax);
 
 
 void
@@ -53,17 +59,22 @@ print_mighist_file (FILE * mighist, world_fmt * world)
             //copies = aa->mighist[j].copies;
             for (i = 0; i < aa->mighist[j].migeventsize; i++)
             {
-                FPRINTF (mighist, "%f %f %f\n",
-                         aa->mighist[j].migevents[i][0],
-                         aa->mighist[j].migevents[i][1],
-                         aa->mighist[j].migevents[i][2]);
+                FPRINTF (mighist, "%c %25.20f %f %f %f T%010li L%010li\n",
+                         aa->mighist[j].migevents[i][1] == aa->mighist[j].migevents[i][2] ? 'c' : 'm', // abbreviation m or c
+                         aa->mighist[j].migevents[i][0],    // time 
+                         aa->mighist[j].migevents[i][1],    // from population 
+                         aa->mighist[j].migevents[i][2],    // to population 
+                         aa->mighist[j].migevents[i][3],    // sumtips
+                         j,                                 // tree number
+                         loc                                // locus
+                         );
             }
         }
     }
 }
 
 void
-calc_mighistvalues (world_fmt * world, double ***migtable,
+calc_mighistvalues (world_fmt * world, MYREAL ***migtable,
                     histogram_fmt ** histogram, long *total)
 {
     mighistloci_fmt *aa;
@@ -89,7 +100,7 @@ calc_mighistvalues (world_fmt * world, double ***migtable,
 }
 
 void
-print_mighist_output (FILE * out, world_fmt * world, double ***migtable,
+print_mighist_output (FILE * out, world_fmt * world, MYREAL ***migtable,
                       long *total)
 {
     long loc, p1;
@@ -162,7 +173,7 @@ print_mighist (world_fmt * world)
     FILE *out = world->outfile;
     long loci1 = world->loci == 1 ? 1 : world->loci + 1;
     long *total;
-    double ***migtable;
+    MYREAL ***migtable;
     //loci x numpop2 x {mean, median, se}
 
     plotfield_fmt **plotfield;
@@ -170,18 +181,18 @@ print_mighist (world_fmt * world)
     //only for overall loci
     if (world->options->mighist)
     {
-        total = (long *) calloc (loci1 + 1, sizeof (long));
-        plotfield = (plotfield_fmt **) calloc (loci1, sizeof (plotfield_fmt *));
-        migtable = (double ***) calloc (loci1, sizeof (double **));
-        histogram = (histogram_fmt **) calloc (loci1, sizeof (histogram_fmt *));
+        total = (long *) mycalloc (loci1 + 1, sizeof (long));
+        plotfield = (plotfield_fmt **) mycalloc (loci1, sizeof (plotfield_fmt *));
+        migtable = (MYREAL ***) mycalloc (loci1, sizeof (MYREAL **));
+        histogram = (histogram_fmt **) mycalloc (loci1, sizeof (histogram_fmt *));
         for (loc = 0; loc < loci1; ++loc)
         {
             plotfield[loc] =
-                (plotfield_fmt *) calloc (world->numpop2, sizeof (plotfield_fmt));
+                (plotfield_fmt *) mycalloc (world->numpop2, sizeof (plotfield_fmt));
             migtable[loc] =
-                (double **) calloc (world->numpop2, sizeof (double *));
+                (MYREAL **) mycalloc (world->numpop2, sizeof (MYREAL *));
             histogram[loc] =
-                (histogram_fmt *) calloc (world->numpop2, sizeof (histogram_fmt));
+                (histogram_fmt *) mycalloc (world->numpop2, sizeof (histogram_fmt));
             for (i = 0; i < world->numpop2; ++i)
             {
                 histogram[loc][i].time = NULL;
@@ -189,7 +200,7 @@ print_mighist (world_fmt * world)
             }
             for (i = 0; i < world->numpop2; ++i)
             {
-                migtable[loc][i] = (double *) calloc (6, sizeof (double));
+                migtable[loc][i] = (MYREAL *) mycalloc (6, sizeof (MYREAL));
             }
         }
 
@@ -257,10 +268,10 @@ setup_plotfield (plotfield_fmt * plotfield, char thisplotype,
     strncpy (plotfield->yaxis, ylabel, 254);
     strncpy (plotfield->yfaxis, yflabel, 254);
     strncpy (plotfield->title, title, 254);
-    plotfield->yfreq = (double *) calloc (ysize, sizeof (double));
-    plotfield->y = (long *) calloc (ysize, sizeof (long));
-    plotfield->data = (char **) malloc (sizeof (char *) * xsize);
-    plotfield->data[0] = (char *) calloc (xsize * (ysize + 1), sizeof (char));
+    plotfield->yfreq = (MYREAL *) mycalloc (ysize, sizeof (MYREAL));
+    plotfield->y = (long *) mycalloc (ysize, sizeof (long));
+    plotfield->data = (char **) mymalloc (sizeof (char *) * xsize);
+    plotfield->data[0] = (char *) mycalloc (xsize * (ysize + 1), sizeof (char));
     for (i = 1; i < xsize; i++)
     {
         plotfield->data[i] = plotfield->data[0] + i * (ysize + 1);
@@ -270,22 +281,22 @@ setup_plotfield (plotfield_fmt * plotfield, char thisplotype,
 
 
 long
-calc_migtable (double **migtable, histogram_fmt * histogram,
+calc_migtable (MYREAL **migtable, histogram_fmt * histogram,
                mighistloci_fmt * aa, world_fmt * world, long loci)
 {
     long p1, p2, pa, pb, i, j;
     long maxloci, locus;
     long copies;
     long n, total = 0, maxsize;
-    double se;
-    double ***migtime;
+    MYREAL se;
+    MYREAL ***migtime;
     long ***gencount;
     long **migcount;
     long **size;
-    migtime = (double ***) calloc (world->numpop, sizeof (double **));
-    gencount = (long ***) calloc (world->numpop, sizeof (long **));
-    migcount = (long **) calloc (world->numpop, sizeof (long *));
-    size = (long **) calloc (world->numpop, sizeof (long *));
+    migtime = (MYREAL ***) mycalloc (world->numpop, sizeof (MYREAL **));
+    gencount = (long ***) mycalloc (world->numpop, sizeof (long **));
+    migcount = (long **) mycalloc (world->numpop, sizeof (long *));
+    size = (long **) mycalloc (world->numpop, sizeof (long *));
     maxsize = 1;
     if (loci == 1)
         maxloci = 1;
@@ -301,14 +312,14 @@ calc_migtable (double **migtable, histogram_fmt * histogram,
     }
     for (p1 = 0; p1 < world->numpop; ++p1)
     {
-        migtime[p1] = (double **) calloc (world->numpop, sizeof (double *));
-        gencount[p1] = (long **) calloc (world->numpop, sizeof (long *));
-        migcount[p1] = (long *) calloc (world->numpop, sizeof (long));
-        size[p1] = (long *) calloc (world->numpop, sizeof (long));
+        migtime[p1] = (MYREAL **) mycalloc (world->numpop, sizeof (MYREAL *));
+        gencount[p1] = (long **) mycalloc (world->numpop, sizeof (long *));
+        migcount[p1] = (long *) mycalloc (world->numpop, sizeof (long));
+        size[p1] = (long *) mycalloc (world->numpop, sizeof (long));
         for (p2 = 0; p2 < world->numpop; ++p2)
         {
-            migtime[p1][p2] = (double *) calloc (maxsize, sizeof (double));
-            gencount[p1][p2] = (long *) calloc (maxsize, sizeof (long));
+            migtime[p1][p2] = (MYREAL *) mycalloc (maxsize, sizeof (MYREAL));
+            gencount[p1][p2] = (long *) mycalloc (maxsize, sizeof (long));
             size[p1][p2] = maxsize;
         }
     }
@@ -319,21 +330,21 @@ calc_migtable (double **migtable, histogram_fmt * histogram,
             copies = aa[locus].mighist[j].copies;
             for (i = 0; i < aa[locus].mighist[j].migeventsize; i++)
             {
-                p1 = aa[locus].mighist[j].migevents[i][1];
-                p2 = aa[locus].mighist[j].migevents[i][2];
+                p1 = (long) aa[locus].mighist[j].migevents[i][1];
+                p2 = (long) aa[locus].mighist[j].migevents[i][2];
                 if (migcount[p1][p2] >= size[p1][p2])
                 {
                     size[p1][p2] += 10;
                     gencount[p1][p2] =
-                        (long *) realloc (gencount[p1][p2],
+                        (long *) myrealloc (gencount[p1][p2],
                                           sizeof (long) * size[p1][p2]);
                     memset (gencount[p1][p2] + migcount[p1][p2], 0,
                             sizeof (long) * 10);
                     migtime[p1][p2] =
-                        (double *) realloc (migtime[p1][p2],
-                                            sizeof (double) * size[p1][p2]);
+                        (MYREAL *) myrealloc (migtime[p1][p2],
+                                            sizeof (MYREAL) * size[p1][p2]);
                     memset (migtime[p1][p2] + migcount[p1][p2], 0,
-                            sizeof (double) * 10);
+                            sizeof (MYREAL) * 10);
                 }
                 gencount[p1][p2][migcount[p1][p2]] += copies;
                 migtime[p1][p2][migcount[p1][p2]] +=
@@ -362,10 +373,10 @@ calc_migtable (double **migtable, histogram_fmt * histogram,
                                        gencount[p2][p1], &se, &n);
             migtable[pb][4] = se;
             migtable[pb][5] = n;
-            migtable[pa][3] = quantile (migtime[p1][p2],
+            migtable[pa][3] = calc_quantile (migtime[p1][p2],
                                         migcount[p1][p2],
                                         gencount[p1][p2], 0.5);
-            migtable[pb][3] = quantile (migtime[p2][p1],
+            migtable[pb][3] = calc_quantile (migtime[p2][p1],
                                         migcount[p2][p1],
                                         gencount[p2][p1], 0.5);
             prepare_hist (&histogram[pa], migtime[p1][p2], migcount[p1][p2],
@@ -377,7 +388,7 @@ calc_migtable (double **migtable, histogram_fmt * histogram,
     }
     for (p1 = 0; p1 < world->numpop2; p1++)
     {
-        total += migtable[p1][5];
+        total += (long) migtable[p1][5];
     }
     for (p1 = 0; p1 < world->numpop; ++p1)
     {
@@ -397,26 +408,26 @@ calc_migtable (double **migtable, histogram_fmt * histogram,
 }
 
 void
-prepare_hist (histogram_fmt * hist, double *time, long count, long *weight)
+prepare_hist (histogram_fmt * hist, MYREAL *mtime, long count, long *weight)
 {
     hist->count = count;
     if (hist->time == NULL)
-        hist->time = (double *) calloc (count + 1, sizeof (double));
+        hist->time = (MYREAL *) mycalloc (count + 1, sizeof (MYREAL));
     else
-        hist->time = realloc (hist->time, sizeof (double) * (count + 1));
+        hist->time = myrealloc (hist->time, sizeof (MYREAL) * (count + 1));
     if (hist->weight == NULL)
-        hist->weight = (long *) calloc (count + 1, sizeof (long));
+        hist->weight = (long *) mycalloc (count + 1, sizeof (long));
     else
-        hist->weight = realloc (hist->weight, sizeof (long) * (count + 1));
-    memcpy (hist->time, time, sizeof (double) * count);
+        hist->weight = myrealloc (hist->weight, sizeof (long) * (count + 1));
+    memcpy (hist->time, mtime, sizeof (MYREAL) * count);
     memcpy (hist->weight, weight, sizeof (long) * count);
 }
 
-double
-average (double *vec, long size, long *weight, double *se, long *n)
+MYREAL
+average (MYREAL *vec, long size, long *weight, MYREAL *se, long *n)
 {
     long i;
-    double mean, sum = 0., sum2 = 0.;
+    MYREAL mean, sum = 0., sum2 = 0.;
     long sumweight = 0;
     for (i = 0; i < size; ++i)
         sumweight += weight[i];
@@ -443,24 +454,24 @@ average (double *vec, long size, long *weight, double *se, long *n)
     return NOAVERAGE;
 }
 
-double
-quantile (double *vec, long size, long *weight, double quantile)
+MYREAL
+calc_quantile (MYREAL *vec, long size, long *weight, MYREAL quantile)
 {
     long i, j, z = 0;
-    double *tmp1;
-    double val;
+    MYREAL *tmp1;
+    MYREAL val;
     long sumweight = 0;
     for (i = 0; i < size; ++i)
         sumweight += weight[i];
 
-    tmp1 = (double *) calloc (sumweight + 1, sizeof (double));
+    tmp1 = (MYREAL *) mycalloc (sumweight + 1, sizeof (MYREAL));
 
     for (i = 0; i < size; ++i)
     {
         for (j = 0; j < weight[i]; ++j)
             tmp1[z++] = vec[i];
     }
-    qsort ((void *) tmp1, sumweight, sizeof (double), numcmp);
+    qsort ((void *) tmp1, sumweight, sizeof (MYREAL), numcmp);
     val = tmp1[(long) (sumweight * quantile)];
     free (tmp1);
     return val;
@@ -474,7 +485,7 @@ increase_mighist (mighistloci_fmt * mighistlocus)
     {
         mighistlocus->allocsize += DEFAULTALLOCSIZE;
         mighistlocus->mighist = (mighist_fmt *)
-                                realloc (mighistlocus->mighist,
+                                 myrealloc (mighistlocus->mighist,
                                          sizeof (mighist_fmt) * mighistlocus->allocsize);
         for (i = mighistlocus->allocsize - DEFAULTALLOCSIZE;
                 i < mighistlocus->allocsize; i++)
@@ -482,7 +493,7 @@ increase_mighist (mighistloci_fmt * mighistlocus)
             mighistlocus->mighist[i].migeventsize = 0;
             mighistlocus->mighist[i].allocsize = DEFAULTALLOCSIZE;
             mighistlocus->mighist[i].migevents =
-                (migevent_fmt *) calloc (DEFAULTALLOCSIZE, sizeof (migevent_fmt));
+                (migevent_fmt *) mycalloc (DEFAULTALLOCSIZE, sizeof (migevent_fmt));
         }
     }
 }
@@ -494,33 +505,34 @@ setup_mighist (world_fmt * world, option_fmt * options)
     long allocsize = DEFAULTALLOCSIZE;
     if (world->options->mighist)
     {
-        world->mighistloci = (mighistloci_fmt *) calloc (world->loci, sizeof (mighistloci_fmt));
+        world->mighistloci = (mighistloci_fmt *)
+                             mycalloc (world->loci, sizeof (mighistloci_fmt));
         world->mighistlocinum = 0;
         for (locus = 0; locus < world->loci; locus++)
         {
             world->mighistloci[locus].allocsize = allocsize;
             world->mighistloci[locus].mighistnum = 0;
             world->mighistloci[locus].mighist =
-                (mighist_fmt *) calloc (world->mighistloci[locus].allocsize,
+                (mighist_fmt *) mycalloc (world->mighistloci[locus].allocsize,
                                         sizeof (mighist_fmt));
             for (i = 0; i < allocsize; i++)
             {
                 world->mighistloci[locus].mighist[i].migeventsize = 0;
                 world->mighistloci[locus].mighist[i].allocsize = allocsize;
                 world->mighistloci[locus].mighist[i].migevents =
-                    (migevent_fmt *) calloc (allocsize, sizeof (migevent_fmt));
+                    (migevent_fmt *) mycalloc (allocsize, sizeof (migevent_fmt));
             }
         }
     }
 }
 
 void
-minmax (histogram_fmt * hist, double *tempmin, double *tempmax)
+minmax (histogram_fmt * hist, MYREAL *tempmin, MYREAL *tempmax)
 {
     long i;
-    double tmp1, tmp2;
-    double tmpmin = DBL_MAX;
-    double tmpmax = -DBL_MAX;
+    MYREAL tmp1, tmp2;
+    MYREAL tmpmin = DBL_MAX;
+    MYREAL tmpmax = -DBL_MAX;
 
     for (i = 0; i < hist->count; i++)
     {
@@ -537,22 +549,22 @@ minmax (histogram_fmt * hist, double *tempmin, double *tempmax)
 void
 print_histogram_ascii (FILE * out, histogram_fmt ** histogram,
                        plotfield_fmt ** plotfield, long loci, long nmigs,
-                       long bins, long *sum, double ***migtable)
+                       long bins, long *sum, MYREAL ***migtable)
 {
     long loc, i, j, z, zz;
-    double biggest = 0.;
-    double *binning;
-    double *binvec;
-    double tempmin = DBL_MAX;
-    double tempmax = -DBL_MAX;
-    double begin = DBL_MAX;
-    double end = -DBL_MAX;
-    double delta;
-    //double        sum = 0;
-    double time;
+    MYREAL biggest = 0.;
+    MYREAL *binning;
+    MYREAL *binvec;
+    MYREAL tempmin = DBL_MAX;
+    MYREAL tempmax = -DBL_MAX;
+    MYREAL begin = DBL_MAX;
+    MYREAL end = -DBL_MAX;
+    MYREAL delta;
+    //MYREAL        sum = 0;
+    MYREAL mtime;
     long weight;
-    binning = (double *) calloc (bins+1, sizeof (double));
-    binvec = (double *) calloc (bins+1, sizeof (double));
+    binning = (MYREAL *) mycalloc (bins+1, sizeof (MYREAL));
+    binvec = (MYREAL *) mycalloc (bins+1, sizeof (MYREAL));
 
     for (loc = 0; loc < loci; loc++)
     {
@@ -582,13 +594,13 @@ print_histogram_ascii (FILE * out, histogram_fmt ** histogram,
             if (migtable[loc][i][2] == NOAVERAGE)
                 continue;
             //no event for this migration i->j
-            memset (binvec, 0, sizeof (double) * (bins+1));
+            memset (binvec, 0, sizeof (MYREAL) * (bins+1));
             for (j = 0; j < histogram[loc][i].count; j++)
             {
-                time = histogram[loc][i].time[j];
+                mtime = histogram[loc][i].time[j];
                 weight = histogram[loc][i].weight[j];
                 z = 0;
-                while (time > binning[z] && z < bins)
+                while (mtime > binning[z] && z < bins)
                     z++;
                 binvec[z] += weight;
             }
