@@ -14,16 +14,17 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: random.c,v 1.37 2003/11/04 16:12:09 beerli Exp $
+$Id: random.c,v 1.40 2004/08/28 19:59:34 beerli Exp $
 -------------------------------------------------------*/
-
+/* \file random.c */
 #include "sighandler.h"
 #include "migration.h"
 #include "heating.h"
@@ -56,12 +57,26 @@ extern tpool_t heating_pool;
 void getseed (option_fmt * options);
 void swap_ptr (long **ptr, long **newptr);
 
-double randum (void);
+MYREAL randum (void);
 #ifdef PTHREADS
-double randum_thread (void);
+MYREAL randum_thread (void);
 #endif
+long random_integer(long low, long high);
 
+
+/// random integer
 //=============================================
+MYINLINE long random_integer(long low, long high)
+{
+    //Math.floor(Math.random()*(N-M+1))%(N-M+1)+M
+    long r;
+    long tt = high-low+1;
+    r =  low + (long) (RANDUM() * tt);
+    if(r<low || r>high)
+        error("F.. [no bad words] but this IS shit\n");
+    return r;
+}
+
 void
 getseed (option_fmt * options)
 {
@@ -126,7 +141,7 @@ getseed (option_fmt * options)
         while (options->inseed != 0);
         for (i = 0; i <= myID; i++)
           {
-            test = RANDINT (1, MAXLONG);
+            test = RANDINT (1, MAXLONG-1);
             //printf("     %i %li> %li\n", myID, i, test);
           }
         switch (test % 4)
@@ -149,11 +164,6 @@ getseed (option_fmt * options)
     FPRINTF (stdout, "%i> random number seed = %li\n", myID, options->inseed);
 #else
         mt_seed32new((unsigned long)options->inseed);
-       // for (i = 0; i <= myID; i++)
-       //   {
-       //     test = RANDINT (1, MAXLONG);
-            //printf("     %i %li> %li\n", myID, i, test);
-       //   }        
 #endif /*MERSENNE_TWISTER*/
 #endif /*MPI*/
 #endif /*SPRNG*/
@@ -193,7 +203,7 @@ swap_ptr (long **ptr, long **newptr)
 }
 
 #ifdef PTHREADS
-double
+MYREAL
 randum_thread (void)
 /* thread save random number generator */
 {
@@ -201,7 +211,7 @@ randum_thread (void)
     return sprng ();
 #else
 
-    double value;
+    MYREAL value;
     //longer newseed;
 #ifdef PTHREADS
 
@@ -232,7 +242,7 @@ randum_thread (void)
 #endif /*SPRNG*/
 }    /* randum */
 #else
-double
+MYREAL
 randum (void)
 /* Non thread-safe random number generator (faster) */
 {
@@ -242,7 +252,7 @@ randum (void)
 #ifdef MERSENNE_TWISTER
     return mt_drand();
 #else
-double value;
+MYREAL value;
 //  longer newseed;
 newseed[0] = 1549 * seed[0];
 newseed[1] = newseed[0] / 2048;
