@@ -12,16 +12,20 @@
  Peter Beerli 1999, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
   This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: reporter.c,v 1.20 2003/11/04 16:12:09 beerli Exp $
+$Id: reporter.c,v 1.24 2004/08/28 19:59:34 beerli Exp $
  
 -------------------------------------------------------*/
+/* \file reporter.c 
+Routines that report progress and also calculates Gelman-Rubin convergence statistic
+*/
 
 #include "migration.h"
 #include "mcmc.h"
@@ -32,33 +36,34 @@ $Id: reporter.c,v 1.20 2003/11/04 16:12:09 beerli Exp $
 #include "broyden.h"
 #include "combroyden.h"
 #include "options.h"
+#include "sighandler.h"
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
 
 
-void both_chain_means (double *mc, double *lc, double *tc, long len,
+void both_chain_means (MYREAL *mc, MYREAL *lc, MYREAL *tc, long len,
                        long lastn, long n);
-void calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc,
+void calc_gelmanb (MYREAL *gelmanb, MYREAL *mc, MYREAL *tc, MYREAL *lc,
                    long len, long lastn, long n);
-void calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
+void calc_gelmanw (MYREAL *gelmanw, world_fmt * world, MYREAL *mc, MYREAL *tc,
                    long len, long lastn, long n);
-void calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb,
+void calc_gelmanr (MYREAL *gelmanr, MYREAL *gelmanw, MYREAL *gelmanb,
                    long len, long lastn, long n);
-void calc_average_biggest_gelmanr (double *gelmanr, long len, double *meanR,
-                                   double *bigR);
-void print_gelmanr (double average, double biggest);
-double calc_s (long tthis, double *tc, world_fmt * world);
-void chain_means (double *thischainmeansm, world_fmt * world);
+void calc_average_biggest_gelmanr (MYREAL *gelmanr, long len, MYREAL *meanR,
+                                   MYREAL *bigR);
+void print_gelmanr (MYREAL average, MYREAL biggest);
+MYREAL calc_s (long tthis, MYREAL *tc, world_fmt * world);
+void chain_means (MYREAL *thischainmeansm, world_fmt * world);
 
 
 //public function
 void
 convergence_check (world_fmt * world, boolean progress)
 {
-    static double *lastchainmeans, *chainmeans, *thischainmeans;
-    static double *gelmanw, *gelmanb, *gelmanr;
+    static MYREAL *lastchainmeans, *chainmeans, *thischainmeans;
+    static MYREAL *gelmanw, *gelmanb, *gelmanr;
 
     static boolean done = FALSE;
     static long len = 0;
@@ -78,15 +83,15 @@ convergence_check (world_fmt * world, boolean progress)
             // len defines the length of arrays that
             // have to hold all km, kt, p, and mindex means
             len = world->numpop2 + world->numpop * 3;
-            lastchainmeans = (double *) calloc (len, sizeof (double));
-            thischainmeans = (double *) calloc (len, sizeof (double));
-            chainmeans = (double *) calloc (len, sizeof (double));
-            gelmanw = (double *) calloc (len, sizeof (double));
-            gelmanb = (double *) calloc (len, sizeof (double));
-            gelmanr = (double *) calloc (len, sizeof (double));
+            lastchainmeans = (MYREAL *) mycalloc (len, sizeof (MYREAL));
+            thischainmeans = (MYREAL *) mycalloc (len, sizeof (MYREAL));
+            chainmeans = (MYREAL *) mycalloc (len, sizeof (MYREAL));
+            gelmanw = (MYREAL *) mycalloc (len, sizeof (MYREAL));
+            gelmanb = (MYREAL *) mycalloc (len, sizeof (MYREAL));
+            gelmanr = (MYREAL *) mycalloc (len, sizeof (MYREAL));
         }
         n = world->atl[world->rep][world->locus].T;
-        memset (thischainmeans, 0, sizeof (double) * len);
+        memset (thischainmeans, 0, sizeof (MYREAL) * len);
         if (first)
         {
             first = FALSE;
@@ -105,14 +110,14 @@ convergence_check (world_fmt * world, boolean progress)
             calc_gelmanr (gelmanr, gelmanw, gelmanb, len, lastn, n);
             calc_average_biggest_gelmanr (gelmanr, len, &world->gelmanmeanR,
                                           &world->gelmanmaxR);
-            memcpy (lastchainmeans, thischainmeans, sizeof (double) * len);
+            memcpy (lastchainmeans, thischainmeans, sizeof (MYREAL) * len);
             lastn = n;
         }
     }
 }
 
 void
-both_chain_means (double *mc, double *lc, double *tc, long len, long lastn,
+both_chain_means (MYREAL *mc, MYREAL *lc, MYREAL *tc, long len, long lastn,
                   long n)
 {
     long i;
@@ -125,11 +130,11 @@ both_chain_means (double *mc, double *lc, double *tc, long len, long lastn,
 
 
 void
-calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc, long len,
+calc_gelmanb (MYREAL *gelmanb, MYREAL *mc, MYREAL *tc, MYREAL *lc, long len,
               long lastn, long n)
 {
     long i;
-    double nn = (n + lastn) / 2.;
+    MYREAL nn = (n + lastn) / 2.;
 
     for (i = 0; i < len; i++)
     {
@@ -140,11 +145,11 @@ calc_gelmanb (double *gelmanb, double *mc, double *tc, double *lc, long len,
 }
 
 void
-calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
+calc_gelmanw (MYREAL *gelmanw, world_fmt * world, MYREAL *mc, MYREAL *tc,
               long len, long lastn, long n)
 {
     long i;
-    double s1, s2;
+    MYREAL s1, s2;
 
     for (i = 0; i < len; i++)
     {
@@ -157,11 +162,11 @@ calc_gelmanw (double *gelmanw, world_fmt * world, double *mc, double *tc,
 
 
 void
-calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len,
+calc_gelmanr (MYREAL *gelmanr, MYREAL *gelmanw, MYREAL *gelmanb, long len,
               long lastn, long n)
 {
     long i;
-    double nn = (n + lastn) / 2.;
+    MYREAL nn = (n + lastn) / 2.;
 
     for (i = 0; i < len; i++)
     {
@@ -172,12 +177,12 @@ calc_gelmanr (double *gelmanr, double *gelmanw, double *gelmanb, long len,
 }
 
 void
-calc_average_biggest_gelmanr (double *gelmanr, long len,
-                              double *meanR, double *bigR)
+calc_average_biggest_gelmanr (MYREAL *gelmanr, long len,
+                              MYREAL *meanR, MYREAL *bigR)
 {
     long i;
-    double average = 0;
-    double biggest = 0.;
+    MYREAL average = 0;
+    MYREAL biggest = 0.;
     for (i = 0; i < len; i++)
     {
         if (biggest < gelmanr[i])
@@ -191,28 +196,11 @@ calc_average_biggest_gelmanr (double *gelmanr, long len,
     *bigR = biggest;
 }
 
-void
-print_gelmanr (double average, double biggest)
-{
-    if (!isnan (average) && average < 100000)
-        FPRINTF (stdout, "           Average Gelman's R = %f\n", average);
-    else
-        FPRINTF (stdout, "           Average Gelman's R = -failed-\n");
-    if (!isnan (biggest) && biggest < 10000000)
-        FPRINTF (stdout,
-                 "           Largest Gelman's R = %f [Value < 1.2 show convergence]\n",
-                 biggest);
-    else
-        FPRINTF (stdout,
-                 "           Largest Gelman's R = -failed- [Value < 1.2 show convergence]\n");
-}
-
-
-double
-calc_s (long tthis, double *tc, world_fmt * world)
+MYREAL
+calc_s (long tthis, MYREAL *tc, world_fmt * world)
 {
     long i, j;
-    double s = 0;
+    MYREAL s = 0;
     long rep = world->rep;
     static long startp, startl, startkm;
     static boolean done = FALSE;
@@ -277,7 +265,7 @@ calc_s (long tthis, double *tc, world_fmt * world)
 
 
 void
-chain_means (double *thischainmeans, world_fmt * world)
+chain_means (MYREAL *thischainmeans, world_fmt * world)
 {
     /*static */ long startp, startl, startkm;
 
