@@ -1,5 +1,6 @@
 #ifndef REPORTER_INCLUDE
 #define REPORTER_INCLUDE
+/** \file reporter.h */
 /*------------------------------------------------------
  Maximum likelihood estimation 
  of migration rate  and effectice population size
@@ -11,13 +12,13 @@
  Peter Beerli 1999, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: reporter.h,v 1.7 2003/11/04 16:12:09 beerli Exp $
+$Id: reporter.h,v 1.9 2004/02/03 16:33:16 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
 
 extern void convergence_check (world_fmt * world, boolean progress);
-extern void print_gelmanr (double average, double biggest);
 #endif
