@@ -11,17 +11,18 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: sequence.h,v 1.14 2003/11/04 16:12:10 beerli Exp $
+$Id: sequence.h,v 1.15 2004/02/03 16:33:16 beerli Exp $
 -------------------------------------------------------*/
 
 extern void initratio (option_fmt * options);
-extern void initfreqs (double *freqa, double *freqc, double *freqg,
-                           double *freqt);
+extern void initfreqs (MYREAL *freqa, MYREAL *freqc, MYREAL *freqg,
+                           MYREAL *freqt);
 extern void initcatn (long *categs);
-extern boolean initcategs (long categs, double *rate, double *probcat);
-extern void initprobcat (long categs, double *probsum, double *probcat);
+extern boolean initcategs (long categs, MYREAL *rate, MYREAL *probcat);
+extern void initprobcat (long categs, MYREAL *probsum, MYREAL *probcat);
 extern void initlambda (option_fmt * options);
 
 extern void init_sequences (world_fmt * world, option_fmt * options,
@@ -35,8 +36,8 @@ extern void print_tbl (FILE * outfile, world_fmt * world,
                            option_fmt * options, long locus);
 extern void print_seqfreqs (FILE * outfile, world_fmt * world,
                                 option_fmt * options);
-extern double treelike_seq (world_fmt * world, long locus);
-extern double treelike_snp (world_fmt * world, long locus);
+extern MYREAL treelike_seq (world_fmt * world, long locus);
+extern MYREAL treelike_snp (world_fmt * world, long locus);
 extern void snp_invariants (contribarr invariants, long endsite, long rcategs,
                                 seqmodel_fmt * seq, phenotype x1);
 extern void make_sequences (world_fmt * world, option_fmt * options,
@@ -46,7 +47,7 @@ extern void make_invarsites_unlinked (world_fmt * world, data_fmt * data,
                                           long locus);
 extern void make_snp (world_fmt * world, option_fmt * options,
                           data_fmt * data, long locus);
-extern double treelike_snp_unlinked (world_fmt * world, long locus);
+extern MYREAL treelike_snp_unlinked (world_fmt * world, long locus);
 extern void copy_seq (world_fmt * original, world_fmt * kopie);
 extern void init_sequences_aliases (world_fmt * world, option_fmt * options,
                                         data_fmt * data, long locus);
