@@ -22,22 +22,11 @@
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-   $Id: sighandler.c,v 1.19 2003/05/31 15:10:15 beerli Exp $                  
+   $Id: sighandler.c,v 1.26 2005/01/21 16:48:37 beerli Exp $                  
    ----------------------------------------------------- */
-//#include <stdio.h>
-#ifndef __MWERKS__
-#include <sys/types.h>
-#endif
-#ifdef __NEXT__
-#include <libc.h>
-#endif
-#include <stdarg.h>
-#include <signal.h>
-#include <stdarg.h>
-#include "migration.h"
-#ifdef MPI
+/*! \file sighandler.c */
+#include "sighandler.h"
 #include "migrate_mpi.h"
-#endif
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -94,7 +83,7 @@ signalhandling (long switcher)
 void
 signalhandler (int sig)
 {
-    fputc ('\040', stderr);
+  //fputc ('\040', stderr);
     fflush (NULL);
 
     switch (sig)
@@ -284,7 +273,15 @@ sig_error (char string[], char filename[], long line)
     exit (-11);
 }
 
-char *
+///
+/// replacement for fgets so that migrate can read 
+/// files from different computers with different line-endings
+/// should be able to read a file with
+/// - windows
+/// - mac
+/// - unix
+/// line-endings.
+char 
 myfgets (char *buffer, long bufsize, FILE * infile)
 {
     long count = 0;
@@ -304,7 +301,9 @@ myfgets (char *buffer, long bufsize, FILE * infile)
             ungetc (ch, infile);
     }
     buffer[count - 1] = '\0';
-    return buffer;
+    //debug
+    //printf("MYFGETS> %s\n",buffer);
+    return ch;
 }
 
 #ifdef NEXTAPP
@@ -334,7 +333,7 @@ malloc_error_found (int value)
                  "I tried to reallocate or free space which was never allocated\n");
         break;
     default:
-        FPRINTF (stderr, "huh, what malloc error was this?\n");
+        FPRINTF (stderr, "huh, what mymalloc error was this?\n");
         break;
     }
     errno = ENOMEM;
@@ -345,21 +344,30 @@ malloc_error_found (int value)
 /* save memory routines, this routines are used
    with defines in migration.h, DO NOT move this part somewhere else */
 #ifdef LAMARC_MALLOC
-#undef malloc
-#undef realloc
-#undef calloc
-#undef free
+//#undef mymalloc
+//#undef realloc
+//#undef calloc
+//#undef free
 
 void *
 LAMARC_malloc (size_t size, const char file[], const long line)
 {
     void *x;
     x = malloc (size);
+//#ifdef MEMDEBUG
+//    fprintf(stderr,"malloc: %li \n",(long) size);
+//#endif    
     if (x == NULL)
     {
+#ifdef MPI
+        FPRINTF (stderr,
+                 "%i> ERROR in memory allocation (malloc(%li)) in file %s on line %li\n",
+                 myID, size, file, line);
+#else
         FPRINTF (stderr,
                  "ERROR in memory allocation (malloc(%li)) in file %s on line %li\n",
                  size, file, line);
+#endif
         exit (-1);
     }
     return x;
@@ -372,11 +380,20 @@ LAMARC_calloc (size_t repeats, size_t size,
 {
     void *x;
     x = calloc (repeats, size);
+//#ifdef MEMDEBUG
+//    fprintf(stderr,"calloc: %li x %li = %li\n",(long) repeats, (long) size, (long)(repeats * size));
+//#endif    
     if (x == NULL)
     {
+#ifdef MPI
+        FPRINTF (stderr,
+                 "%i> ERROR in memory allocation (calloc(%li,%li)) in file %s on line %li\n",
+                 myID, repeats, size, file, line);
+#else
         FPRINTF (stderr,
                  "ERROR in memory allocation (calloc(%li,%li)) in file %s on line %li\n",
                  repeats, size, file, line);
+#endif
         exit (-1);
     }
     return x;
@@ -388,11 +405,21 @@ LAMARC_realloc (void *ptr, size_t size, const char file[],
 {
     void *x;
     x = realloc (ptr, size);
+//#ifdef MEMDEBUG
+//    fprintf(stderr,"realloc:  %li\n",(long) size);
+//#endif    
+    
     if (x == NULL)
     {
+#ifdef MPI
         FPRINTF (stderr,
-                 "ERROR in memory allocation (realloc(ptr,%li)) in file %s on lin %li\n",
-                 size, file, line);
+                 "%i> ERROR in memory allocation (realloc(ptr,%li)) in file %s on line %li\n",
+                 myID, size, file, line);
+#else
+        FPRINTF (stderr,
+                 "ERROR in memory allocation (realloc(ptr,%li)) in file %s on line %li\n",
+                  size, file, line);
+#endif
         exit (-1);
     }
     return x;
