@@ -6,15 +6,18 @@
 /*                              */
 /* P. Beerli                        */
 /*
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
  This software is distributed free of charge for non-commercial use
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: sort.c,v 1.10 2002/06/20 06:25:38 beerli Exp $ */
+$Id: sort.c,v 1.14 2004/08/28 19:59:35 beerli Exp $ */
 /* ----------------------------------------------------- */
+/*! \file sort.c */
+
 
 /* include files */
 #include "migration.h"
@@ -59,13 +62,13 @@ stringcmp (const void *v1, const void *v2)
 int
 numcmp (const void *v1, const void *v2)
 {
-    if (*(double *) v1 < *(double *) v2)
+    if (*(MYREAL *) v1 < *(MYREAL *) v2)
     {
         return -1;
     }
     else
     {
-        if (*(double *) v1 > *(double *) v2)
+        if (*(MYREAL *) v1 > *(MYREAL *) v2)
         {
             return 1;
         }
@@ -73,6 +76,94 @@ numcmp (const void *v1, const void *v2)
             return 0;
     }
 }
+
+int
+paircmp (const void *v1, const void *v2)
+{
+    if (((MYREAL *) v1)[1] < ((MYREAL *) v2)[1])
+    {
+        return -1;
+    }
+    else
+    {
+        if (((MYREAL *) v1)[1] > ((MYREAL *) v2)[1])
+        {
+            return 1;
+        }
+        else
+            return 0;
+    }
+}
+
+int
+paircmp_first (const void *v1, const void *v2)
+{
+    if (((MYREAL *) v1)[0] < ((MYREAL *) v2)[0])
+    {
+        return -1;
+    }
+    else
+    {
+        if (((MYREAL *) v1)[0] > ((MYREAL *) v2)[0])
+        {
+            return 1;
+        }
+        else
+            return 0;
+    }
+}
+
+
+
+void paired_qsort2(pair *x, long xelem)
+{
+    long oldli;
+    long li;
+    long elements;
+    
+    // sort using the second element
+    qsort(x,xelem,sizeof(pair),paircmp);
+    
+    // with elements of the same histogram size sort by their first element
+    oldli = 0;
+    elements = 1;
+    for(li=1; li < xelem; li++)
+    {
+        if(x[li][1] == x[li-1][1])
+        {
+            elements++;
+            continue;
+        }
+        else
+        {
+            if(elements > 1)
+                qsort(x+oldli,elements,sizeof(pair),paircmp_first);
+            elements=1;
+            oldli = li;
+        }
+    }
+}
+
+
+int
+floatcmp (const void *v1, const void *v2)
+{
+    if (*(float *) v1 < *(float *) v2)
+    {
+        return -1;
+    }
+    else
+    {
+        if (*(float *) v1 > *(float *) v2)
+        {
+            return 1;
+        }
+        else
+            return 0;
+    }
+}
+
+
 int
 longcmp (const void *v1, const void *v2)
 {
@@ -168,8 +259,8 @@ migr_time_cmp (const void *x, const void *y)
 int
 searchagecmp (const void *x, const void *y)
 {
-    double xx = (double) *((double *) x);
-    double age = (double) ((vtlist *) y)->age;
+    MYREAL xx = (MYREAL) *((MYREAL *) x);
+    MYREAL age = (MYREAL) ((vtlist *) y)->age;
 
     if (xx < age)
     {
