@@ -11,60 +11,61 @@
  Peter Beerli 1996, Seattle
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003-2004 Peter Beerli, Tallahassee FL
  
-$Id: tools.h,v 1.26 2003/11/04 16:12:10 beerli Exp $
+$Id: tools.h,v 1.29 2004/04/19 15:27:14 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
 
-extern double lengthof (node * p);
+extern MYREAL lengthof (node * p);
 extern node *crawlback (const node * theNode);
 extern node *crawl (node * theNode);
 extern node *showtop (node * theNode);
-extern void adjust_time (node * theNode, double tyme);
+extern void adjust_time (node * theNode, MYREAL tyme);
 extern void insert_migr_node (world_fmt * world, node * up, node * down,
                                   migr_table_fmt * migr_table,
                                   long *migr_table_counter);
 extern void children (node * mother, node ** brother, node ** sister);
 /* math tools */
-extern double incompletegamma (double x, double alpha);
-extern double polygamma (long n, double z);
-extern void invert_matrix (double **a, long nsize);
-extern boolean nrcheck (double **m, double **tm, double *v, long nrows,
-                            double *r1, double *r2, boolean do_newton);
-extern double rannor (double mean, double sd);
-extern double find_chi (long df, double prob);
-extern double probchiboundary (double chi, long zeros, long all);
-extern double chiboundary (long zeros, long nonzeros, double alpha);
-extern double probchi (long df, double chi);
-extern double chisquare (long df, double alpha);
-extern void gamma_rates (double *rate, double *probcat, long categs,
+extern MYREAL incompletegamma (MYREAL x, MYREAL alpha);
+extern MYREAL polygamma (long n, MYREAL z);
+extern void invert_matrix (MYREAL **a, long nsize);
+extern boolean nrcheck (MYREAL **m, MYREAL **tm, MYREAL *v, long nrows,
+                            MYREAL *r1, MYREAL *r2, boolean do_newton);
+extern MYREAL rannor (MYREAL mean, MYREAL sd);
+extern MYREAL find_chi (long df, MYREAL prob);
+extern MYREAL probchiboundary (MYREAL chi, long zeros, long all);
+extern MYREAL chiboundary (long zeros, long nonzeros, MYREAL alpha);
+extern MYREAL probchi (long df, MYREAL chi);
+extern MYREAL chisquare (long df, MYREAL alpha);
+extern void gamma_rates (MYREAL *rate, MYREAL *probcat, long categs,
                              char *input);
 #ifndef HAVE_LGAMMA
-extern double mylgamma (double z);
+extern MYREAL mylgamma (MYREAL z);
 #endif
-extern double sum (double *vector, long n);
-extern double logfac (long n);
-extern float fast_log(float val);
+extern MYREAL calc_sum (MYREAL *vector, long n);
+extern MYREAL logfac (long n);
+extern float fast_log(float vval);
 
 extern char lowercase (char c);
 extern char uppercase (char c);
+extern void upper(char *from, char **to);
 
 /* vector initialization and calc*/
-extern void doublevec1d (double **v, long size);
-extern void doublevec2d (double ***v, long size1, long size2);
-extern void free_doublevec2d (double **v);
-extern void setdoublevec1d (double **v, double *w, long size);
-extern void add_vector (double *result, double *v, long size);
+extern void doublevec1d (MYREAL **v, long size);
+extern void doublevec2d (MYREAL ***v, long size1, long size2);
+extern void free_doublevec2d (MYREAL **v);
+extern void setdoublevec1d (MYREAL **v, MYREAL *w, long size);
+extern void add_vector (MYREAL *result, MYREAL *v, long size);
 
 /*filemanipulation */
 extern void init_files (world_fmt * world, data_fmt * data,
                             option_fmt * options);
 extern void exit_files (world_fmt * world, data_fmt * data,
                             option_fmt * options);
-extern void openfile (FILE ** fp, char *filename, char *mode,
-                          const char *application, char *perm);
+extern void openfile (FILE ** fp, char *filename, char *mode, char *perm);
 extern long read_savesum (world_fmt * world, option_fmt * options,
                               data_fmt * data);
 extern void write_savesum (world_fmt * world);
@@ -77,7 +78,7 @@ extern void unread_word(FILE *infile, char *word);
 /* time reporting */
 extern void get_time (char *nowstr, char ts[]);
 /* printing aid */
-extern void print_llike (double llike, char *strllike);
+extern void print_llike (MYREAL llike, char *strllike);
 /* searching and finding*/
 extern boolean find (long i, long *list, long listlen);
 /* conversion between the different parameter schemes*/
