@@ -14,13 +14,22 @@ Copyright 2002 Peter Beerli and Joseph Felsenstein
         and is copyrighted. Of course, we do not guarantee that the software
         works and are not responsible for any damage you may cause or have.
  
-$Id: uep.c,v 1.33 2003/11/04 16:12:10 beerli Exp $
+$Id: uep.c,v 1.37 2004/08/28 19:59:36 beerli Exp $
+*/
+/* \file uep.c 
+Unique event polymorphism manipulation
 */
 #ifdef UEP
 #include "migration.h"
 #include "tools.h"
 #include "random.h"
 #include "uep.h"
+#include "sighandler.h"
+#include "migrate_mpi.h"
+
+#ifdef DMALLOC_DEBUG
+#include <dmalloc.h>
+#endif
 
 #define AVERAGE    0  /* used for time of uep */
 #define POPULATION 1  /* used for time of uep given that it raised in */
@@ -29,11 +38,11 @@ $Id: uep.c,v 1.33 2003/11/04 16:12:10 beerli Exp $
 node *first_uep (node * p, node * nop, long uepsites);
 boolean uep_check (node * root, int *uep, long uepsites);
 node *first_uep2 (node * p, node * root, long uepsites);
-double sum_ueplike (proposal_fmt * proposal);
+MYREAL sum_ueplike (proposal_fmt * proposal);
 
 void print_uep (FILE * out, world_fmt * world, long copies);
-double *calc_uep_popprob (world_fmt * world, long copies, long rootstate);
-double *calc_uep_timeprob (world_fmt * world, long copies, int type,
+MYREAL *calc_uep_popprob (world_fmt * world, long copies, long rootstate);
+MYREAL *calc_uep_timeprob (world_fmt * world, long copies, int type,
                            long rootstate);
 long normalize_uep (world_fmt * world);
 void analyze_uep (world_fmt * world);
@@ -53,7 +62,7 @@ setup_uep (world_fmt * world, option_fmt * options)
                        &(world->ueptimestore), world->options->lsteps,
                        world->numpop, world->loci);
         world->oldrootuep =
-            (long *) calloc (world->data->uepsites, sizeof (long));
+            (long *) mycalloc (world->data->uepsites, sizeof (long));
 
         alloc_uepanc (&(world->uepanc), world->data->uepsites, world->numpop,
                       world->loci);
@@ -62,16 +71,16 @@ setup_uep (world_fmt * world, option_fmt * options)
 
 void
 pseudonu_twostate (proposal_fmt * proposal,
-                   ueparray_fmt * xx1, double *lx1, double v1,
-                   ueparray_fmt * xx2, double *lx2, double v2)
+                   ueparray_fmt * xx1, MYREAL *lx1, MYREAL v1,
+                   ueparray_fmt * xx2, MYREAL *lx2, MYREAL v2)
 {
     //  static long count=0;
     long j;
-    double pija11, pija12, pija21, pija22;
-    //  double x3m = -DBL_MAX;
-    double r = proposal->world->options->uepmu;
-    double s = proposal->world->options->uepnu;
-    double rs = r + s;
+    MYREAL pija11, pija12, pija21, pija22;
+    //  MYREAL x3m = -DBL_MAX;
+    MYREAL r = proposal->world->options->uepmu;
+    MYREAL s = proposal->world->options->uepnu;
+    MYREAL rs = r + s;
     pair *x1;
     pair *x2;
     v1 = EXP (-v1 * rs);
@@ -111,14 +120,14 @@ twostate_nuview (node * mother, world_fmt * world, const long locus)
     //  static long count=0;
     long j;
     node *d1 = NULL, *d2 = NULL;
-    double v1, v2;
-    double pija11, pija12, pija21, pija22;
+    MYREAL v1, v2;
+    MYREAL pija11, pija12, pija21, pija22;
     pair *xx1, *xx2;
-    //  double x3m = -DBL_MAX;
+    //  MYREAL x3m = -DBL_MAX;
     pair *xx3;
-    double r = world->options->uepmu;
-    double s = world->options->uepnu;
-    double rs = r + s;
+    MYREAL r = world->options->uepmu;
+    MYREAL s = world->options->uepnu;
+    MYREAL rs = r + s;
     children (mother, &d1, &d2);
     v1 = EXP (-d1->v * rs);
     v2 = EXP (-d2->v * rs);
@@ -156,38 +165,38 @@ void
 alloc_uepanc (long ***uepanc, long size, long numpop, long loci)
 {
     long locus;
-    (*uepanc) = (long **) calloc (loci, sizeof (long *));
+    (*uepanc) = (long **) mycalloc (loci, sizeof (long *));
     for (locus = 0; locus < loci; locus++)
-        (*uepanc)[locus] = (long *) calloc (2 * size * numpop, sizeof (long));
+        (*uepanc)[locus] = (long *) mycalloc (2 * size * numpop, sizeof (long));
 }
 
 void
-alloc_ueplike (double ***ueplike, long size,
-               double *****ueplikestore, long storesize,
+alloc_ueplike (MYREAL ***ueplike, long size,
+               MYREAL *****ueplikestore, long storesize,
                long ****ueprootstore, long numpop, long loci)
 {
     long i, j, locus;
-    (*ueplike) = (double **) calloc (size, sizeof (double *));
-    (*ueplike)[0] = (double *) calloc (size * numpop, sizeof (double));
+    (*ueplike) = (MYREAL **) mycalloc (size, sizeof (MYREAL *));
+    (*ueplike)[0] = (MYREAL *) mycalloc (size * numpop, sizeof (MYREAL));
     for (i = 1; i < size; ++i)
         (*ueplike)[i] = (*ueplike)[0] + i * numpop;
 
-    (*ueprootstore) = (long ***) calloc (loci, sizeof (long **));
-    (*ueplikestore) = (double ****) calloc (loci, sizeof (double ***));
+    (*ueprootstore) = (long ***) mycalloc (loci, sizeof (long **));
+    (*ueplikestore) = (MYREAL ****) mycalloc (loci, sizeof (MYREAL ***));
 
     for (locus = 0; locus < loci; locus++)
     {
-        (*ueprootstore)[locus] = (long **) calloc (storesize, sizeof (long *));
+        (*ueprootstore)[locus] = (long **) mycalloc (storesize, sizeof (long *));
         (*ueplikestore)[locus] =
-            (double ***) calloc (storesize, sizeof (double **));
+            (MYREAL ***) mycalloc (storesize, sizeof (MYREAL **));
         for (i = 0; i < storesize; ++i)
         {
-            (*ueprootstore)[locus][i] = (long *) calloc (size, sizeof (long));
+            (*ueprootstore)[locus][i] = (long *) mycalloc (size, sizeof (long));
             (*ueplikestore)[locus][i] =
-                (double **) calloc (size, sizeof (double *));
+                (MYREAL **) mycalloc (size, sizeof (MYREAL *));
             for (j = 0; j < size; ++j)
                 (*ueplikestore)[locus][i][j] =
-                    (double *) calloc (numpop, sizeof (double));
+                    (MYREAL *) mycalloc (numpop, sizeof (MYREAL));
         }
     }
 }
@@ -198,21 +207,21 @@ alloc_ueptime (ueptime_fmt ** ueptime, long size,
                long storesize, long numpop, long loci)
 {
     long i, j, locus;
-    (*ueptime) = (ueptime_fmt *) calloc (size, sizeof (ueptime_fmt));
+    (*ueptime) = (ueptime_fmt *) mycalloc (size, sizeof (ueptime_fmt));
     for (j = 0; j < size; ++j)
     {
         (*ueptime)[j].size = 3;
-        (*ueptime)[j].populations = (long *) calloc (3, sizeof (long));
-        (*ueptime)[j].ueptime = (double *) calloc (3, sizeof (double));
+        (*ueptime)[j].populations = (long *) mycalloc (3, sizeof (long));
+        (*ueptime)[j].ueptime = (MYREAL *) mycalloc (3, sizeof (MYREAL));
     }
-    (*ueptimestore) = (ueptime_fmt ***) calloc (loci, sizeof (ueptime_fmt **));
+    (*ueptimestore) = (ueptime_fmt ***) mycalloc (loci, sizeof (ueptime_fmt **));
     for (locus = 0; locus < loci; locus++)
     {
-        (*ueptimestore)[locus] = (ueptime_fmt **) calloc (storesize,
+        (*ueptimestore)[locus] = (ueptime_fmt **) mycalloc (storesize,
                                  sizeof (ueptime_fmt
                                          *));
         (*ueptimestore)[locus][0] =
-            (ueptime_fmt *) calloc (size * storesize, sizeof (ueptime_fmt));
+            (ueptime_fmt *) mycalloc (size * storesize, sizeof (ueptime_fmt));
         for (i = 1; i < storesize; ++i)
         {
             (*ueptimestore)[locus][i] = (*ueptimestore)[locus][0] + i * size;
@@ -223,9 +232,9 @@ alloc_ueptime (ueptime_fmt ** ueptime, long size,
             {
                 (*ueptimestore)[locus][i][j].size = 3;
                 (*ueptimestore)[locus][i][j].populations =
-                    (long *) calloc (3, sizeof (long));
+                    (long *) mycalloc (3, sizeof (long));
                 (*ueptimestore)[locus][i][j].ueptime =
-                    (double *) calloc (3, sizeof (double));
+                    (MYREAL *) mycalloc (3, sizeof (MYREAL));
             }
         }
     }
@@ -240,19 +249,19 @@ allocate_uep (node * p, world_fmt * world, char datatype, boolean withtips)
             allocate_uep (crawlback (p->next), world, datatype, withtips);
         if (p->next->next->back != NULL)
             allocate_uep (crawlback (p->next->next), world, datatype, withtips);
-        //      p->ux.s = (pair *) calloc (world->data->uepsites, sizeof (pair));
+        //      p->ux.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
     }
     else
     {
         if (withtips)
         {
-            //   p->ux.s = (pair *) calloc (world->data->uepsites, sizeof (pair));
+            //   p->ux.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
         }
     }
 }
 
 void
-constrain_distance_uep (int **uep, long uepsites, double **distm, long tips)
+constrain_distance_uep (int **uep, long uepsites, MYREAL **distm, long tips)
 {
     long i, j;
     //  long ueppatt = howmany_uep_pattern(uep,uepsites, tips);
@@ -330,13 +339,13 @@ make_uep_values (world_fmt * world, data_fmt * data, long locus)
     }
 }
 
-double
-pseudo_tl_uep (ueparray_fmt * xx1, ueparray_fmt * xx2, double v1, double v2,
+MYREAL
+pseudo_tl_uep (ueparray_fmt * xx1, ueparray_fmt * xx2, MYREAL v1, MYREAL v2,
                proposal_fmt * proposal, world_fmt * world)
 {
-    double summ;
+    MYREAL summ;
     long i;
-    double tterm;
+    MYREAL tterm;
     pair *x1;
     summ = 0.0;
     for (i = 0; i < world->data->uepsites; i++)
@@ -350,13 +359,13 @@ pseudo_tl_uep (ueparray_fmt * xx1, ueparray_fmt * xx2, double v1, double v2,
 }
 
 
-double
+MYREAL
 treelike_uep (world_fmt * world, long locus)
 {
-    double tterm;
-    double summ;
+    MYREAL tterm;
+    MYREAL summ;
     long i;
-    //  double scale;
+    //  MYREAL scale;
     node *p;
     pair *x1;
     p = crawlback (world->root->next);
@@ -509,13 +518,13 @@ store_uep (world_fmt * world)
             for (i = 0; i < world->data->uepsites; ++i)
             {
                 memcpy (world->ueplikestore[world->locus][world->G][i],
-                        world->ueplike[i], sizeof (double) * world->numpop);
+                        world->ueplike[i], sizeof (MYREAL) * world->numpop);
                 memcpy (world->ueptimestore[world->locus][world->G][i].
                         populations, world->ueptime[i].populations,
                         sizeof (long) * world->ueptime[i].size);
                 memcpy (world->ueptimestore[world->locus][world->G][i].ueptime,
                         world->ueptime[i].ueptime,
-                        sizeof (double) * world->ueptime[i].size);
+                        sizeof (MYREAL) * world->ueptime[i].size);
                 world->ueptimestore[world->locus][world->G][i].size =
                     world->ueptime[i].size;
             }
@@ -526,7 +535,7 @@ store_uep (world_fmt * world)
 void
 print_uep (FILE * out, world_fmt * world, long copies)
 {
-    double *prob, *tyme, *timep;
+    MYREAL *prob, *tyme, *timep;
     long pop;
     FPRINTF (out, "\n\n\nUnique event polymorphism\n");
     FPRINTF (out, "=========================\n\n");
@@ -583,12 +592,12 @@ print_uep (FILE * out, world_fmt * world, long copies)
     free (timep);
 }
 
-double *
+MYREAL *
 calc_uep_popprob (world_fmt * world, long copies, long rootstate)
 {
     long tree, pop, j, locus;
-    double *result;
-    result = (double *) calloc (world->numpop, sizeof (double));
+    MYREAL *result;
+    result = (MYREAL *) mycalloc (world->numpop, sizeof (MYREAL));
     for (locus = 0; locus < world->loci; locus++)
     {
         for (tree = 0; tree < world->atl[world->repstop - 1][locus].T - 1;
@@ -611,11 +620,11 @@ calc_uep_popprob (world_fmt * world, long copies, long rootstate)
     return result;
 }
 
-double
+MYREAL
 ueptime_report (ueptime_fmt * ueptime, int type, long pop)
 {
     long i;
-    double summ = 0.0;
+    MYREAL summ = 0.0;
     long elem = 0;
 
     switch (type)
@@ -634,7 +643,7 @@ ueptime_report (ueptime_fmt * ueptime, int type, long pop)
             }
         }
         if (elem > 0)
-            return summ / (double) elem;
+            return summ / (MYREAL) elem;
         else
             return 0.0;
         break;
@@ -642,12 +651,12 @@ ueptime_report (ueptime_fmt * ueptime, int type, long pop)
     return 0.0;
 }
 
-double *
+MYREAL *
 calc_uep_timeprob (world_fmt * world, long copies, int type, long rootstate)
 {
     long pop, tree, j, locus;
-    double *result;
-    result = (double *) calloc (world->numpop, sizeof (double));
+    MYREAL *result;
+    result = (MYREAL *) mycalloc (world->numpop, sizeof (MYREAL));
     for (locus = 0; locus < world->loci; locus++)
     {
         for (tree = 0; tree < world->atl[world->repstop - 1][locus].T - 1;
@@ -676,7 +685,7 @@ long
 normalize_uep (world_fmt * world)
 {
     long tree, pop, j, locus;
-    double summ;
+    MYREAL summ;
     long copies = 0;
     for (locus = 0; locus < world->loci; locus++)
     {
@@ -694,7 +703,7 @@ normalize_uep (world_fmt * world)
             {
                 for (j = 0; j < world->data->uepsites; ++j)
                     world->ueplikestore[locus][tree][j][pop] *=
-                        ((double) world->atl[world->repstop - 1][locus].tl[tree].
+                        ((MYREAL) world->atl[world->repstop - 1][locus].tl[tree].
                          copies) / summ;
             }
         }
@@ -717,7 +726,7 @@ show_uep_store (world_fmt * world)
     long i, j, z, steps, locus;
     FILE *file;
     char filename[20] = "uepout";
-    openfile (&file, filename, "w+", appl, NULL);
+    openfile (&file, filename, "w+",  NULL);
     for (locus = 0; locus < world->loci; locus++)
     {
         for (steps = 0; steps < world->G; ++steps)
@@ -761,10 +770,10 @@ print_ancestor (FILE * out, world_fmt * world, long copies)
 {
     char uepsite[10];
     long us, pop, half;
-    double subtotal = 0.0;
-    double total = 0;
+    MYREAL subtotal = 0.0;
+    MYREAL total = 0;
     long locus;
-    double ancsum = 0.;
+    MYREAL ancsum = 0.;
     for (locus = 0; locus < world->loci; locus++)
     {
         for (us = 0; us < 2 * world->numpop * world->data->uepsites; us++)
@@ -814,7 +823,7 @@ print_ancestor (FILE * out, world_fmt * world, long copies)
 }
 
 void
-calc_ueplike (node * p, world_fmt * world, double **ueplike)
+calc_ueplike (node * p, world_fmt * world, MYREAL **ueplike)
 {
     boolean done;
     int temp;
@@ -846,7 +855,7 @@ calc_ueplike (node * p, world_fmt * world, double **ueplike)
                     }
                     else
                         memcpy (ueplike[i], ueplike[save_i],
-                                sizeof (double) * world->numpop);
+                                sizeof (MYREAL) * world->numpop);
                     done = TRUE;
                 }
             }
@@ -871,10 +880,10 @@ fill_ueptime (ueptime_fmt * ueptime, node * p, node * last)
     count += 2;
     if (count > ueptime->size)
     {
-        ueptime->populations = (long *) realloc (ueptime->populations,
+        ueptime->populations = (long *) myrealloc (ueptime->populations,
                                sizeof (long) * count);
-        ueptime->ueptime = (double *) realloc (ueptime->ueptime,
-                                               sizeof (double) * count);
+        ueptime->ueptime = (MYREAL *) myrealloc (ueptime->ueptime,
+                                               sizeof (MYREAL) * count);
         ueptime->size = count;
     }
     while ((nod = showtop (last->back)) != p)
@@ -891,13 +900,13 @@ fill_ueptime (ueptime_fmt * ueptime, node * p, node * last)
 
 void
 collect_ueplike (node * p, node * d1, node * d2, long uepsite,
-                 world_fmt * world, double *ueplike)
+                 world_fmt * world, MYREAL *ueplike)
 {
     node *nod, *last;
     //  long pop;
-    //  double interval = world->root->next->back->tyme - p->tyme;
+    //  MYREAL interval = world->root->next->back->tyme - p->tyme;
     //  printf("collect_ueplike: %f %f %f\n",world->root->next->back->tyme,p->tyme,interval);
-    memset (ueplike, 0, sizeof (double) * world->numpop);
+    memset (ueplike, 0, sizeof (MYREAL) * world->numpop);
     //  if (d1->uep[uepsite] == 1)
     if (d1->uep[uepsite] != world->root->next->back->uep[uepsite])
         last = d1;
@@ -954,12 +963,12 @@ first_uep2 (node * p, node * root, long uepsites)
     return p;
 }
 
-double
+MYREAL
 sum_ueplike (proposal_fmt * proposal)
 {
     long u, i;
-    double like = 1.;
-    double poplike;
+    MYREAL like = 1.;
+    MYREAL poplike;
 
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
@@ -973,7 +982,6 @@ sum_ueplike (proposal_fmt * proposal)
     }
     if (proposal->world->options->ueprate > 0.0)
     {
-        //      printf("%s>c ptreelen=%f like=%g\n",proposal->world->name, proposal->treelen, like);
         return log (like) -
                (proposal->world->options->ueprate * proposal->treelen);
     }
@@ -982,15 +990,15 @@ sum_ueplike (proposal_fmt * proposal)
 }
 
 
-double
-adjust_uep_base (proposal_fmt * proposal, double interval, double oldinterval)
+MYREAL
+adjust_uep_base (proposal_fmt * proposal, MYREAL interval, MYREAL oldinterval)
 {
     long u, pop;
-    // double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    // MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         memcpy (proposal->ueplike[u], proposal->world->ueplike[u],
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         //                  printf("adjust_base   ");
         for (pop = 0; pop < proposal->world->numpop; ++pop)
         {
@@ -1004,21 +1012,21 @@ adjust_uep_base (proposal_fmt * proposal, double interval, double oldinterval)
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_target (node * first, node * firstb, proposal_fmt * proposal,
-                   double interval, double oldinterval)
+                   MYREAL interval, MYREAL oldinterval)
 {
     long u, pop;
     node *last, *p;
-    double lasttime, endtime;
-    //double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    MYREAL lasttime, endtime;
+    //MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     boolean o =
         uep_check (proposal->world->root->next->back, proposal->origin->uep,
                    proposal->world->data->uepsites);
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         last = first;
         p = showtop (first->back);
         if (o)
@@ -1056,23 +1064,23 @@ adjust_uep_target (node * first, node * firstb, proposal_fmt * proposal,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_origin (node * first, node * firstb, proposal_fmt * proposal,
-                   double interval, double oldinterval)
+                   MYREAL interval, MYREAL oldinterval)
 {
     long u, i, pop;
-    double lasttime;
+    MYREAL lasttime;
     //  node *last, *p;
     migr_table_fmt *mt = proposal->migr_table;
     long mtc = proposal->migr_table_counter;
-    //double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    //MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     //  printf("@@@@@@@ adjust branch below origin @@@@@@@@@\n");
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         // memcpy(proposal->ueplike[u], proposal->world->ueplike[u],
-        //             sizeof(double)*proposal->world->numpop);
+        //             sizeof(MYREAL)*proposal->world->numpop);
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         lasttime = first->tyme;
         for (i = 0; i < mtc; ++i)
         {
@@ -1094,22 +1102,22 @@ adjust_uep_origin (node * first, node * firstb, proposal_fmt * proposal,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_oback (node * first, node * firstb, proposal_fmt * proposal,
-                  double interval, double oldinterval)
+                  MYREAL interval, MYREAL oldinterval)
 {
     long u, pop;
-    double lasttime = 0.;
+    MYREAL lasttime = 0.;
     node *last, *p;
-    //double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    //MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     if (first->tyme < proposal->time) // target == osister
         first = proposal->oback;
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         //memcpy(proposal->ueplike[u], proposal->world->ueplike[u],
-        //             sizeof(double)*proposal->world->numpop);
+        //             sizeof(MYREAL)*proposal->world->numpop);
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         last = first;
         p = showtop (first->back);
         if (last != proposal->oback)
@@ -1139,18 +1147,18 @@ adjust_uep_oback (node * first, node * firstb, proposal_fmt * proposal,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_osister (node * first, node * firstb, proposal_fmt * proposal,
-                    double interval, double oldinterval)
+                    MYREAL interval, MYREAL oldinterval)
 {
     long u, pop;
-    double lasttime = 0.;
+    MYREAL lasttime = 0.;
     node *last, *p;
-    //double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    //MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         last = first;
         p = showtop (first->back);
         lasttime = last->tyme;
@@ -1175,19 +1183,19 @@ adjust_uep_osister (node * first, node * firstb, proposal_fmt * proposal,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_osistertarget (node * first, node * firstb,
-                          proposal_fmt * proposal, double interval,
-                          double oldinterval)
+                          proposal_fmt * proposal, MYREAL interval,
+                          MYREAL oldinterval)
 {
     long u, pop;
-    double lasttime = 0.;
+    MYREAL lasttime = 0.;
     node *last, *p;
-    //double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    //MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         last = first;
         p = showtop (first->back);
         lasttime = last->tyme;
@@ -1215,19 +1223,19 @@ adjust_uep_osistertarget (node * first, node * firstb,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 adjust_uep_osistertarget2 (node * first, node * firstb,
-                           proposal_fmt * proposal, double interval,
-                           double oldinterval)
+                           proposal_fmt * proposal, MYREAL interval,
+                           MYREAL oldinterval)
 {
     long u, pop, actualpop;
-    double lasttime = 0., endtime;
+    MYREAL lasttime = 0., endtime;
     node *last, *p;
-    // double expT = EXP (-(proposal->treelen-proposal->world->treelen));
+    // MYREAL expT = EXP (-(proposal->treelen-proposal->world->treelen));
     for (u = 0; u < proposal->world->data->uepsites; ++u)
     {
         memset (proposal->ueplike[u], 0,
-                sizeof (double) * proposal->world->numpop);
+                sizeof (MYREAL) * proposal->world->numpop);
         last = first;
         p = showtop (first->back);
         actualpop = p->actualpop;
@@ -1254,10 +1262,10 @@ adjust_uep_osistertarget2 (node * first, node * firstb,
     return sum_ueplike (proposal);
 }
 
-double
+MYREAL
 pseudo_ueplikelihood (world_fmt * world, proposal_fmt * proposal)
 {
-    double interval = 0., oldinterval = 0.; //  node *first, *firstb;
+    MYREAL interval = 0., oldinterval = 0.; //  node *first, *firstb;
     node *root = proposal->root->next->back;
     long uepsites = proposal->world->data->uepsites;
 
@@ -1378,12 +1386,12 @@ pseudo_ueplikelihood (world_fmt * world, proposal_fmt * proposal)
     return -DBL_MAX;  //never come here
 }
 
-double
+MYREAL
 ueplikelihood (world_fmt * world)
 {
     long u, i;
-    double like = 1.;
-    double poplike = 0;
+    MYREAL like = 1.;
+    MYREAL poplike = 0;
     calc_ueplike (world->root->next->back, world, world->ueplike);
     //  printf("update_likeli ");
     for (u = 0; u < world->data->uepsites; ++u)
@@ -1399,7 +1407,6 @@ ueplikelihood (world_fmt * world)
     }
     if (world->options->ueprate > 0.0)
     {
-        //      printf("%s> wtreelen=%f like=%g\n",world->name, world->treelen,like);
         return log (like) - (world->options->ueprate * world->treelen);
     }
     else
