@@ -1,5 +1,7 @@
 #ifndef WORLD_INCLUDE
 #define WORLD_INCLUDE
+/** \file world.h
+*/
 /*------------------------------------------------------
  Maximum likelihood estimation 
  of migration rate  and effectice population size
@@ -14,17 +16,21 @@
  prints results,
  and finally helps to destroy itself.
                                                                                                                
- Peter Beerli 1996, Seattle
+ Peter Beerli, started 1996
  beerli@csit.fsu.edu
  
-Copyright 2001 Peter Beerli and Joseph Felsenstein
- 
-$Id: world.h,v 1.26 2003/11/04 16:12:10 beerli Exp $
- 
+Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
+Copyright 2003 Peter Beerli, Tallahassee FL
+
+This software is distributed free of charge for non-commercial use
+and is copyrighted. Of course, we do not guarantee that the software
+works and are not responsible for any damage you may cause or have.
+
+$Id: world.h,v 1.30 2004/10/24 20:53:21 beerli Exp $
 -------------------------------------------------------*/
 
 #include "migration.h"
-extern void create_world (world_fmt ** world, char name[], long loci);
+extern void create_world (world_fmt ** world, long loci);
 extern void init_world (world_fmt * world, data_fmt * data,
                             option_fmt * options);
 extern void increase_timearchive (world_fmt * world, long locus, long sample,
@@ -39,37 +45,35 @@ extern void print_menu_chain (char type, long chain, long steps,
                                   long rep);
 extern void print_menu_coalnodes (FILE * file, world_fmt * world, long G,
                                       long rep);
-//extern void heating_ratio (world_fmt * world);
+extern void print_progress(worldoption_fmt * options, world_fmt * world,
+               long rep, long visited, long accepted);
 extern void burnin_chain (world_fmt * world);
 extern void print_finish (world_fmt * world, long filepos);
 extern void copy_time (world_fmt * world, timelist_fmt * ltl, long from,
                            long to, long np, long rep);
-//void copy_atl (world_fmt * world, long rep, timearchive_fmt * old, timearchive_fmt * new,
-//       long steps, long locus);
-extern void create_loci_plot (world_fmt * world, char ***plane,
-                                  nr_fmt * nr, long loci);
-extern void create_locus_plot (world_fmt * world, char ***plane, nr_fmt * nr,
-                                   long G);
+extern void create_plot (world_fmt * world, char ***plane,
+                                  nr_fmt * nr, long Gloci, boolean multilocus);
 extern void print_simresults (world_fmt * world);
 extern void print_list (world_fmt ** universe, option_fmt * options,
                             data_fmt * data);
 extern void cleanup_world (world_fmt * world, long locus);
 extern void free_universe (world_fmt ** worlds, long numworlds);
-extern void test_locus_like (nr_fmt * nr, double *param0, double *param1,
+extern void test_locus_like (nr_fmt * nr, MYREAL *param0, MYREAL *param1,
                                  long df, long locus, world_fmt * world,
                                  long *maxwhich, long maxnum, boolean withhead,
                                  char *this_string);
-extern void test_loci_like (nr_fmt * nr, double *param0, double *param1,
+extern void test_loci_like (nr_fmt * nr, MYREAL *param0, MYREAL *param1,
                                 long df, long loci, world_fmt * world,
                                 long *maxwhich, long maxnum, boolean withhead,
                                 char *this_string);
 
 
 extern void precalc_world (world_fmt * world);
+extern void reprecalc_world (world_fmt * world, long that);
 extern void klone (world_fmt * original, world_fmt * kopie,
-                       option_fmt * options, data_fmt * data, double temperature);
+                       option_fmt * options, data_fmt * data, MYREAL temperature);
 extern void klone_part (world_fmt * original, world_fmt * kopie,
-                            option_fmt * options, data_fmt * data, double temperature);
+                            option_fmt * options, data_fmt * data, MYREAL temperature);
 extern void clone_polish (world_fmt * original, world_fmt * kopie);
 extern long chance_swap_tree (world_fmt * tthis, world_fmt * that);
 extern void advance_clone_like (world_fmt * world, long accepted, long *j);
@@ -77,12 +81,10 @@ extern void polish_world (world_fmt * world);
 extern void fill_worldoptions (worldoption_fmt * wopt, option_fmt * options);
 extern void fill_worlddata (worlddata_fmt * wdata, data_fmt * data);
 extern void print_alpha_curve (world_fmt * world, timearchive_fmt ** atl, long *gmaxptr);
-extern void print_cov (world_fmt * world, long numpop, long loci, double ***cov);
-
-
+extern void print_cov (world_fmt * world, long numpop, long loci, MYREAL ***cov);
 extern void print_mighist (world_fmt * world);
-//=================================
-#include "reporter.h"
-
+extern void print_gelmanr (MYREAL average, MYREAL biggest);
+extern void prognose_time (char *nowstr, world_fmt * world,
+                           long options_increment, long steps, char *spacer, boolean tobuffer);
 
 #endif /*WORLD_INCLUDE */
